;****************************************************************************
;****************************************************************************
;*
;* c1541memory.n
;*
;****************************************************************************
;****************************************************************************




%define C1541MEMORY_N
%include "c1541memory.i"
%include "c1541link.i"
%include "core.i"





;****************************************************************************
;****************************************************************************
;*
;* PART 1: assign's & define's
;*
;****************************************************************************
;****************************************************************************





%define MODULID			C1541MEMORY_MODULID
%define STATE			C1541MEMORY_STATE
%define RAM			C1541MEMORY_RAM
%define KERNEL_ROM		C1541MEMORY_KERNEL_ROM



%assign STACK_LIMIT_EXCEEDED	C1541MEMORY_STACK_LIMIT_EXCEEDED
%assign WORD_LIMIT_EXCEEDED	C1541MEMORY_WORD_LIMIT_EXCEEDED





;****************************************************************************
;****************************************************************************
;*
;* PART 2: Initialisation & Management Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* c1541memory_initialize ***************************************************
;****************************************************************************
section .text
c1541memory_initialize:
			ret



;****************************************************************************
;* c1541memory_exit *********************************************************
;****************************************************************************
section .text
c1541memory_exit:
			ret



;****************************************************************************
;* c1541memory_instance_initialize ******************************************
;****************************************************************************
section .text
c1541memory_instance_initialize:
			INSTANCE_INITIALIZE  tc1541memory_size



;****************************************************************************
;* c1541memory_instance_exit ************************************************
;****************************************************************************
section .text
c1541memory_instance_exit:
			ret



;****************************************************************************
;* c1541memory_reset_hard ***************************************************
;****************************************************************************
section .text
c1541memory_reset_hard:
			push	dword eax
			push	dword ecx
			push	dword edi
			xor	dword eax, eax
			mov	dword ecx, 00200h
			lea	dword edi, [RAM]
			cld
			rep stosd
			pop	dword edi
			pop	dword ecx
			pop	dword eax
			jmp	c1541memory_reset_soft



;****************************************************************************
;* c1541memory_reset_soft ***************************************************
;****************************************************************************
section .text
c1541memory_reset_soft:
			mov	dword [STATE], 0
			ret



;****************************************************************************
;* c1541memory_load_rom_kernel **********************************************
;****************************************************************************
;* eax=>  position of the ROM
;****************************************************************************
section .text
c1541memory_load_rom_kernel:
			push	dword esi
			lea	dword edi, [KERNEL_ROM]
			mov	dword esi, eax
			mov	dword ecx, 04000h
			cld
			rep movsb
			pop	dword esi
			ret





;****************************************************************************
;****************************************************************************
;*
;* PART 3: Memory Access Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* c1541memory_read *********************************************************
;****************************************************************************
section .text
c1541memory_read:
			and	dword ebx, 0ffffh
			cmp	dword ebx, 0c000h
			jae	.read_kernel_rom
			cmp	dword ebx, 00800h
			jb	.read_ram_1
			cmp	dword ebx, 01000h
			jb	.read_ram_2
			cmp	dword ebx, 02000h
			jae	.read_rubbish
			cmp	dword ebx, 01c00h
			jae	.read_via2
			cmp	dword ebx, 01800h
			jae	.read_via1
.read_rubbish:		mov	byte  al, bh
			ret
.read_ram_1:		lea	dword ebp, [RAM]
			mov	byte  al, [ebp + ebx]
			ret
.read_ram_2:		lea	dword ebp, [RAM - 00800h]
			mov	byte  al, [ebp + ebx]
			ret
.read_via1:		lea	dword edi, [esi + tvia6522__c1541_1]
			call	via6522_read
			ret
.read_via2:		lea	dword edi, [esi + tvia6522__c1541_2]
			call	via6522_read
			ret
.read_kernel_rom:	mov	byte  al, [KERNEL_ROM - 0c000h + ebx]
			ret



;****************************************************************************
;* c1541memory_write ********************************************************
;****************************************************************************
section .text
c1541memory_write:
			and	dword ebx, 0ffffh
			cmp	dword ebx, 00800h
			jb	.write_ram_1
			cmp	dword ebx, 01000h
			jb	.write_ram_2
			cmp	dword ebx, 02000h
			jae	.write_ignore
			cmp	dword ebx, 01c00h
			jae	.write_via2
			cmp	dword ebx, 01800h
			jae	.write_via1
.write_ignore:		ret
.write_ram_1:		lea	dword ebp, [RAM]
			mov	byte  [ebp + ebx], al
			ret
.write_ram_2:		lea	dword ebp, [RAM - 00800h]
			mov	byte  [ebp + ebx], al
			ret
.write_via1:		lea	dword edi, [esi + tvia6522__c1541_1]
			call	via6522_write
			ret
.write_via2:		lea	dword edi, [esi + tvia6522__c1541_2]
			call	via6522_write
			ret



;****************************************************************************
;* read *********************************************************************
;****************************************************************************
section .text
read_opcode_ram:		
			cmp	dword ebx, 00800h
			jb	.read
			jmp	read_opcode_switch
.read:			mov	byte  al, [RAM + ebx]
			ret

read_opcode_mirrored_ram:
			cmp	dword ebx, 00800h
			jae	.check_high_limit
			jmp	read_opcode_switch
.check_high_limit:	cmp	dword ebx, 01000h
			jb	.read
			jmp	read_opcode_switch
.read:			mov	byte  al, [RAM + ebx - 00800h]
			ret

read_opcode_io:
			cmp	dword ebx, 01000h
			jae	.read
			jmp	read_opcode_switch
.read:			cmp	dword ebx, 01800h
			jae	.read_vias
			mov	byte  al, bh
			ret
.read_vias:		lea	dword edi, [esi + tvia6522__c1541_1]
			cmp	dword ebx, 01c00h
			jb	near  via6522_read
			lea	dword edi, [esi + tvia6522__c1541_2]
			cmp	dword ebx, 02000h
			jb	near  via6522_read
			jmp	read_opcode_switch

read_opcode_rubbish:
			cmp	dword ebx, 02000h
			jae	.check_high_limit
			jmp	read_opcode_switch
.check_high_limit:	cmp	dword ebx, 0c000h
			jb	.read
			jmp	read_opcode_switch
.read:			mov	byte  al, bh
			ret

read_opcode_kernel_rom:
			cmp	dword ebx, 0c000h
%ifdef C1541MEMORY_ASSERTION
%ifdef C1541MEMORY_ASSERTION_KERNEL
			jae	.check_high_limit
			jmp	read_opcode_switch
.check_high_limit:	cmp	dword ebx, 010000h
			jb	.read
			mov	dword eax, [MODULID]
			xor	dword ebx, ebx
			mov	dword ecx, __LINE__
			jmp	core_assertion
%else
			jae	.read
			jmp	read_opcode_switch
%endif
%else
			jae	.read
			jmp	read_opcode_switch
%endif
.read:			mov	byte  al, [KERNEL_ROM - 0c000h + ebx]
			ret

read_opcode_switch:
			cmp	dword ebx, 010000h
			jb	.address_ok
			mov	dword ebx, WORD_LIMIT_EXCEEDED
			mov	dword eax, [MODULID]
			mov	dword [STATE], ebx
			xor	dword ebx, ebx
			mov	dword ecx, __LINE__
			jmp	core_assertion
.address_ok:		mov	dword eax, read_opcode_kernel_rom
			cmp	dword ebx, 0c000h
			jae	.write_address
			mov	dword eax, read_opcode_ram
			cmp	dword ebx, 00800h
			jb	.write_address
			mov	dword eax, read_opcode_mirrored_ram
			cmp	dword ebx, 01000h
			jb	.write_address
			mov	dword eax, read_opcode_io
			cmp	dword ebx, 02000h
			jb	.write_address
			mov	dword eax, read_opcode_rubbish
.write_address:		CPU65XX_SET_READ_OPCODE  eax
			jmp	dword eax

.b__dummy:		db	05ah



;****************************************************************************
;* read_stack ***************************************************************
;****************************************************************************
section .text
read_stack:
%ifdef C1541MEMORY_ASSERTION
%ifdef C1541MEMORY_ASSERTION_STACK
			cmp	dword ebx, 00100h
			jae	.check_high_limit
.limit_exceeded:	mov	dword ebx, STACK_LIMIT_EXCEEDED
			mov	dword eax, [MODULID]
			mov	dword [STATE], ebx
			xor	dword ebx, ebx
			mov	dword ecx, __LINE__
			jmp	core_assertion
.check_high_limit:	cmp	dword ebx, 00200h
			jb	.read
			jmp	.limit_exceeded
.read:
%endif
%endif
			mov	byte  al, [RAM + ebx]
			ret



;****************************************************************************
;* write_stack **************************************************************
;****************************************************************************
section .text
write_stack:
%ifdef C1541MEMORY_ASSERTION
%ifdef C1541MEMORY_ASSERTION_STACK
			cmp	dword ebx, 00100h
			jae	.check_high_limit
.limit_exceeded:	mov	dword ebx, STACK_LIMIT_EXCEEDED
			mov	dword eax, [MODULID]
			mov	dword [STATE], ebx
			xor	dword ebx, ebx
			mov	dword ecx, __LINE__
			jmp	core_assertion
.check_high_limit:	cmp	dword ebx, 00200h
			jb	.write
			jmp	.limit_exceeded
.write:
%endif
%endif
			mov	byte  [RAM + ebx], al
			ret

c1541memory_read_opcode:	equ	read_opcode_ram
c1541memory_read_stack:		equ	read_stack
c1541memory_read_data:		equ	c1541memory_read
c1541memory_write_stack:	equ	write_stack
c1541memory_write_data:		equ	c1541memory_write
;******************************************************** ec64@unusedino.de *

