;****************************************************************************
;****************************************************************************
;*
;* c64link.i
;*
;****************************************************************************
;****************************************************************************





%ifndef C64LINK_I
%define C64LINK_I

%include "defines.i"
%include "macros.i"
%include "ec64types.i"

%include "cpu65xx.i"
%include "cia6526.i"
%include "sid6581.i"
%include "vic6569.i"
%include "c64memory.i"

;****************************************************************************
;* Exported Symbols *********************************************************
;****************************************************************************
%ifdef C64LINK_N
global c64link_initialize
global c64link_exit
global c64link_instance_initialize
global c64link_instance_exit
global c64link_reset_hard
global c64link_reset_soft
global c64link_cia_counter_set
global c64link_cia_counter_refresh
global c64link_cia_counter_clear
global c64link_cia_counter_process
global c64link_keyboard
global c64link_joystick1
global c64link_joystick2
global c64link_cia1_irq
global c64link_cia1_pra
global c64link_cia1_prb
global c64link_cia1_sp
global c64link_cia1_pc
global c64link_cia2_nmi
global c64link_cia2_pra
global c64link_cia2_prb
global c64link_cia2_sp
global c64link_cia2_pc
global c64link_vic_irq
%else
extern c64link_initialize
extern c64link_exit
extern c64link_instance_initialize
extern c64link_instance_exit
extern c64link_reset_hard
extern c64link_reset_soft
extern c64link_cia_counter_set
extern c64link_cia_counter_refresh
extern c64link_cia_counter_clear
extern c64link_cia_counter_process
extern c64link_keyboard
extern c64link_joystick1
extern c64link_joystick2
extern c64link_cia1_irq
extern c64link_cia1_pra
extern c64link_cia1_prb
extern c64link_cia1_sp
extern c64link_cia1_pc
extern c64link_cia2_nmi
extern c64link_cia2_pra
extern c64link_cia2_prb
extern c64link_cia2_sp
extern c64link_cia2_pc
extern c64link_vic_irq
%endif



;****************************************************************************
;* Module Data **************************************************************
;****************************************************************************
%assign C64LINK_COUNTER_TABLE_ENTRIES	4
%assign C64LINK_KEY_CODE_ENTRIES	9

				struc	tc64link
				alignb	4
d__c64link_modulid:		resd	1
d__c64link_counter_base:	resd	1
d__c64link_counter:		resd	1
ad__c64link_counter_table:	resd	(2 * C64LINK_COUNTER_TABLE_ENTRIES)
ad__c64link_key_codes:		resd	C64LINK_KEY_CODE_ENTRIES
b__c64link_joystick1:		resb	1
b__c64link_joystick2:		resb	1
b__c64link_vic_high_bits:	resb	1
b__c64link_serial_bus:		resb	1
				endstruc



				struc	tc64
				alignb	32
tcpu65xx__c64:			resb	tcpu65xx_size
				alignb	32
tlink__c64:			resb	tc64link_size
				alignb	32
tcia6526__c64_1:		resb	tcia6526_size
				alignb	32
tcia6526__c64_2:		resb	tcia6526_size
				alignb	32
tsid6581__c64:			resb	tsid6581_size
				alignb	32
tvic6569__c64:			resb	tvic6569_size
				alignb	32
tmemory__c64:			resb	tc64memory_size
				endstruc



;****************************************************************************
;* assign's & defines's *****************************************************
;****************************************************************************
%assign C64LINK_COUNTER_DEFAULT_VALUE	010000000h

%define C64LINK_MODULID			esi + tlink__c64 + d__c64link_modulid
%define C64LINK_COUNTER_BASE		esi + tlink__c64 + d__c64link_counter_base
%define C64LINK_COUNTER			esi + tlink__c64 + d__c64link_counter
%define C64LINK_COUNTER_TABLE		esi + tlink__c64 + ad__c64link_counter_table
%define C64LINK_KEY_CODES		esi + tlink__c64 + ad__c64link_key_codes
%define C64LINK_JOYSTICK1		esi + tlink__c64 + b__c64link_joystick1
%define C64LINK_JOYSTICK2		esi + tlink__c64 + b__c64link_joystick2
%define C64LINK_VIC_HIGH_BITS		esi + tlink__c64 + b__c64link_vic_high_bits
%define C64LINK_SERIAL_BUS		esi + tlink__c64 + b__c64link_serial_bus

%assign C64LINK_CIA1_COUNTER_OFFSET	000h
%assign C64LINK_CIA2_COUNTER_OFFSET	002h

%assign C64LINK_CIA1_IRQ_NUMBER		001h
%assign C64LINK_VIC_IRQ_NUMBER		002h



;****************************************************************************
;* Macro's ******************************************************************
;****************************************************************************
%macro C64LINK_CALL_INITIALIZE 0
			call	c64link_initialize
%endmacro



%macro C64LINK_CALL_INSTANCE_INITIALIZE 1
			call	c64link_instance_initialize
			mov	dword eax, %1
			mov	dword [C64LINK_MODULID], eax
%endmacro



%macro C64LINK_CALL_RESET_HARD 0
			call	c64link_reset_hard
%endmacro



%macro C64LINK_CALL_RESET_SOFT 0
			call	c64link_reset_soft
%endmacro



%macro C64LINK_CALL_DECREMENT_COUNTER 0
			dec	dword [C64LINK_COUNTER]
			jnz	%%no_call
			call	c64link_cia_counter_process
%%no_call:
%endmacro


%endif
;******************************************************** ec64@unusedino.de *

