;****************************************************************************
;****************************************************************************
;*
;* c64memory.n
;*
;****************************************************************************
;****************************************************************************





%define C64MEMORY_N
%include "c64memory.i"
%include "c64link.i"
%include "core.i"





;****************************************************************************
;****************************************************************************
;*
;* PART 1: assign's & define's
;*
;****************************************************************************
;****************************************************************************





%define MODULID			C64MEMORY_MODULID
%define STATE			C64MEMORY_STATE
%define CPU6510_DDR		C64MEMORY_CPU6510_DDR
%define CPU6510_IO		C64MEMORY_CPU6510_IO
%define CPU6510_DATA		C64MEMORY_CPU6510_DATA
%define CPU6510_MASK		C64MEMORY_CPU6510_MASK
%define READ_OPCODE_LOW_LIMIT	C64MEMORY_READ_OPCODE_LOW_LIMIT
%define READ_OPCODE_HIGH_LIMIT	C64MEMORY_READ_OPCODE_HIGH_LIMIT
%define RAM			C64MEMORY_RAM
%define COLOR_RAM		C64MEMORY_COLOR_RAM
%define BASIC_ROM		C64MEMORY_BASIC_ROM
%define CHARACTER_ROM		C64MEMORY_CHARACTER_ROM
%define KERNEL_ROM		C64MEMORY_KERNEL_ROM



%assign STACK_LIMIT_EXCEEDED	C64MEMORY_STACK_LIMIT_EXCEEDED





;****************************************************************************
;****************************************************************************
;*
;* PART 2: Initialisation & Management Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* c64memory_initialize *****************************************************
;****************************************************************************
section .text
c64memory_initialize:
			ret



;****************************************************************************
;* c64memory_exit ***********************************************************
;****************************************************************************
section .text
c64memory_exit:
			ret



;****************************************************************************
;* c64memory_instance_initialize ********************************************
;****************************************************************************
section .text
c64memory_instance_initialize:
			INSTANCE_INITIALIZE  tc64memory_size



;****************************************************************************
;* c64memory_instance_exit **************************************************
;****************************************************************************
section .text
c64memory_instance_exit:
			ret



;****************************************************************************
;* c64memory_reset_hard *****************************************************
;****************************************************************************
section .text
c64memory_reset_hard:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edi
			mov	dword ebx, 00200h
			xor	dword ecx, ecx
			lea	dword edi, [RAM]
			cld
.fill_memory:		xor	dword eax, eax
			mov	byte  cl, 010h
			rep stosd
			dec	dword eax
			mov	byte  cl, 010h
			rep stosd
			dec	dword ebx
			jnz	.fill_memory
			pop	dword edi
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			jmp	c64memory_reset_soft



;****************************************************************************
;* c64memory_reset_soft *****************************************************
;****************************************************************************
section .text
c64memory_reset_soft:
			push	dword eax
			push	dword ebx
			xor	dword ebx, ebx
			mov	dword [STATE], ebx
							;set the correct 
			mov	byte  al, 02fh		;data direction
			call	c64memory_write		;on the 6510
			inc	dword ebx
			mov	byte  al, 027h		;Basic, I/O, Kernel
			call	c64memory_write
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* c64memory_load_rom_basic *************************************************
;****************************************************************************
;* eax=>  position of the ROM
;****************************************************************************
section .text
c64memory_load_rom_basic:
			push	dword esi
			lea	dword edi, [BASIC_ROM]
			mov	dword esi, eax
			mov	dword ecx, 02000h
			cld
			rep movsb
			pop	dword esi
			ret



;****************************************************************************
;* c64memory_load_rom_character *********************************************
;****************************************************************************
;* eax=>  position of the ROM
;****************************************************************************
section .text
c64memory_load_rom_character:
			push	dword esi
			lea	dword edi, [CHARACTER_ROM]
			mov	dword esi, eax
			mov	dword ecx, 01000h
			cld
			rep movsb
			pop	dword esi
			ret



;****************************************************************************
;* c64memory_load_rom_kernel ************************************************
;****************************************************************************
;* eax=>  position of the ROM
;****************************************************************************
section .text
c64memory_load_rom_kernel:
			push	dword esi
			lea	dword edi, [KERNEL_ROM]
			mov	dword esi, eax
			mov	dword ecx, 02000h
			cld
			rep movsb
			pop	dword esi
			ret





;****************************************************************************
;****************************************************************************
;*
;* PART 3: Memory Access Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* c64memory_read ***********************************************************
;****************************************************************************
section .text
c64memory_read:
			and	dword ebx, 0ffffh
			mov	dword eax, ebx
			shr	dword eax, 12
			add	byte  al, [.b__functions_offset]
			jmp	dword [.ap__read_functions + 4 * eax]
.read_cpu6510:		cmp	word  bx, 00002h
			jb	.read_cpu6510_1
.read_ram:		mov	byte  al, [RAM + ebx]
			ret
.read_cpu6510_1:	mov	byte  al, [CPU6510_DDR + ebx]
			ret
.read_basic_rom:	mov	byte  al, [BASIC_ROM - 0a000h + ebx]
			ret
.read_io:		cmp	word  bx, 0d800h
			jae	.read_io_1
			cmp	word  bx, 0d400h
			jae	.read_sid
.read_vic:		call	vic6569_read
			ret
.read_sid:		call	sid6581_read
			ret
.read_io_1:		cmp	word  bx, 0dc00h
			jae	.read_io_2
.read_color_ram:	mov	byte  al, [COLOR_RAM - 0d800h + ebx]
			or	byte  al, 0f0h
			ret
.read_io_2:		cmp	word  bx, 0dd00h
			jb	.read_cia1
			cmp	word  bx, 0de00h
			jb	.read_cia2
.read_rubbish:		mov	byte  al, 05ah
			ret
.read_cia1:		CIA6526_CALL_READ  tcia6526__c64_1
			ret
.read_cia2:		CIA6526_CALL_READ  tcia6526__c64_2
			ret
.read_character_rom:	mov	byte  al, [CHARACTER_ROM - 0d000h + ebx]
			ret
.read_kernel_rom:	mov	byte  al, [KERNEL_ROM - 0e000h + ebx]
			ret

			align	4
.ap__read_functions:	
			;BASIC ROM - I/O - KERNEL ROM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_basic_rom		;$a000 - $afff
			dd	.read_basic_rom		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_io		;$d000 - $dfff
			dd	.read_kernel_rom	;$e000 - $efff
			dd	.read_kernel_rom	;$f000 - $ffff

			;RAM - I/O - KERNEL ROM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_ram		;$a000 - $afff
			dd	.read_ram		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_io		;$d000 - $dfff
			dd	.read_kernel_rom	;$e000 - $efff
			dd	.read_kernel_rom	;$f000 - $ffff

			;RAM - I/O - RAM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_ram		;$a000 - $afff
			dd	.read_ram		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_io		;$d000 - $dfff
			dd	.read_ram		;$e000 - $efff
			dd	.read_ram		;$f000 - $ffff

			;RAM - RAM - RAM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_ram		;$a000 - $afff
			dd	.read_ram		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_ram		;$d000 - $dfff
			dd	.read_ram		;$e000 - $efff
			dd	.read_ram		;$f000 - $ffff

			;BASIC ROM - CHARACTER ROM - KERNEL ROM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_basic_rom		;$a000 - $afff
			dd	.read_basic_rom		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_character_rom	;$d000 - $dfff
			dd	.read_kernel_rom	;$e000 - $efff
			dd	.read_kernel_rom	;$f000 - $ffff

			;RAM - CHARACTER ROM - KERNEL ROM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_ram		;$a000 - $afff
			dd	.read_ram		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_character_rom	;$d000 - $dfff
			dd	.read_kernel_rom	;$e000 - $efff
			dd	.read_kernel_rom	;$f000 - $ffff

			;RAM - CHARACTER ROM - RAM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_ram		;$a000 - $afff
			dd	.read_ram		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_character_rom	;$d000 - $dfff
			dd	.read_ram		;$e000 - $efff
			dd	.read_ram		;$f000 - $ffff

section .data
.b__functions_offset:	db	0



;****************************************************************************
;* c64memory_write **********************************************************
;****************************************************************************
section .text
c64memory_write:
			and	dword ebx, 0ffffh
			cmp	word  bx, 00002h
			jb	.write_cpu6510
			jmp	dword [.p__write_function]
.write_cpu6510:		cmp	byte  bl, 001h
			je	.write_cpu6510_io
.write_cpu6510_ddr:	mov	byte  [CPU6510_DDR], al
			not	byte  al
			and	byte  al, 0dfh
			test	byte  al, 008h
			jz	.write_cpu6510_ddr_1
			mov	byte  ah, [CPU6510_IO]
			and	byte  al, 0f7h
			and	byte  ah, 008h
			or	byte  al, ah
.write_cpu6510_ddr_1:	and	byte  al, 03fh
			mov	byte  [CPU6510_MASK], al
			mov	byte  al, [CPU6510_DATA]
			jmp	.calculate_offset
.write_cpu6510_io:	mov	byte  [CPU6510_DATA], al
.calculate_offset:	and	byte  al, [CPU6510_DDR]
			or	byte  al, [CPU6510_MASK]
			mov	byte  [CPU6510_IO], al
			and	byte  al, 007h
			jnz	.calculate_offset_1
			mov	byte  al, 004h
.calculate_offset_1:	xor	byte  al, 007h
			shl	byte  al, 4
			mov	byte  [c64memory_read.b__functions_offset], al
			cmp	byte  al, 030h
			jb	.calculate_offset_2
			mov	dword [.p__write_function], .write_ram
			ret
.calculate_offset_2:	mov	dword [.p__write_function], .write_io
			ret
.write_io:		cmp	word  bx, 0d000h
			jb	.write_ram
			cmp	word  bx, 0e000h
			jb	.write_io_1
.write_ram:		mov	byte  [RAM + ebx], al
			ret
.write_io_1:		cmp	word  bx, 0d800h
			jae	.write_io_2
			cmp	word  bx, 0d400h
			jae	.write_sid
.write_vic:		call	vic6569_write
			ret
.write_sid:		call	sid6581_write
			ret
.write_io_2:		cmp	word  bx, 0dc00h
			jae	.write_io_3
.write_color_ram:	mov	byte  [COLOR_RAM - 0d800h + ebx], al
			ret
.write_io_3:		cmp	word  bx, 0dd00h
			jb	.write_cia1
			cmp	word  bx, 0de00h
			jae	.write_io_4
.write_cia2:		CIA6526_CALL_WRITE  tcia6526__c64_2
.write_io_4:		ret
.write_cia1:		CIA6526_CALL_WRITE  tcia6526__c64_1
			ret

section .data
.p__write_function:	dd	.write_io



;****************************************************************************
;* read_stack ***************************************************************
;****************************************************************************
section .text
read_stack:
%ifdef C64MEMORY_ASSERTION
%ifdef C64MEMORY_ASSERTION_STACK
			cmp	dword ebx, 00100h
			jae	.check_high_limit
.limit_exceeded:	mov	dword ebx, STACK_LIMIT_EXCEEDED
			mov	dword eax, [MODULID]
			mov	dword [STATE], ebx
			xor	dword ebx, ebx
			mov	dword ecx, __LINE__
			jmp	core_assertion
.check_high_limit:	cmp	dword ebx, 00200h
			jb	.read
			jmp	.limit_exceeded
.read:
%endif
%endif
			mov	byte  al, [RAM + ebx]
			ret



;****************************************************************************
;* read_data ****************************************************************
;****************************************************************************
section .text
read_data:
			ret



;****************************************************************************
;* write_stack **************************************************************
;****************************************************************************
section .text
write_stack:
%ifdef C64MEMORY_ASSERTION
%ifdef C64MEMORY_ASSERTION_STACK
			cmp	dword ebx, 00100h
			jae	.check_high_limit
.limit_exceeded:	mov	dword ebx, STACK_LIMIT_EXCEEDED
			mov	dword eax, [MODULID]
			mov	dword [STATE], ebx
			xor	dword ebx, ebx
			mov	dword ecx, __LINE__
			jmp	core_assertion
.check_high_limit:	cmp	dword ebx, 00200h
			jb	.write
			jmp	.limit_exceeded
.write:
%endif
%endif
			mov	byte  [RAM + ebx], al
			ret



;****************************************************************************
;* write_data ***************************************************************
;****************************************************************************
section .text
write_data:
			ret

c64memory_read_opcode:	equ	c64memory_read
c64memory_read_stack:	equ	read_stack
c64memory_read_data:	equ	c64memory_read
c64memory_write_stack:	equ	write_stack
c64memory_write_data:	equ	c64memory_write
;******************************************************** ec64@unusedino.de *

