;****************************************************************************
;****************************************************************************
;*
;* cia6526.i
;*
;****************************************************************************
;****************************************************************************




%ifndef CIA6526_I
%define CIA6526_I

%include "defines.i"
%include "macros.i"
%include "ec64types.i"

;****************************************************************************
;* Exported Symbols *********************************************************
;****************************************************************************
%ifdef CIA6526_N
global cia6526_initialize
global cia6526_exit
global cia6526_instance_initialize
global cia6526_instance_exit
global cia6526_reset_hard
global cia6526_reset_soft
global cia6526_signal_pra
global cia6526_signal_prb
global cia6526_read
global cia6526_write
%else
extern cia6526_initialize
extern cia6526_exit
extern cia6526_instance_initialize
extern cia6526_instance_exit
extern cia6526_reset_hard
extern cia6526_reset_soft
extern cia6526_signal_pra
extern cia6526_signal_prb
extern cia6526_read
extern cia6526_write
%endif



;****************************************************************************
;* Module Data **************************************************************
;****************************************************************************
				struc	tcia6526
				alignb	4
d__cia6526_modulid:		resd	1
d__cia6526_state:		resd	1
p__cia6526_link_irq:		resd	1
p__cia6526_link_pra:		resd	1
p__cia6526_link_prb:		resd	1
d__cia6526_counter_offset:	resd	1
q__cia6526_timestamp_ta:	resd	2
d__cia6526_counter_ta:		resd	1
q__cia6526_timestamp_tb:	resd	2
d__cia6526_counter_tb:		resd	1
b__cia6526_latch_pra:		resb	1
b__cia6526_signal_pra:		resb	1
b__cia6526_latch_prb:		resb	1
b__cia6526_signal_prb:		resb	1
b__cia6526_latch_ta_hi:		resb	1
b__cia6526_latch_tb_hi:		resb	1
b__cia6526_irq_mask:		resb	1
ab__cia6526_registers:		resb	16
				endstruc



;****************************************************************************
;* assign's & defines's *****************************************************
;****************************************************************************
%assign CIA6526_PRA		000h
%assign CIA6526_PRB		001h
%assign CIA6526_DDRA		002h
%assign CIA6526_DDRB		003h
%assign CIA6526_TA_LO		004h
%assign CIA6526_TA_HI		005h
%assign CIA6526_TB_LO		006h
%assign CIA6526_TB_HI		007h
%assign CIA6526_TOD_10THS	008h
%assign CIA6526_TOD_SEC		009h
%assign CIA6526_TOD_MIN		00ah
%assign CIA6526_TOD_HR		00bh
%assign CIA6526_SDR		00ch
%assign CIA6526_ICR		00dh
%assign CIA6526_CRA		00eh
%assign CIA6526_CRB		00fh

%define CIA6526_MODULID			edi + d__cia6526_modulid
%define CIA6526_STATE			edi + d__cia6526_state
%define CIA6526_LINK_IRQ		edi + p__cia6526_link_irq
%define CIA6526_LINK_PRA		edi + p__cia6526_link_pra
%define CIA6526_LINK_PRB		edi + p__cia6526_link_prb
%define CIA6526_COUNTER_OFFSET		edi + d__cia6526_counter_offset
%define CIA6526_TIMESTAMP_TA		edi + q__cia6526_timestamp_ta
%define CIA6526_COUNTER_TA		edi + d__cia6526_counter_ta
%define CIA6526_TIMESTAMP_TB		edi + q__cia6526_timestamp_tb
%define CIA6526_COUNTER_TB		edi + d__cia6526_counter_tb
%define CIA6526_LATCH_PRA		edi + b__cia6526_latch_pra
%define CIA6526_SIGNAL_PRA		edi + b__cia6526_signal_pra
%define CIA6526_LATCH_PRB		edi + b__cia6526_latch_prb
%define CIA6526_SIGNAL_PRB		edi + b__cia6526_signal_prb
%define CIA6526_LATCH_TA_HI		edi + b__cia6526_latch_ta_hi
%define CIA6526_LATCH_TB_HI		edi + b__cia6526_latch_tb_hi
%define CIA6526_IRQ_MASK		edi + b__cia6526_irq_mask
%define CIA6526_REGISTERS		edi + ab__cia6526_registers

%assign CIA6526_TA_FORCE_LOAD		000000001h
%assign CIA6526_TA_STOPPED		000000002h
%assign CIA6526_TA_LOCKED		000000004h
%assign CIA6526_TB_FORCE_LOAD		000000008h
%assign CIA6526_TB_STOPPED		000000010h
%assign CIA6526_TB_LOCKED		000000020h
%assign CIA6526_TB_CASCADED		000000040h



;****************************************************************************
;* Macro's ******************************************************************
;****************************************************************************
%macro CIA6526_CALL_INITIALIZE 0
			call	cia6526_initialize
%endmacro



%macro CIA6526_CALL_INSTANCE_INITIALIZE 6
			lea	dword edi, [esi + %1]
			call	cia6526_instance_initialize
			mov	dword eax, %2
			mov	dword ebx, %3
			mov	dword [CIA6526_MODULID], eax
			mov	dword [CIA6526_COUNTER_OFFSET], ebx
			mov	dword eax, %4
			mov	dword ebx, %5
			mov	dword ecx, %6
			mov	dword [CIA6526_LINK_IRQ], eax
			mov	dword [CIA6526_LINK_PRA], ebx
			mov	dword [CIA6526_LINK_PRB], ecx
%endmacro



%macro CIA6526_CALL_RESET_HARD 1
			lea	dword edi, [esi + %1]
			call	cia6526_reset_hard
%endmacro



%macro CIA6526_CALL_RESET_SOFT 1
			lea	dword edi, [esi + %1]
			call	cia6526_reset_soft
%endmacro



%macro CIA6526_CALL_READ 1
			lea	dword edi, [esi + %1]
			call	cia6526_read
%endmacro



%macro CIA6526_CALL_WRITE 1
			lea	dword edi, [esi + %1]
			call	cia6526_write
%endmacro



%endif
;******************************************************** ec64@unusedino.de *

