;****************************************************************************
;****************************************************************************
;*
;* cia6526.n
;*
;****************************************************************************
;****************************************************************************





%define CIA6526_N
%include "cia6526.i"
%include "core.i"
%include "c64link.i"





;****************************************************************************
;****************************************************************************
;*
;* PART 1: assign's & defines's
;*
;****************************************************************************
;****************************************************************************





%assign PRA		CIA6526_PRA
%assign PRB		CIA6526_PRB
%assign DDRA		CIA6526_DDRA
%assign DDRB		CIA6526_DDRB
%assign TA_LO		CIA6526_TA_LO
%assign TA_HI		CIA6526_TA_HI
%assign TB_LO		CIA6526_TB_LO
%assign TB_HI		CIA6526_TB_HI
%assign TOD_10THS	CIA6526_TOD_10THS
%assign TOD_SEC		CIA6526_TOD_SEC
%assign TOD_MIN		CIA6526_TOD_MIN
%assign TOD_HR		CIA6526_TOD_HR
%assign SDR		CIA6526_SDR
%assign ICR		CIA6526_ICR
%assign CRA		CIA6526_CRA
%assign CRB		CIA6526_CRB



%define MODULID		CIA6526_MODULID
%define STATE		CIA6526_STATE
%define LINK_IRQ	CIA6526_LINK_IRQ
%define LINK_PRA	CIA6526_LINK_PRA
%define LINK_PRB	CIA6526_LINK_PRB
%define COUNTER_OFFSET	CIA6526_COUNTER_OFFSET
%define TIMESTAMP_TA	CIA6526_TIMESTAMP_TA
%define COUNTER_TA	CIA6526_COUNTER_TA
%define TIMESTAMP_TB	CIA6526_TIMESTAMP_TB
%define COUNTER_TB	CIA6526_COUNTER_TB
%define LATCH_PRA	CIA6526_LATCH_PRA
%define SIGNAL_PRA	CIA6526_SIGNAL_PRA
%define LATCH_PRB	CIA6526_LATCH_PRB
%define SIGNAL_PRB	CIA6526_SIGNAL_PRB
%define LATCH_TA_HI	CIA6526_LATCH_TA_HI
%define LATCH_TB_HI	CIA6526_LATCH_TB_HI
%define IRQ_MASK	CIA6526_IRQ_MASK
%define REGISTERS	CIA6526_REGISTERS



%assign TA_FORCE_LOAD	CIA6526_TA_FORCE_LOAD
%assign TA_STOPPED	CIA6526_TA_STOPPED
%assign TA_LOCKED	CIA6526_TA_LOCKED
%assign TB_FORCE_LOAD	CIA6526_TB_FORCE_LOAD
%assign TB_STOPPED	CIA6526_TB_STOPPED
%assign TB_LOCKED	CIA6526_TB_LOCKED
%assign TB_CASCADED	CIA6526_TB_CASCADED

%macro GET_CYCLES 0
			call	core_get_c64_cycles
%endmacro





;****************************************************************************
;****************************************************************************
;*
;* PART 2: Initialisation & Management Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* cia6526_initialize *******************************************************
;****************************************************************************
section .text
cia6526_initialize:
			ret



;****************************************************************************
;* cia6526_exit *************************************************************
;****************************************************************************
section .text
cia6526_exit:
			ret



;****************************************************************************
;* cia6526_instance_initialize **********************************************
;****************************************************************************
section .text
cia6526_instance_initialize:
			INSTANCE_INITIALIZE  tcia6526_size



;****************************************************************************
;* cia6526_instance_exit ****************************************************
;****************************************************************************
section .text
cia6526_instance_exit:
			ret





;****************************************************************************
;****************************************************************************
;*
;* PART 3: Signal Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* cia6526_reset_hard *******************************************************
;****************************************************************************
section .text
cia6526_reset_hard:
			jmp	cia6526_reset_soft



;****************************************************************************
;* cia6526_reset_soft *******************************************************
;****************************************************************************
section .text
cia6526_reset_soft:
			push	dword eax
			push	dword ebx
			mov	dword [STATE], (TA_STOPPED | TB_STOPPED)
			xor	dword ebx, ebx
			xor	dword eax, eax
			dec	dword ebx
			mov	dword [COUNTER_TA], eax
			mov	dword [COUNTER_TB], eax
			mov	byte  [LATCH_PRA], al
			mov	byte  [SIGNAL_PRA], bl
			mov	byte  [LATCH_PRB], al
			mov	byte  [SIGNAL_PRB], bl
			mov	byte  [LATCH_TA_HI], al
			mov	byte  [LATCH_TB_HI], al
			mov	byte  [IRQ_MASK], al
			mov	dword [REGISTERS], eax
			mov	dword [REGISTERS + 4], eax
			mov	dword [REGISTERS + 8], eax
			mov	dword [REGISTERS + 12], eax
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* cia6526_signal_pra *******************************************************
;****************************************************************************
section .text
cia6526_signal_pra:
			push	dword eax
			mov	byte  [SIGNAL_PRA], al
			mov	byte  ah, [REGISTERS + DDRA]
			xor	byte  ah, NOT8
			or	byte  ah, [LATCH_PRA]
			and	byte  al, ah
			mov	byte  [REGISTERS + PRA], al
			pop	dword eax
			ret



;****************************************************************************
;* cia6526_signal_prb *******************************************************
;****************************************************************************
section .text
cia6526_signal_prb:
			push	dword eax
			mov	byte  [SIGNAL_PRB], al
			mov	byte  ah, [REGISTERS + DDRB]
			xor	byte  ah, NOT8
			or	byte  ah, [LATCH_PRB]
			and	byte  al, ah
			mov	byte  [REGISTERS + PRB], al
			pop	dword eax
			ret





;****************************************************************************
;****************************************************************************
;*
;* PART 4: Emulation Code
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* cia6526_read *************************************************************
;****************************************************************************
section .text
cia6526_read:
			mov	dword ebp, ebx
			push	dword ecx
			and	dword ebp, 00000000fh
			mov	dword ecx, [STATE]
			jmp	dword [.ap__registers + 4 * ebp]

			align	4
.ap__registers:		dd	read_register
			dd	read_register
			dd	read_register
			dd	read_register
			dd	read_ta_lo
			dd	read_ta_hi
			dd	read_tb_lo
			dd	read_tb_hi
			dd	read_register
			dd	read_register
			dd	read_register
			dd	read_tod_hr
			dd	read_register
			dd	read_icr
			dd	read_register
			dd	read_register

read_register:
			mov	byte  al, [REGISTERS + ebp]
			pop	dword ecx
			ret

read_ta_lo:
			test	dword ecx, TA_STOPPED
			jnz	.stopped
			or	dword ecx, TA_LOCKED
			call	read_timer_a
			mov	dword [STATE], ecx
			mov	byte  [LATCH_TA_HI], ah
			pop	dword ecx
			ret
.stopped:		mov	byte  al, [COUNTER_TA]
			pop	dword ecx
			ret

read_ta_hi:
			test	dword ecx, TA_STOPPED
			jnz	.stopped
			test	dword ecx, TA_LOCKED
			jnz	.read_from_latch
			call	read_timer_a
			mov	byte  al, ah
			pop	dword ecx
			ret
.read_from_latch:	and	dword ecx, (NOT32 - TA_LOCKED)
			mov	byte  al, [LATCH_TA_HI]
			mov	dword [STATE], ecx
			pop	dword ecx
			ret
.stopped:		mov	byte  al, [COUNTER_TA + 1]
			pop	dword ecx
			ret

read_timer_a:
			push	dword ebx
			push	dword edx
			GET_CYCLES
			mov	dword ebx, [COUNTER_TA]
			sub	dword eax, [TIMESTAMP_TA]
			mov	dword edx, ebx
			add	dword ebx, 2
			test	dword [STATE], TA_FORCE_LOAD
			jz	.no_force_load
			inc	dword ebx
.no_force_load:		sub	dword ebx, eax
			cmp	dword ebx, edx
			jb	.no_correction
			mov	dword ebx, edx
.no_correction:		pop	dword edx
			mov	dword eax, ebx
			pop	dword ebx
			ret

read_tb_lo:
			test	dword ecx, TB_STOPPED
			jnz	.stopped
			or	dword ecx, TB_LOCKED
			call	read_timer_b
			mov	dword [STATE], ecx
			mov	byte  [LATCH_TB_HI], ah
			pop	dword ecx
			ret
.stopped:		mov	byte  al, [COUNTER_TB]
			pop	dword ecx
			ret

read_tb_hi:
			test	dword ecx, TB_STOPPED
			jnz	.stopped
			test	dword ecx, TB_LOCKED
			jnz	.read_from_latch
			call	read_timer_b
			mov	byte  al, ah
			pop	dword ecx
			ret
.read_from_latch:	and	dword ecx, (NOT32 - TB_LOCKED)
			mov	byte  al, [LATCH_TB_HI]
			mov	dword [STATE], ecx
			pop	dword ecx
			ret
.stopped:		mov	byte  al, [COUNTER_TB + 1]
			pop	dword ecx
			ret

read_timer_b:
			push	dword ebx
			push	dword edx
			GET_CYCLES
			mov	dword ebx, [COUNTER_TB]
			sub	dword eax, [TIMESTAMP_TB]
			mov	dword edx, ebx
			add	dword ebx, 2
			test	dword [STATE], TB_FORCE_LOAD
			jz	.no_force_load
			inc	dword ebx
.no_force_load:		sub	dword ebx, eax
			cmp	dword ebx, edx
			jb	.no_correction
			mov	dword ebx, edx
.no_correction:		pop	dword edx
			mov	dword eax, ebx
			pop	dword ebx
			ret

read_tod_hr:
			mov	byte  al, 080h
			pop	dword ecx
			ret

read_icr:
			xor	byte  al, al
			call	dword [LINK_IRQ]
			xor	byte  cl, cl
			mov	byte  al, [REGISTERS + ICR]
			mov	byte  [REGISTERS + ICR], cl
			pop	dword ecx
			ret



;****************************************************************************
;* cia6526_write ************************************************************
;****************************************************************************
section .text
cia6526_write:
			mov	dword ebp, ebx
			and	dword ebp, 00000000fh
			jmp	dword [.ap__registers + 4 * ebp]

			align	4
.ap__registers:		dd	write_pra
			dd	write_prb
			dd	write_ddra
			dd	write_ddrb
			dd	write_ta_lo
			dd	write_ta_hi
			dd	write_tb_lo
			dd	write_tb_hi
			dd	write_ignore
			dd	write_ignore
			dd	write_ignore
			dd	write_ignore
			dd	write_ignore
			dd	write_icr
			dd	write_cra
			dd	write_crb

write_ignore:
			ret

write_pra:
			mov	byte  [LATCH_PRA], al
			mov	byte  ah, [REGISTERS + DDRA]
			xor	byte  ah, NOT8
			or	byte  al, ah
			and	byte  al, [SIGNAL_PRA]
			mov	byte  [REGISTERS + PRA], al
			jmp	dword [LINK_PRA]

write_prb:
			mov	byte  [LATCH_PRB], al
			mov	byte  ah, [REGISTERS + DDRB]
			xor	byte  ah, NOT8
			or	byte  al, ah
			and	byte  al, [SIGNAL_PRB]
			mov	byte  [REGISTERS + PRB], al
			jmp	dword [LINK_PRB]

write_ddra:
			mov	byte  [REGISTERS + DDRA], al
			xor	byte  al, NOT8
			or	byte  al, [LATCH_PRA]
			and	byte  al, [SIGNAL_PRA]
			mov	byte  [REGISTERS + PRA], al
			jmp	dword [LINK_PRA]

write_ddrb:
			mov	byte  [REGISTERS + DDRB], al
			xor	byte  al, NOT8
			or	byte  al, [LATCH_PRB]
			and	byte  al, [SIGNAL_PRB]
			mov	byte  [REGISTERS + PRB], al
			jmp	dword [LINK_PRB]

write_ta_lo:
			mov	byte  [REGISTERS + TA_LO], al
			ret

write_ta_hi:
			mov	byte  [REGISTERS + TA_HI], al
			test	byte  [REGISTERS + CRA], 001h
			jnz	.timer_in_use
			mov	byte  ah, al
			mov	byte  al, [REGISTERS + TA_LO]
			and	dword eax, NOT16
			mov	dword [COUNTER_TA], eax
.timer_in_use:		ret

write_tb_lo:
			mov	byte  [REGISTERS + TB_LO], al
			ret

write_tb_hi:
			mov	byte  [REGISTERS + TB_HI], al
			test	byte  [REGISTERS + CRB], 001h
			jnz	.timer_in_use
			mov	byte  ah, al
			mov	byte  al, [REGISTERS + TB_LO]
			and	dword eax, NOT16
			mov	dword [COUNTER_TA], eax
.timer_in_use:		ret

write_icr:
			test	byte  al, 080h
			jz	.clear_bits
			and	byte  al, 07fh
			or	byte  [IRQ_MASK], al
			ret
.clear_bits:		xor	byte  al, 07fh
			and	byte  [IRQ_MASK], al
			ret

write_cra:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: fix the behavior if the timer is already running
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			push	dword ebx
			push	dword ecx
			push	dword edx
			mov	dword edx, [STATE]
			mov	dword ecx, [COUNTER_TA]
			and	dword edx, (NOT32 - TA_FORCE_LOAD)
			test	byte  al, 010h
			jz	.no_force_load
			mov	byte  cl, [REGISTERS + TA_LO]
			and	byte  al, 0efh
			mov	byte  ch, [REGISTERS + TA_HI]
			or	dword edx, TA_FORCE_LOAD
			mov	dword [COUNTER_TA], ecx
			inc	dword ecx
.no_force_load:		mov	byte  [REGISTERS + CRA], al
			test	byte  al, 001h
			jz	.stop
			and	dword edx, (NOT32 - TA_LOCKED)
			test	byte  al, 020h
			jz	.count_phi2
;!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: implement count cnt
;!!!!!!!!!!!!!!!!!!!!!!!!!
			mov	dword eax, [MODULID]
			xor	dword ebx, ebx
			mov	dword ecx, __LINE__
			jmp	core_exception
.stop:			call	read_timer_a
			sub	dword eax, 2
			or	dword edx, TA_STOPPED
			mov	dword ebx, eax
			mov	dword [STATE], edx
			and	dword ebx, NOT16
			mov	dword eax, [COUNTER_OFFSET]
			mov	dword [COUNTER_TA], ebx
			call	c64link_cia_counter_clear
			jmp	.end
.count_phi2:		mov	dword eax, [COUNTER_OFFSET]
			mov	dword ebx, process_timer_a
			and	dword edx, (NOT32 - TA_STOPPED)
			add	dword ecx, 2
			mov	dword [STATE], edx
			call	c64link_cia_counter_set
			GET_CYCLES
			mov	dword [TIMESTAMP_TA], eax
			mov	dword [TIMESTAMP_TA + 4], edx
.end:			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			ret

process_timer_a:
			mov	dword edx, [STATE]
			test	byte  [REGISTERS + CRA], 008h
			jnz	.one_shot
			xor	dword ecx, ecx
			mov	dword eax, [COUNTER_OFFSET]
			mov	byte  cl, [REGISTERS + TA_LO]
			mov	dword ebx, process_timer_a
			mov	byte  ch, [REGISTERS + TA_HI]
			and	dword edx, (NOT32 - TA_FORCE_LOAD)
			mov	dword [COUNTER_TA], ecx
			mov	dword [STATE], edx
			inc	dword ecx
			call	c64link_cia_counter_refresh
			GET_CYCLES
			mov	dword [TIMESTAMP_TA], eax
			mov	dword [TIMESTAMP_TA + 4], edx
			jmp	.end
.one_shot:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: whats the correct behavior ?
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			or	dword edx, TA_STOPPED
			mov	dword [STATE], edx
.end:			test	byte  [IRQ_MASK], 001h
			mov	byte  bl, [REGISTERS + ICR]
			jz	.no_irq
			or	byte  bl, 081h
			mov	byte  al, 001h
			mov	byte  [REGISTERS + ICR], bl
			jmp	dword [LINK_IRQ]
.no_irq:		or	byte  bl, 001h
			mov	byte  [REGISTERS + ICR], bl
			ret

write_crb:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: fix the behavior if the timer is already running
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			push	dword ebx
			push	dword ecx
			push	dword edx
			mov	dword edx, [STATE]
			mov	dword ecx, [COUNTER_TB]
			and	dword edx, (NOT32 - TB_FORCE_LOAD)
			test	byte  al, 010h
			jz	.no_force_load
			mov	byte  cl, [REGISTERS + TB_LO]
			and	byte  al, 0efh
			mov	byte  ch, [REGISTERS + TB_HI]
			or	dword edx, TB_FORCE_LOAD
			mov	dword [COUNTER_TB], ecx
			inc	dword ecx
.no_force_load:		mov	byte  [REGISTERS + CRB], al
			test	byte  al, 001h
			jz	.stop
			and	dword edx, (NOT32 - TB_LOCKED)
			test	byte  al, 060h
			jz	.count_phi2
			test	byte  al, 040h
			jnz	.cascaded
;!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: implement count cnt
;!!!!!!!!!!!!!!!!!!!!!!!!!
			mov	dword eax, [MODULID]
			xor	dword ebx, ebx
			mov	dword ecx, __LINE__
			jmp	core_exception
.stop:			call	read_timer_b
			sub	dword eax, 2
			or	dword edx, TB_STOPPED
			mov	dword ebx, eax
			mov	dword [STATE], edx
			and	dword ebx, NOT16
			mov	dword eax, [COUNTER_OFFSET]
			mov	dword [COUNTER_TB], ebx
			inc	dword eax
			call	c64link_cia_counter_clear
			jmp	.end
.cascaded:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: implement cascaded mode
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			mov	dword eax, [MODULID]
			xor	dword ebx, ebx
			mov	dword ecx, __LINE__
			jmp	core_exception
.count_phi2:		mov	dword eax, [COUNTER_OFFSET]
			mov	dword ebx, process_timer_b
			add	dword ecx, 2
			and	dword edx, (NOT32 - TB_STOPPED)
			inc	dword eax
			mov	dword [STATE], edx
			call	c64link_cia_counter_set
			GET_CYCLES
			mov	dword [TIMESTAMP_TB], eax
			mov	dword [TIMESTAMP_TB + 4], edx
.end:			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			ret

process_timer_b:
			mov	dword edx, [STATE]
			test	byte  [REGISTERS + CRB], 008h
			jnz	.one_shot
			xor	dword ecx, ecx
			mov	dword eax, [COUNTER_OFFSET]
			mov	byte  cl, [REGISTERS + TB_LO]
			mov	dword ebx, process_timer_b
			mov	byte  ch, [REGISTERS + TB_HI]
			and	dword edx, (NOT32 - TB_FORCE_LOAD)
			mov	dword [COUNTER_TB], ecx
			mov	dword [STATE], edx
			inc	dword ecx
			inc	dword eax
			call	c64link_cia_counter_refresh
			GET_CYCLES
			mov	dword [TIMESTAMP_TB], eax
			mov	dword [TIMESTAMP_TB + 4], edx
			jmp	.end
.one_shot:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: whats the correct behavior ?
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			or	dword edx, TB_STOPPED
			mov	dword [STATE], edx
.end:			test	byte  [IRQ_MASK], 002h
			mov	byte  bl, [REGISTERS + ICR]
			jz	.no_irq
			or	byte  bl, 082h
			mov	byte  al, 001h
			mov	byte  [REGISTERS + ICR], bl
			jmp	dword [LINK_IRQ]
.no_irq:		or	byte  bl, 002h
			mov	byte  [REGISTERS + ICR], bl
			ret
;******************************************************** ec64@unusedino.de *

