;****************************************************************************
;****************************************************************************
;*
;* core.n
;*
;****************************************************************************
;****************************************************************************





%define CORE_N
%include "core.i"



%define MODULID			CORE_MODULID
%define C64_CYCLES		CORE_C64_CYCLES
%define C1541_CYCLES		CORE_C1541_CYCLES
%define CYCLE_DIFFERENCE1	CORE_CYCLE_DIFFERENCE1
%define CYCLE_DIFFERENCE2	CORE_CYCLE_DIFFERENCE2
%define CYCLE_COUNTER		CORE_CYCLE_COUNTER
%define CYCLE_COUNTER_BASE	CORE_CYCLE_COUNTER_BASE
%define CYCLES_COUNTED		CORE_CYCLES_COUNTED
%define CYCLES_LEFT		CORE_CYCLES_LEFT

%define STACK_POINTER_EXTERN	CORE_STACK_POINTER_EXTERN
%define STACK_POINTER		CORE_STACK_POINTER
%define STACK			CORE_STACK

%define INFORMATION_MODULID	CORE_INFORMATION_MODULID
%define INFORMATION_TYPE	CORE_INFORMATION_TYPE
%define INFORMATION_NUMBER	CORE_INFORMATION_NUMBER

%define LOG_BUFFER		CORE_LOG_BUFFER
%define LOG_OFFSET		CORE_LOG_OFFSET
%define LOG_SIZE		CORE_LOG_SIZE

%define C64			CORE_C64
%define C1541			CORE_C1541



;****************************************************************************
;* some data ****************************************************************
;****************************************************************************
section .bss
tcore__1:			resb	tcore_size

tc64__1:			equ	C64
tc1541__1:			equ	C1541



;****************************************************************************
;* core_enter ***************************************************************
;****************************************************************************
section .text
core_enter:
			mov	dword [STACK_POINTER_EXTERN], esp
			mov	dword esp, [STACK_POINTER]
			pop	dword ebp
			pop	dword edi
			pop	dword esi
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* core_leave ***************************************************************
;****************************************************************************
section .text
core_leave:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			push	dword esi
			push	dword edi
			push	dword ebp
			mov	dword [STACK_POINTER], esp
			mov	dword esp, [STACK_POINTER_EXTERN]
			ret



;****************************************************************************
;* core_reset_hard **********************************************************
;****************************************************************************
section .text
core_reset_hard:
			;-------------------------------
			;initialize the local core stack
			;-------------------------------

			mov	dword [STACK_POINTER_EXTERN], esp
			lea	dword esp, [STACK + CORE_STACK_SIZE]

			;--------------------------
			;reset the emulated modules
			;--------------------------

			lea	dword esi, [C64]
			C64LINK_CALL_RESET_HARD
			C64MEMORY_CALL_RESET_HARD
			CPU65XX_CALL_RESET_HARD
			VIC6569_CALL_RESET_HARD
			SID6581_CALL_RESET_HARD
			CIA6526_CALL_RESET_HARD  tcia6526__c64_1
			CIA6526_CALL_RESET_HARD  tcia6526__c64_2
			lea	dword esi, [C1541]
			C1541LINK_CALL_RESET_HARD
			C1541MEMORY_CALL_RESET_HARD
			CPU65XX_CALL_RESET_HARD
			VIA6522_CALL_RESET_HARD  tvia6522__c1541_1
			VIA6522_CALL_RESET_HARD  tvia6522__c1541_2
			DISC_CALL_RESET_HARD
			xor	dword eax, eax
			call	core_leave
			jmp	core_cycle_loop



;****************************************************************************
;* core_reset_soft **********************************************************
;****************************************************************************
section .text
core_reset_soft:
			;-------------------------------
			;initialize the local core stack
			;-------------------------------

			mov	dword [STACK_POINTER_EXTERN], esp
			lea	dword esp, [STACK + CORE_STACK_SIZE]

			;--------------------------
			;reset the emulated modules
			;--------------------------

			lea	dword esi, [C64]
			C64LINK_CALL_RESET_SOFT
			C64MEMORY_CALL_RESET_SOFT
			CPU65XX_CALL_RESET_SOFT
			VIC6569_CALL_RESET_SOFT
			SID6581_CALL_RESET_SOFT
			CIA6526_CALL_RESET_SOFT  tcia6526__c64_1
			CIA6526_CALL_RESET_SOFT  tcia6526__c64_2
			lea	dword esi, [C1541]
			C1541LINK_CALL_RESET_SOFT
			C1541MEMORY_CALL_RESET_SOFT
			CPU65XX_CALL_RESET_SOFT
			VIA6522_CALL_RESET_SOFT  tvia6522__c1541_1
			VIA6522_CALL_RESET_SOFT  tvia6522__c1541_2
			DISC_CALL_RESET_SOFT
			xor	dword eax, eax
			call	core_leave
			jmp	core_cycle_loop



;****************************************************************************
;* core_initialize **********************************************************
;****************************************************************************
section .text
core_initialize:
			;-----------------------------
			;reset the core data structure
			;-----------------------------

			xor	dword eax, eax
			mov	dword ecx, tcore_size
			lea	dword edi, [MODULID]
			cld
			rep stosb

			;-------------------------------
			;initialize the local core stack
			;-------------------------------

			mov	dword [STACK_POINTER_EXTERN], esp
			lea	dword esp, [STACK + CORE_STACK_SIZE]

			;--------------------------------
			;initialize the emulation modules
			;--------------------------------

			C64LINK_CALL_INITIALIZE
			C1541LINK_CALL_INITIALIZE
			C64MEMORY_CALL_INITIALIZE
			C1541MEMORY_CALL_INITIALIZE
			CPU65XX_CALL_INITIALIZE
			VIC6569_CALL_INITIALIZE
			SID6581_CALL_INITIALIZE
			CIA6526_CALL_INITIALIZE
			VIA6522_CALL_INITIALIZE
			DISC_CALL_INITIALIZE

			;-------------------------------
			;initialize the module instances
			;-------------------------------

			lea	dword esi, [C64]
			C64LINK_CALL_INSTANCE_INITIALIZE  CORE_MODULID_C64LINK
			C64MEMORY_CALL_INSTANCE_INITIALIZE  CORE_MODULID_C64MEMORY
			CPU65XX_CALL_INSTANCE_INITIALIZE  CORE_MODULID_C64CPU, core_get_c64_cycles, c64memory_read_opcode, c64memory_read_stack, c64memory_read_data, c64memory_write_stack, c64memory_write_data
			VIC6569_CALL_INSTANCE_INITIALIZE  CORE_MODULID_C64VIC
			SID6581_CALL_INSTANCE_INITIALIZE  CORE_MODULID_C64SID
			CIA6526_CALL_INSTANCE_INITIALIZE  tcia6526__c64_1, CORE_MODULID_C64CIA1, C64LINK_CIA1_COUNTER_OFFSET, c64link_cia1_irq, c64link_cia1_pra, c64link_cia1_prb
			CIA6526_CALL_INSTANCE_INITIALIZE  tcia6526__c64_2, CORE_MODULID_C64CIA2, C64LINK_CIA2_COUNTER_OFFSET, c64link_cia2_nmi, c64link_cia2_pra, c64link_cia2_prb
			lea	dword esi, [C1541]
			C1541LINK_CALL_INSTANCE_INITIALIZE  CORE_MODULID_C1541LINK
			C1541MEMORY_CALL_INSTANCE_INITIALIZE  CORE_MODULID_C1541MEMORY
			CPU65XX_CALL_INSTANCE_INITIALIZE  CORE_MODULID_C1541CPU, core_get_c1541_cycles, c1541memory_read_opcode, c1541memory_read_stack, c1541memory_read_data, c1541memory_write_stack, c1541memory_write_data
			VIA6522_CALL_INSTANCE_INITIALIZE  tvia6522__c1541_1, CORE_MODULID_C1541VIA1, C1541LINK_VIA1_COUNTER_OFFSET, c1541link_via1_irq, c1541link_via1_pb, c1541link_via1_cb2, c1541link_via1_pa, c1541link_via1_ca2
			VIA6522_CALL_INSTANCE_INITIALIZE  tvia6522__c1541_2, CORE_MODULID_C1541VIA2, C1541LINK_VIA2_COUNTER_OFFSET, c1541link_via2_irq, c1541link_via2_pb, c1541link_via2_cb2, c1541link_via2_pa, c1541link_via2_ca2
			DISC_CALL_INSTANCE_INITIALIZE  CORE_MODULID_C1541DISC

			;--------------------
			;return to the caller
			;--------------------

			xor	dword eax, eax
			call	core_leave
			mov	dword esp, [STACK_POINTER_EXTERN]
			jmp	core_reset_hard



;****************************************************************************
;* core_exit ****************************************************************
;****************************************************************************
section .text
core_exit:
			ret



;****************************************************************************
;* core_new_frame ***********************************************************
;****************************************************************************
section .text
core_new_frame:
			xor	dword eax, eax
			jmp	core_leave



;****************************************************************************
;* core_emulate *************************************************************
;****************************************************************************
section .text
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: throw an exception if the core isn't initialized
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
core_emulate:		equ	core_enter



;****************************************************************************
;* core_cycle_loop **********************************************************
;****************************************************************************
section .text
core_cycle_loop:
			;-------------------------------------------------
			;if we jump in from the reset routines and we have
			;something emulated before, then we have to add
			;the CYCLE_COUNTER to C64_CYCLES and C1541_CYCLES,
			;otherwise we can get an exception from the VIC
			;in change_log_write, because of the missing
			;cycles.
			;If we would do this very often we may lose the
			;exact propotion over all C64 and C1541 emulated
			;cycles but this doesn't affect the emulation
			;itself, only the cycle counters, so it's ok for
			;now.
			;-------------------------------------------------

			mov	dword eax, [CYCLES_COUNTED]
			sub	dword eax, [CYCLE_COUNTER]
			add	dword [C64_CYCLES], eax
			adc	dword [C64_CYCLES + 4], 0
			add	dword [C1541_CYCLES], eax
			adc	dword [C1541_CYCLES + 4], 0

			;----------------------------
			;initialize the clockcounters
			;----------------------------

			mov	dword ebx, C1541_CLOCK
			mov	dword ecx, C64_CLOCK
			mov	dword eax, ebx
			xor	dword edx, edx
			sub	dword ebx, ecx
			mov	dword [CYCLE_DIFFERENCE1], ebx
			div	dword ebx
			mov	dword [CYCLE_COUNTER_BASE], eax
			mov	dword [CYCLE_DIFFERENCE2], edx
			mov	dword ecx, edx
			xor	dword edx, edx
			jmp	.calculate_cycles

.run_c64_cycle:		
			;-------------------
			;one c64 clock cycle
			;-------------------

			lea	dword esi, [C64]
			CPU65XX_CALL_CYCLE
			VIC6569_CALL_CYCLE
			SID6581_CALL_CYCLE
			C64LINK_CALL_DECREMENT_COUNTER

.run_c1541_cycle:
			;----------------------
			;one c1541 clock cycle
			;----------------------

			lea	dword esi, [C1541]
			CPU65XX_CALL_CYCLE
			C1541LINK_CALL_DECREMENT_COUNTER
			DISC_CALL_CYCLE
			dec	dword [CYCLE_COUNTER]
			jnz	near  .run_c64_cycle

			;-----------------------------------
			;add counted cycles to main counters
			;-----------------------------------

			mov	dword eax, [CYCLES_COUNTED]
			mov	dword ebx, [C1541_CYCLES]
			mov	dword ecx, [C1541_CYCLES + 4]
			add	dword ebx, eax
			mov	dword edx, [C64_CYCLES]
			adc	dword ecx, 0
			dec	dword eax
			mov	dword ebp, [C64_CYCLES + 4]
			add	dword edx, eax
			mov	dword [C1541_CYCLES], ebx
			mov	dword [C1541_CYCLES + 4], ecx
			mov	dword eax, [CYCLE_COUNTER_BASE]
			adc	dword ebp, 0
			mov	dword ebx, [CYCLE_DIFFERENCE1]
			mov	dword [C64_CYCLES], edx
			mov	dword ecx, [CYCLE_DIFFERENCE2]
			mov	dword edx, [CYCLES_LEFT]
			mov	dword [C64_CYCLES + 4], ebp

			;----------------------------------
			;calculate cycles for CYCLE_COUNTER
			;----------------------------------

.calculate_cycles:	add	dword edx, ecx
			cmp	dword edx, ebx
			jb	.no_counter_increase
			inc	dword eax
			sub	dword edx, ebx
.no_counter_increase:	mov	dword [CYCLES_COUNTED], eax
			mov	dword [CYCLE_COUNTER], eax
			mov	dword [CYCLES_LEFT], edx
			jmp	.run_c1541_cycle



;****************************************************************************
;* core_exception ***********************************************************
;****************************************************************************
;* eax=>  modulid
;* ebx=>  type
;* ecx=>  number
;****************************************************************************
section .text
core_exception:
			mov	dword [INFORMATION_MODULID], eax
			mov	dword [INFORMATION_TYPE], ebx
			mov	dword [INFORMATION_NUMBER], ecx
			mov	dword eax, -CORE_ERROR_EXCEPTION
			call	core_leave
			mov	dword esp, [STACK_POINTER_EXTERN]
			jmp	core_initialize



;****************************************************************************
;* core_assertion ***********************************************************
;****************************************************************************
;* eax=>  modulid
;* ebx=>  type
;* ecx=>  number
;****************************************************************************
section .text
core_assertion:
			mov	dword [INFORMATION_MODULID], eax
			mov	dword [INFORMATION_TYPE], ebx
			mov	dword [INFORMATION_NUMBER], ecx
			mov	dword eax, -CORE_ERROR_ASSERTION
			call	core_leave
			mov	dword esp, [STACK_POINTER_EXTERN]
			jmp	core_initialize



;****************************************************************************
;* core_trap ****************************************************************
;****************************************************************************
;* eax=>  modulid
;* ebx=>  type
;* ecx=>  number
;****************************************************************************
section .text
core_trap:
			mov	dword [INFORMATION_MODULID], eax
			mov	dword [INFORMATION_TYPE], ebx
			mov	dword [INFORMATION_NUMBER], ecx
			mov	dword eax, -CORE_ERROR_TRAP
			jmp	core_leave



;****************************************************************************
;* core_log *****************************************************************
;****************************************************************************
;* eax=>  modul id
;* ebx=>  breakpoint type
;* ecx=>  breakpoint number
;* edx=>  data !! QUICK HACK FOR CPU !!
;****************************************************************************
section .text
core_log:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: remove this QUICK HACK for saving cpu log
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			push	dword esi
			push	dword edi
			mov	dword esi, [LOG_BUFFER]
			mov	dword edi, [LOG_OFFSET]
			test	dword esi, esi
			jz	.no_log
			mov	dword edi, [edi]
			call	.write
			mov	dword eax, ebx
			call	.write
			mov	dword eax, ecx
			call	.write
			mov	dword eax, edx
			call	.write
			call	core_get_c64_cycles
			call	.write
			mov	dword eax, edx
			call	.write
			call	core_get_c1541_cycles
			call	.write
			mov	dword eax, edx
			mov	dword ebx, [LOG_OFFSET]
			call	.write
			mov	dword [ebx], edi
.no_log:		pop	dword edi
			pop	dword esi
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret

.write:			mov	dword [esi + edi], eax
			add	dword edi, 4
			cmp	dword edi, [LOG_SIZE]
			jb	.write_end
			mov	dword eax, -CORE_ERROR_BPLOG_FULL
			push	dword ebx
			mov	dword ebx, [LOG_OFFSET]
			mov	dword [ebx], edi
			pop	dword ebx
			call	core_leave
			xor	dword edi, edi
.write_end:		ret



;****************************************************************************
;* core_get_c64_cycles ******************************************************
;****************************************************************************
;* <=eax  low 32 Bits of cycle counter
;* <=edx  high 32 Bits of cycle counter
;****************************************************************************
section .text
core_get_c64_cycles:
			mov	dword eax, [CYCLES_COUNTED]
			sub	dword eax, [CYCLE_COUNTER]
			jz	.no_decrease
			dec	dword eax
.no_decrease:		mov	dword edx, [C64_CYCLES + 4]
			add	dword eax, [C64_CYCLES]
			adc	dword edx, 0
			ret



;****************************************************************************
;* core_get_c1541_cycles ****************************************************
;****************************************************************************
;* <=eax  low 32 Bits of cycle counter
;* <=edx  high 32 Bits of cycle counter
;****************************************************************************
section .text
core_get_c1541_cycles:
			mov	dword eax, [CYCLES_COUNTED]
			sub	dword eax, [CYCLE_COUNTER]
			mov	dword edx, [C1541_CYCLES + 4]
			add	dword eax, [C1541_CYCLES]
			adc	dword edx, 0
			ret



%ifndef CORE_ENABLE_BP
;****************************************************************************
;* core_bp_reject ***********************************************************
;****************************************************************************
section .text
core_bp_reject:
			mov	dword eax, -CORE_ERROR_NOT_IMPLEMENTED
			ret

core_bp_entries:	equ	core_bp_reject
core_bp_get:		equ	core_bp_reject
core_bp_add:		equ	core_bp_reject
core_bp_delete:		equ	core_bp_reject



%else
;****************************************************************************
;* MODULID_SELECTOR *********************************************************
;****************************************************************************
%macro MODULID_SELECTOR 1
			lea	dword esi, [C64]
			mov	dword edx, cpu65xx_%1
			cmp	dword eax, CORE_MODULID_C64CPU
			je	%%call
			lea	dword esi, [C1541]
			mov	dword edx, cpu65xx_%1
			cmp	dword eax, CORE_MODULID_C1541CPU
			je	%%call
			mov	dword eax, -CORE_ERROR_NOT_IMPLEMENTED
			ret
%%call:			mov	dword eax, ebx
			mov	dword ebx, ecx
			jmp	dword edx
%endmacro



;****************************************************************************
;* core_bp_entries **********************************************************
;****************************************************************************
;* eax=>  modul id
;* ebx=>  breakpoint type
;* ecx=>  pointer to dword variable
;****************************************************************************
section .text
core_bp_entries:
			MODULID_SELECTOR  bp_entries



;****************************************************************************
;* core_bp_get **************************************************************
;****************************************************************************
;* eax=>  modul id
;* ebx=>  breakpoint type
;* ecx=>  pointer to data structure
;****************************************************************************
section .text
core_bp_get:
			MODULID_SELECTOR  bp_get



;****************************************************************************
;* core_bp_add **************************************************************
;****************************************************************************
;* eax=>  modul id
;* ebx=>  breakpoint type
;* ecx=>  pointer to data structure
;****************************************************************************
section .text
core_bp_add:
			MODULID_SELECTOR  bp_add



;****************************************************************************
;* core_bp_delete ***********************************************************
;****************************************************************************
;* eax=>  modul id
;* ebx=>  breakpoint type
;* ecx=>  breakpoint number
;****************************************************************************
section .text
core_bp_delete:
			MODULID_SELECTOR  bp_delete
%endif



;****************************************************************************
;* core_get_information *****************************************************
;****************************************************************************
;* eax=>  pointer to tcoreinfo structure
;* <=eax  return status
;****************************************************************************
section .text
core_get_information:
			mov	dword ebx, [INFORMATION_MODULID]
			mov	dword ecx, [INFORMATION_TYPE]
			mov	dword edx, [INFORMATION_NUMBER]
			mov	dword [eax + d__coreinfo_modulid], ebx
			mov	dword [eax + d__coreinfo_type], ecx
			mov	dword [eax + d__coreinfo_number], edx
			xor	dword eax, eax
			ret



;****************************************************************************
;* core_set_log_options *****************************************************
;****************************************************************************
;* eax=>  pointer to log buffer
;* ebx=>  point to log offset variable
;* ecx=>  log size
;* <=eax  return status
;****************************************************************************
section .text
core_set_log_options:
			mov	dword [LOG_BUFFER], eax
			mov	dword [LOG_OFFSET], ebx
			and	byte  cl, 0fch
			mov	dword [LOG_SIZE], ecx
			xor	dword eax, eax
			ret
;******************************************************** ec64@unusedino.de *

