;****************************************************************************
;****************************************************************************
;*
;* disc.n
;*
;****************************************************************************
;****************************************************************************


;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO:
;* allow SYNC's also on non byte aligned positions
;* read rubbish, if the read speed differs from the write speed
;* include the mechanic characteristics of the step- and drivemotor 
;  [delaycounter]
;* read rubbish if drivemotor is turned off
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!





%define DISC_N
%include "disc.i"
%include "link.i"
%include "c1541link.i"
%include "core.i"



;****************************************************************************
;* assign's & defines's *****************************************************
;****************************************************************************
%define MODULID			DISC_MODULID
%define STATE			DISC_STATE
%define CYCLES_PER_TRACK	DISC_CYCLES_PER_TRACK
%define BYTES_PER_TRACK		DISC_BYTES_PER_TRACK
%define CYCLE_COUNTER		DISC_CYCLE_COUNTER
%define TRACK_OFFSET		DISC_TRACK_OFFSET
%define SPEED_TABLE		DISC_SPEED_TABLE
%define LOAD_D64		DISC_LOAD_D64
%define TRACK			DISC_TRACK
%define DATA_BYTE		DISC_DATA_BYTE
%define LAST_BYTE		DISC_LAST_BYTE
%define HEAD_POSITION		DISC_HEAD_POSITION
%define GCR_IMAGE		DISC_GCR_IMAGE

%assign DATA_RW_MODE		001h
%assign NO_BYTE_READY		002h
%assign SYNC_FOUND		004h



;****************************************************************************
;* disc_initialize **********************************************************
;****************************************************************************
section .text
disc_initialize:
			ret



;****************************************************************************
;* disc_exit ****************************************************************
;****************************************************************************
section .text
disc_exit:
			ret



;****************************************************************************
;* disc_instance_initialize *************************************************
;****************************************************************************
section .text
disc_instance_initialize:
			INSTANCE_INITIALIZE  tdisc_size



;****************************************************************************
;* disc_instance_exit *******************************************************
;****************************************************************************
section .text
disc_instance_exit:
			ret



;****************************************************************************
;* disc_reset_hard **********************************************************
;****************************************************************************
section .text
disc_reset_hard:
			jmp	disc_reset_soft



;****************************************************************************
;* disc_reset_soft **********************************************************
;****************************************************************************
section .text
disc_reset_soft:
			push	dword eax
			push	dword ebx
			push	dword ecx
			mov	dword eax, C1541_CLOCK / 5	;300rpm / 60 seconds = 5
			mov	dword ebx, DISC_GCR_SPEED0_BYTES
			mov	dword ecx, DISC_GCR_SPEED1_BYTES
			mov	dword [CYCLES_PER_TRACK], eax
			mov	dword [SPEED_TABLE], ebx
			mov	dword [BYTES_PER_TRACK], ebx
			mov	dword [SPEED_TABLE + 4], ecx
			xor	dword eax, eax
			mov	dword ebx, DISC_GCR_SPEED2_BYTES
			mov	dword ecx, DISC_GCR_SPEED3_BYTES
			mov	dword [SPEED_TABLE + 8], ebx
			mov	dword [SPEED_TABLE + 12], ecx
			mov	dword [STATE], eax
			mov	dword [CYCLE_COUNTER], eax
			mov	dword [TRACK_OFFSET], eax
			mov	byte  [DATA_BYTE], al
			mov	byte  [LAST_BYTE], al
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* disc_signal_data *********************************************************
;****************************************************************************
section .text
disc_signal_data:
			mov	byte  [DATA_BYTE], al
			ret



;****************************************************************************
;* disc_signal_rw ***********************************************************
;****************************************************************************
section .text
disc_signal_rw:
			test	byte  al, 001h
			jnz	.read
.write:			or	dword [STATE], DATA_RW_MODE
			ret
.read:			and	dword [STATE], (NOT32 - DATA_RW_MODE)
			ret



;****************************************************************************
;* disc_signal_byte_ready ***************************************************
;****************************************************************************
section .text
disc_signal_byte_ready:
			test	byte  al, 001h
			jz	.disable
.enable:		and	dword [STATE], (NOT32 - NO_BYTE_READY)
			ret
.disable:		or	dword [STATE], NO_BYTE_READY
			ret



;****************************************************************************
;* disc_signal_head *********************************************************
;****************************************************************************
section .text
disc_signal_head:
			push	dword eax
			and	byte  al, 3
			mov	byte  ah, [HEAD_POSITION]
			inc	byte  ah
			and	byte  ah, 3
.compare_1:		cmp	byte  al, ah
			jne	.compare_2
			mov	byte  ah, [TRACK]
			cmp	byte  ah, DISC_GCR_LAST_TRACK
			je	.end
			inc	byte  ah
			mov	byte  [TRACK], ah
			jmp	.end
.compare_2:		sub	byte  ah, 2
			and	byte  ah, 3
			cmp	byte  al, ah
			jne	.end
			mov	byte  ah, [TRACK]
			cmp	byte  ah, 0
			je	.end
			dec	byte  ah
			mov	byte  [TRACK], ah
.end:			mov	byte  [HEAD_POSITION], al
			pop	dword eax
			ret



;****************************************************************************
;* disc_signal_speed ********************************************************
;****************************************************************************
section .text
disc_signal_speed:
			push	dword eax
			and	dword eax, 3
			mov	dword eax, [SPEED_TABLE + 4 * eax]
			mov	dword [BYTES_PER_TRACK], eax
			pop	dword eax
			ret



;****************************************************************************
;* disc_load_gcr ************************************************************
;****************************************************************************
;* eax=>  pointer to gcr data
;****************************************************************************
section .text
disc_load_gcr:
			push	dword ecx
			push	dword esi
			push	dword edi
			lea	dword edi, [GCR_IMAGE]
			mov	dword esi, eax
			mov	dword ecx, (DISC_GCR_IMAGE_SIZE / 4)
			cld
			rep movsd
			pop	dword edi
			pop	dword esi
			pop	dword ecx
			ret



;****************************************************************************
;* disc_cycle ***************************************************************
;****************************************************************************
section .text
disc_cycle:
			mov	dword eax, [STATE]
			test	dword eax, (NO_BYTE_READY | SYNC_FOUND)
			jnz	.no_byte_ready
			call	cpu65xx_signal_set_overflow
.no_byte_ready:		xor	dword ebx, ebx
			mov	dword ecx, [TRACK_OFFSET]
			mov	byte  bl, [TRACK]
			inc	dword ecx
			shl	dword ebx, 13		;TRACK * 02000h Bytes
			cmp	dword ecx, [BYTES_PER_TRACK]
			jb	.no_clear
			xor	dword ecx, ecx
.no_clear:		add	dword ebx, ecx
			mov	dword [TRACK_OFFSET], ecx
			test	dword eax, DATA_RW_MODE
			jz	.read
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: write one byte ahead, so that a endless SYNC stream would be broken
;      at least once in a rotation
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
.write:			and	dword eax, (NOT32 - SYNC_FOUND)
			mov	dword [STATE], eax
			mov	byte  al, [DATA_BYTE]
			mov	byte  [LAST_BYTE], al
			xor	byte  [GCR_IMAGE + ebx], al
			mov	byte  [GCR_IMAGE + DISC_GCR_TRACK_SIZE + ebx], al
			xor	byte  [GCR_IMAGE + 2 * DISC_GCR_TRACK_SIZE + ebx], al
			mov	byte  al, 1
			call	c1541link_disc_sync
			ret
.read:			and	dword eax, (NOT32 - SYNC_FOUND)
			mov	byte  bl, [GCR_IMAGE + DISC_GCR_TRACK_SIZE + ebx]
			mov	byte  cl, 1
			cmp	byte  bl, 0ffh
			jne	.read_1
			cmp	byte  [LAST_BYTE], 0ffh
			jne	.read_1
			or	dword eax, SYNC_FOUND
			xor	byte  cl, cl
.read_1:		mov	dword [STATE], eax
			mov	byte  al, [LAST_BYTE]
			mov	byte  [LAST_BYTE], bl
			call	c1541link_disc_data
			mov	byte  al, cl
			call	c1541link_disc_sync
			ret
;******************************************************** ec64@unusedino.de *

