;****************************************************************************
;****************************************************************************
;*
;* ec64core.n
;*
;****************************************************************************
;****************************************************************************





%define EC64CORE_N
%include "ec64core.i"
%include "core.i"
%include "c64link.i"
%include "c1541link.i"



;****************************************************************************
;* PUSH_REGISTERS ***********************************************************
;****************************************************************************
%macro PUSH_REGISTERS 0
			push	dword ebx
			push	dword ecx
			push	dword edx
			push	dword esi
			push	dword edi
			push	dword ebp
%endmacro



;****************************************************************************
;* POP_REGISTERS ************************************************************
;****************************************************************************
%macro POP_REGISTERS 0
			pop	dword ebp
			pop	dword edi
			pop	dword esi
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
%endmacro



;****************************************************************************
;* ec64core_get_version *****************************************************
;****************************************************************************
%ifdef EC64_ENABLE_BP_SUPPORT
%define EC64CORE_BP	" +bp"
%else
%define EC64CORE_BP	""
%endif

section .text
ec64core_get_version:
			mov	dword eax, .text
			ret

.text:			db	"0.2 (noisy snail)", EC64CORE_BP, 0



;****************************************************************************
;* ec64core_initialize ******************************************************
;****************************************************************************
section .text
ec64core_initialize:
			push	dword eax
			PUSH_REGISTERS
			call	core_initialize
			POP_REGISTERS
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_reset_hard ******************************************************
;****************************************************************************
section .text
ec64core_reset_hard:
			push	dword eax
			PUSH_REGISTERS
			call	core_reset_hard
			POP_REGISTERS
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_reset_soft ******************************************************
;****************************************************************************
section .text
ec64core_reset_soft:
			push	dword eax
			PUSH_REGISTERS
			call	core_reset_soft
			POP_REGISTERS
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_emulate_pal_frame ***********************************************
;****************************************************************************
section .text
ec64core_emulate_pal_frame:
			PUSH_REGISTERS
			call	core_emulate
			POP_REGISTERS
			ret



;****************************************************************************
;* ec64core_c64_set_video_options *******************************************
;****************************************************************************
;* [1]=>  pointer to tvideo struct
;****************************************************************************
section .text
ec64core_c64_set_video_options:
			push	dword eax
			PUSH_REGISTERS
			mov	dword ebp, [esp + 32]
			mov	dword ecx, [ebp + d__video_c64_x_resolution]
			mov	dword edx, [ebp + d__video_c64_y_resolution]
			mov	dword eax, [ebp + d__video_c64_x_offset]
			mov	dword ebx, [ebp + d__video_c64_y_offset]
			mov	dword esi, tc64__1
			mov	dword edi, [ebp + p__video_buffer]
			push	dword [ebp + p__video_decoded_lines_list]
			mov	dword ebp, [ebp + d__video_virtual_x_resolution]
			call	vic6569_screen_initialize
			add	dword esp, 4
			POP_REGISTERS
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_c64_set_audio_options *******************************************
;****************************************************************************
;* [1]=>  pointer to taudio struct
;****************************************************************************
section .text
ec64core_c64_set_audio_options:
			push	dword eax
			PUSH_REGISTERS
			mov	dword ebp, [esp + 32]
			mov	dword eax, [ebp + d__audio_buffer_size]
			mov	dword ebx, [ebp + p__audio_buffer]
			mov	dword ecx, [ebp + p__audio_buffer_offset]
			mov	dword edx, [ebp + d__audio_sample_rate]
			mov	dword esi, tc64__1
			call	sid6581_buffer_initialize
			POP_REGISTERS
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_c64_set_keyboard ************************************************
;****************************************************************************
;* [1]=>  pointer to c64 keys
;****************************************************************************
section .text
ec64core_c64_set_keyboard:
			push	dword eax
			push	dword esi
			mov	dword eax, [esp + 12]
			mov	dword esi, tc64__1
			call	c64link_keyboard
			pop	dword esi
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_c64_set_joystick1 ***********************************************
;****************************************************************************
;* [1]=>  joystick bits
;****************************************************************************
section .text
ec64core_c64_set_joystick1:
			push	dword eax
			push	dword esi
			mov	dword eax, [esp + 12]
			mov	dword esi, tc64__1
			call	c64link_joystick1
			pop	dword esi
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_c64_set_joystick2 ***********************************************
;****************************************************************************
;* [1]=>  joystick bits
;****************************************************************************
section .text
ec64core_c64_set_joystick2:
			push	dword eax
			push	dword esi
			mov	dword eax, [esp + 12]
			mov	dword esi, tc64__1
			call	c64link_joystick2
			pop	dword esi
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_c64_set_basic_rom ***********************************************
;****************************************************************************
;* [1]=>  pointer to basic rom
;****************************************************************************
section .text
ec64core_c64_set_basic_rom:
			push	dword eax
			PUSH_REGISTERS
			mov	dword eax, [esp + 32]
			mov	dword esi, tc64__1
			call	c64memory_load_rom_basic
			POP_REGISTERS
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_c64_set_character_rom *******************************************
;****************************************************************************
;* [1]=>  ponter to character rom
;****************************************************************************
section .text
ec64core_c64_set_character_rom:
			push	dword eax
			PUSH_REGISTERS
			mov	dword eax, [esp + 32]
			mov	dword esi, tc64__1
			call	c64memory_load_rom_character
			POP_REGISTERS
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_c64_set_kernel_rom **********************************************
;****************************************************************************
;* [1]=>  pointer to kernel rom
;****************************************************************************
section .text
ec64core_c64_set_kernel_rom:
			push	dword eax
			PUSH_REGISTERS
			mov	dword eax, [esp + 32]
			mov	dword esi, tc64__1
			call	c64memory_load_rom_kernel
			POP_REGISTERS
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_c64_get_status **************************************************
;****************************************************************************
;* <=eax  pointer to tc64stat struct
;****************************************************************************
section .text
ec64core_c64_get_status:
			PUSH_REGISTERS
			call	core_get_c64_cycles
			mov	dword [.tc64stat__status + tcycles__c64stat + d__cycles_low], eax
			mov	dword [.tc64stat__status + tcycles__c64stat + d__cycles_high], edx
			mov	dword eax, .tc64stat__status
			POP_REGISTERS			
			ret

section .bss
			align	4
.tc64stat__status:	resb	tc64stat_size



;****************************************************************************
;* ec64core_c1541_get_disc_image ********************************************
;****************************************************************************
;* <=eax  address of gcr image
;****************************************************************************
section .text
ec64core_c1541_get_disc_image:
			PUSH_REGISTERS
			mov	dword esi, tc1541__1
			lea	dword eax, [DISC_GCR_IMAGE]
			POP_REGISTERS
			ret



;****************************************************************************
;* ec64core_c1541_set_disc_image ********************************************
;****************************************************************************
;* [1]=>  address of gcr image
;****************************************************************************
section .text
ec64core_c1541_set_disc_image:
			push	dword eax
			PUSH_REGISTERS
			mov	dword eax, [esp + 32]
			mov	dword esi, tc1541__1
			call	disc_load_gcr
			POP_REGISTERS
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_c1541_set_kernel_rom ********************************************
;****************************************************************************
;* [1]=>  pointer to kernel rom
;****************************************************************************
section .text
ec64core_c1541_set_kernel_rom:
			push	dword eax
			PUSH_REGISTERS
			mov	dword eax, [esp + 32]
			mov	dword esi, tc1541__1
			call	c1541memory_load_rom_kernel
			POP_REGISTERS
			pop	dword eax
			ret



;****************************************************************************
;* ec64core_c1541_get_status ************************************************
;****************************************************************************
;* <=eax  pointer to tc1541stat
;****************************************************************************
section .text
ec64core_c1541_get_status:
			PUSH_REGISTERS
			call	core_get_c1541_cycles
			mov	dword [.tc1541stat__status + tcycles__c1541stat + d__cycles_low], eax
			mov	dword [.tc1541stat__status + tcycles__c1541stat + d__cycles_high], edx
			xor	dword eax, eax
			mov	dword esi, tc1541__1
			lea	dword edi, [esi + tvia6522__c1541_2]
			mov	byte  al, [DISC_TRACK]
			mov	byte  [.tc1541stat__status + b__c1541stat_track], al
			mov	byte  al, [VIA6522_LATCH_PB]
			mov	byte  bl, [VIA6522_REGISTERS + VIA6522_DDRB]
			and	byte  al, bl
			xor	byte  bl, NOT8
			or	byte  al, bl
			shr	byte  al, 3
			and	byte  al, 1
			mov	byte  [.tc1541stat__status + b__c1541stat_led], al
			mov	byte  al, [VIA6522_LATCH_PB]
			mov	byte  bl, [VIA6522_REGISTERS + VIA6522_DDRB]
			and	byte  al, bl
			xor	byte  bl, NOT8
			or	byte  al, bl
			shr	byte  al, 2
			and	byte  al, 1
			mov	byte  [.tc1541stat__status + b__c1541stat_motor], al
			mov	dword eax, .tc1541stat__status
			POP_REGISTERS
			ret

section .bss
			align	4
.tc1541stat__status:	resb	tc1541stat_size



;****************************************************************************
;* ec64core_bp_entries ******************************************************
;****************************************************************************
;* [1]=>  modul id
;* [2]=>  breakpoint type
;* [3]=>  pointer to dword variable
;* <=eax  return status
;****************************************************************************
section .text
ec64core_bp_entries:
			PUSH_REGISTERS
			mov	dword eax, [esp + 28]
			mov	dword ebx, [esp + 32]
			mov	dword ecx, [esp + 36]
			call	core_bp_entries
			POP_REGISTERS
			ret



;****************************************************************************
;* ec64core_bp_get **********************************************************
;****************************************************************************
;* [1]=>  modul id
;* [2]=>  breakpoint type
;* [3]=>  pointer to data structure
;* <=eax  return status
;****************************************************************************
section .text
ec64core_bp_get:
			PUSH_REGISTERS
			mov	dword eax, [esp + 28]
			mov	dword ebx, [esp + 32]
			mov	dword ecx, [esp + 36]
			call	core_bp_get
			POP_REGISTERS
			ret



;****************************************************************************
;* ec64core_bp_add **********************************************************
;****************************************************************************
;* [1]=>  modul id
;* [2]=>  breakpoint type
;* [3]=>  pointer to data structure
;* <=eax  return status
;****************************************************************************
section .text
ec64core_bp_add:
			PUSH_REGISTERS
			mov	dword eax, [esp + 28]
			mov	dword ebx, [esp + 32]
			mov	dword ecx, [esp + 36]
			call	core_bp_add
			POP_REGISTERS
			ret



;****************************************************************************
;* ec64core_bp_delete *******************************************************
;****************************************************************************
;* [1]=>  modul id
;* [2]=>  breakpoint type
;* [3]=>  breakpoint number
;* <=eax  return status
;****************************************************************************
section .text
ec64core_bp_delete:
			PUSH_REGISTERS
			mov	dword eax, [esp + 28]
			mov	dword ebx, [esp + 32]
			mov	dword ecx, [esp + 36]
			call	core_bp_delete
			POP_REGISTERS
			ret



;****************************************************************************
;* ec64core_get_information *************************************************
;****************************************************************************
;* [1]=>  pointer to tcoreinfo structure
;* <=eax  return status
;****************************************************************************
section .text
ec64core_get_information:
			PUSH_REGISTERS
			mov	dword eax, [esp + 28]
			call	core_get_information
			POP_REGISTERS
			ret



;****************************************************************************
;* ec64core_set_log_options *************************************************
;****************************************************************************
;* [1]=>  pointer to log buffer
;* [2]=>  pointer to log offset variable
;* [3]=>  log size
;* <=eax  return status
;****************************************************************************
section .text
ec64core_set_log_options:
			PUSH_REGISTERS
			mov	dword eax, [esp + 28]
			mov	dword ebx, [esp + 32]
			mov	dword ecx, [esp + 36]
			call	core_set_log_options
			POP_REGISTERS
			ret
;******************************************************** ec64@unusedino.de *

