;****************************************************************************
;****************************************************************************
;*
;* sid6581.n
;*
;****************************************************************************
;****************************************************************************





%define SID6581_N
%include "sid6581.i"
%include "c64link.i"
%include "core.i"





;****************************************************************************
;****************************************************************************
;*
;* PART 1: assign's & define's
;*
;****************************************************************************
;****************************************************************************





%define MODULID		SID6581_MODULID
%define SAMPLE_RATE	SID6581_SAMPLE_RATE
%define BUFFER		SID6581_BUFFER
%define BUFFER_SIZE	SID6581_BUFFER_SIZE
%define BUFFER_OFFSET	SID6581_BUFFER_OFFSET
%define SAMPLE_COUNTER	SID6581_SAMPLE_COUNTER
%define TIMESTAMP	SID6581_TIMESTAMP
%define VOICE1		SID6581_VOICE1
%define VOICE2		SID6581_VOICE2
%define VOICE3		SID6581_VOICE3
%define VOLUME		SID6581_VOLUME





;****************************************************************************
;****************************************************************************
;*
;* PART 2: Initialisation & Management Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* sid6581_initialize *******************************************************
;****************************************************************************
section .text
sid6581_initialize:
			ret



;****************************************************************************
;* sid6581_exit *************************************************************
;****************************************************************************
section .text
sid6581_exit:
			ret



;****************************************************************************
;* sid6581_instance_initialize **********************************************
;****************************************************************************
section .text
sid6581_instance_initialize:
			INSTANCE_INITIALIZE  tsid6581_size



;****************************************************************************
;* sid6581_instance_exit ****************************************************
;****************************************************************************
section .text
sid6581_instance_exit:
			ret



;****************************************************************************
;* sid6581_buffer_initialize ************************************************
;****************************************************************************
;* eax=>  buffer size
;* ebx=>  pointer to buffer
;* ecx=>  pointer to buffer offset
;* edx=>  sample rate
;****************************************************************************
section .text
sid6581_buffer_initialize:
			mov	dword [BUFFER_SIZE], eax
			mov	dword [BUFFER], ebx
			mov	dword [BUFFER_OFFSET], ecx
			mov	dword [SAMPLE_RATE], edx
			ret





;****************************************************************************
;****************************************************************************
;*
;* PART 3: Signal Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* sid6581_reset_hard *******************************************************
;****************************************************************************
section .text
sid6581_reset_hard:
			jmp	sid6581_reset_soft



;****************************************************************************
;* sid6581_reset_soft *******************************************************
;****************************************************************************
section .text
sid6581_reset_soft:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			push	dword edi

			push	dword [MODULID]
			push	dword [SAMPLE_RATE]
			push	dword [BUFFER]
			push	dword [BUFFER_SIZE]
			push	dword [BUFFER_OFFSET]
			xor	dword eax, eax
			mov	dword ecx, tsid6581_size
			lea	dword edi, [MODULID]
			cld
			rep stosb
			pop	dword [BUFFER_OFFSET]
			pop	dword [BUFFER_SIZE]
			pop	dword [BUFFER]
			pop	dword [SAMPLE_RATE]
			pop	dword [MODULID]
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: if [BUFFER] or [BUFFER_OFFSET] are zero set them to a default
;      location otherwise the core causes a segfault if sid6581_cycle
;      is called
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

			mov	dword eax, 0007ffff8h
			mov	dword [VOICE1 + SIDVOICE_NOISE_REGISTER], eax
			mov	dword [VOICE2 + SIDVOICE_NOISE_REGISTER], eax
			mov	dword [VOICE3 + SIDVOICE_NOISE_REGISTER], eax

			pop	dword edi
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret





;****************************************************************************
;****************************************************************************
;*
;* PART 4: Emulation Code
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* sid6581_read *************************************************************
;****************************************************************************
section .text
sid6581_read:
			xor	byte  al, al
			ret



;****************************************************************************
;* sid6581_write ************************************************************
;****************************************************************************
section .text
sid6581_write:
			mov	dword ebp, ebx
			and	dword ebp, 00000001fh
			jmp	dword [.ap__registers + 4 * ebp]

.ap__registers:		dd	write_voice1_freq_lo
			dd	write_voice1_freq_hi
			dd	write_voice1_pw_lo
			dd	write_voice1_pw_hi
			dd	write_voice1_control
			dd	write_voice1_ad
			dd	write_voice1_sr
			dd	write_voice2_freq_lo
			dd	write_voice2_freq_hi
			dd	write_voice2_pw_lo
			dd	write_voice2_pw_hi
			dd	write_voice2_control
			dd	write_voice2_ad
			dd	write_voice2_sr
			dd	write_voice3_freq_lo
			dd	write_voice3_freq_hi
			dd	write_voice3_pw_lo
			dd	write_voice3_pw_hi
			dd	write_voice3_control
			dd	write_voice3_ad
			dd	write_voice3_sr
			dd	write_filter_lo
			dd	write_filter_hi
			dd	write_filter_control
			dd	write_volume
			dd	write_ignore
			dd	write_ignore
			dd	write_ignore
			dd	write_ignore
			dd	write_ignore
			dd	write_ignore
			dd	write_ignore

write_ignore:
			ret

write_voice1_freq_lo:
			mov	byte  [VOICE1 + SIDVOICE_FREQUENCY], al
			ret

write_voice1_freq_hi:
			mov	byte  [VOICE1 + SIDVOICE_FREQUENCY + 1], al
			ret

write_voice1_pw_lo:
			mov	byte  [VOICE1 + SIDVOICE_PULSE_WIDTH], al
			ret

write_voice1_pw_hi:
			and	byte  al, 00fh
			mov	byte  [VOICE1 + SIDVOICE_PULSE_WIDTH + 1], al
			ret

write_voice1_control:
			mov	byte  ah, [VOICE1 + SIDVOICE_CONTROL]
			mov	byte  [VOICE1 + SIDVOICE_CONTROL], al
			xor	byte  ah, al
			test	byte  ah, 1
			jz	.end
			push	dword edx
			call	core_get_c64_cycles
			mov	dword [VOICE1 + SIDVOICE_TIMESTAMP_ENVELOPE], eax
			mov	dword [VOICE1 + SIDVOICE_TIMESTAMP_ENVELOPE + 4], edx
			pop	dword edx
.end:			ret

write_voice1_ad:
			mov	byte  ah, al
			shr	byte  al, 4
			mov	byte  [VOICE1 + SIDVOICE_ATTACK], al
			and	byte  ah, 00fh
			mov	byte  [VOICE1 + SIDVOICE_DECAY], ah
			ret

write_voice1_sr:
			mov	byte  ah, al
			shr	byte  al, 4
			mov	byte  [VOICE1 + SIDVOICE_SUSTAIN], al
			and	byte  ah, 00fh
			mov	byte  [VOICE1 + SIDVOICE_RELEASE], ah
			ret

write_voice2_freq_lo:
			mov	byte  [VOICE2 + SIDVOICE_FREQUENCY], al
			ret

write_voice2_freq_hi:
			mov	byte  [VOICE2 + SIDVOICE_FREQUENCY + 1], al
			ret

write_voice2_pw_lo:
			mov	byte  [VOICE2 + SIDVOICE_PULSE_WIDTH], al
			ret

write_voice2_pw_hi:
			and	byte  al, 00fh
			mov	byte  [VOICE2 + SIDVOICE_PULSE_WIDTH + 1], al
			ret

write_voice2_control:
			mov	byte  ah, [VOICE2 + SIDVOICE_CONTROL]
			mov	byte  [VOICE2 + SIDVOICE_CONTROL], al
			xor	byte  ah, al
			test	byte  ah, 1
			jz	.end
			push	dword edx
			call	core_get_c64_cycles
			mov	dword [VOICE2 + SIDVOICE_TIMESTAMP_ENVELOPE], eax
			mov	dword [VOICE2 + SIDVOICE_TIMESTAMP_ENVELOPE + 4], edx
			pop	dword edx
.end:			ret

write_voice2_ad:
			mov	byte  ah, al
			shr	byte  al, 4
			mov	byte  [VOICE2 + SIDVOICE_ATTACK], al
			and	byte  ah, 00fh
			mov	byte  [VOICE2 + SIDVOICE_DECAY], ah
			ret

write_voice2_sr:
			mov	byte  ah, al
			shr	byte  al, 4
			mov	byte  [VOICE2 + SIDVOICE_SUSTAIN], al
			and	byte  ah, 00fh
			mov	byte  [VOICE2 + SIDVOICE_RELEASE], ah
			ret

write_voice3_freq_lo:
			mov	byte  [VOICE3 + SIDVOICE_FREQUENCY], al
			ret

write_voice3_freq_hi:
			mov	byte  [VOICE3 + SIDVOICE_FREQUENCY + 1], al
			ret

write_voice3_pw_lo:
			mov	byte  [VOICE3 + SIDVOICE_PULSE_WIDTH], al
			ret

write_voice3_pw_hi:
			and	byte  al, 00fh
			mov	byte  [VOICE3 + SIDVOICE_PULSE_WIDTH + 1], al
			ret

write_voice3_control:
			mov	byte  ah, [VOICE3 + SIDVOICE_CONTROL]
			mov	byte  [VOICE3 + SIDVOICE_CONTROL], al
			xor	byte  ah, al
			test	byte  ah, 1
			jz	.end
			push	dword edx
			call	core_get_c64_cycles
			mov	dword [VOICE3 + SIDVOICE_TIMESTAMP_ENVELOPE], eax
			mov	dword [VOICE3 + SIDVOICE_TIMESTAMP_ENVELOPE + 4], edx
			pop	dword edx
.end:			ret

write_voice3_ad:
			mov	byte  ah, al
			shr	byte  al, 4
			mov	byte  [VOICE3 + SIDVOICE_ATTACK], al
			and	byte  ah, 00fh
			mov	byte  [VOICE3 + SIDVOICE_DECAY], ah
			ret

write_voice3_sr:
			mov	byte  ah, al
			shr	byte  al, 4
			mov	byte  [VOICE3 + SIDVOICE_SUSTAIN], al
			and	byte  ah, 00fh
			mov	byte  [VOICE3 + SIDVOICE_RELEASE], ah
			ret

write_filter_lo:
			ret

write_filter_hi:
			ret

write_filter_control:
			ret

write_volume:
			and	byte  al, 00fh
			mov	byte  [VOLUME], al
			ret



;****************************************************************************
;* sid6581_get_sample *******************************************************
;****************************************************************************
;* <=eax  16 bit signed sample
;****************************************************************************
section .text
sid6581_get_sample:
			call	core_get_c64_cycles
			mov	dword [TIMESTAMP], eax
			mov	dword [TIMESTAMP + 4], edx
			push	dword ebp
			xor	dword ecx, ecx
			lea	dword edi, [VOICE1]
			mov	byte  cl, 3
			mov	dword ebp, 01e00h
.next_voice:		push	dword ecx

			mov	dword eax, [TIMESTAMP]
			mov	dword ebx, [TIMESTAMP + 4]
			mov	dword ecx, eax
			mov	dword edx, ebx
			sub	dword eax, [edi + SIDVOICE_TIMESTAMP_OSCILLATOR]
			sbb	dword ebx, [edi + SIDVOICE_TIMESTAMP_OSCILLATOR + 4]
			mov	dword [edi + SIDVOICE_TIMESTAMP_OSCILLATOR], ecx
			mov	dword [edi + SIDVOICE_TIMESTAMP_OSCILLATOR + 4], edx

;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: more accurate exception handling here ?
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

			test	dword ebx, ebx
			jnz	near  .skip_voice

			;----------------------------------------
			;get the sample from the current waveform
			;----------------------------------------

			test	dword [edi + SIDVOICE_FREQUENCY], NOT32
			jz	near  .skip_voice
			mov	byte  bl, [edi + SIDVOICE_CONTROL]
			shr	byte  bl, 4
			mov	dword ecx, sid6581_triangle
			cmp	byte  bl, 1
			je	.call_waveform
			mov	dword ecx, sid6581_sawtooth
			cmp	byte  bl, 2
			je	.call_waveform
			mov	dword ecx, sid6581_pulse
			cmp	byte  bl, 4
			je	.call_waveform
			mov	dword ecx, sid6581_noise
			cmp	byte  bl, 8
			jne	near  .skip_voice
.call_waveform:		call	dword ecx

			;----------------------
			;calculate the envelope
			;----------------------

			mov	dword ebx, [TIMESTAMP]
			mov	dword ecx, [TIMESTAMP + 4]
			sub	dword ebx, [edi + SIDVOICE_TIMESTAMP_ENVELOPE]
			sbb	dword ecx, [edi + SIDVOICE_TIMESTAMP_ENVELOPE + 4]
			test	dword ebx, ebx
			jz	near  .skip_voice
			test	byte  [edi + SIDVOICE_CONTROL], 001h
			jz	near  .check_release
			test	dword ebx, 080000000h
			jnz	.force_sustain
			test	dword ecx, ecx
			jnz	.force_sustain
			mov	byte  cl, [edi + SIDVOICE_ATTACK]
			mov	dword ecx, [.ad__attack_cycles + 4 * ecx]
			cmp	dword ebx, ecx
			ja	.check_decay
			imul	dword ebx
			idiv	dword ecx
			jmp	.adsr_end

.check_decay:		sub	dword ebx, ecx
			xor	dword ecx, ecx
			mov	byte  cl, [edi + SIDVOICE_DECAY]
			mov	dword ecx, [.ad__decay_cycles + 4 * ecx]
			cmp	dword ebx, ecx
			ja	.force_sustain
			push	dword eax
			mov	dword edx, ecx
			mov	byte  al, 00fh
			sub	dword edx, ebx
			sub	byte  al, [edi + SIDVOICE_SUSTAIN]
			mov	dword ebx, edx
			jz	.no_div
			shl	dword eax, 24
			imul	dword ebx
			idiv	dword ecx
			mov	byte  bl, [edi + SIDVOICE_SUSTAIN]
			mov	dword ecx, 00f000000h
			shl	dword ebx, 24
			add	dword ebx, eax
			pop	dword eax
			imul	dword ebx
			idiv	dword ecx
			jmp	.adsr_end
.no_div:		pop	dword eax
			jmp	.adsr_end

.force_sustain:		xor	dword ebx, ebx
			mov	dword ecx, 00000000fh
			mov	byte  bl, [edi + SIDVOICE_SUSTAIN]
			imul	dword ebx
			idiv	dword ecx
			jmp	.adsr_end

.check_release:		test	dword ebx, 080000000h
			jnz	.skip_voice
			test	dword ecx, ecx
			jnz	.skip_voice
			mov	byte  cl, [edi + SIDVOICE_RELEASE]
			mov	dword ecx, [.ad__decay_cycles + 4 * ecx]
			cmp	dword ebx, ecx
			ja	.skip_voice
			push	dword eax
			mov	dword edx, ecx
			mov	byte  al, [edi + SIDVOICE_SUSTAIN]
			sub	dword edx, ebx
			shl	dword eax, 24
			mov	dword ebx, edx
			imul	dword ebx
			idiv	dword ecx
			mov	dword ebx, eax
			mov	dword ecx, 00f0000000h
			pop	dword eax
			imul	dword ebx
			idiv	dword ecx
.adsr_end:		add	dword ebp, eax

			;----------------------
			;prepare for next voice
			;----------------------

.skip_voice:		pop	dword ecx
			add	dword edi, SIDVOICE_SIZE
			dec	dword ecx
			jnz	near  .next_voice

			;----------------
			;calculate volume
			;----------------

			mov	dword ecx, 00000000fh
			mov	dword eax, ebp
			imul	dword [VOLUME]
			idiv	dword ecx
			and	dword eax, NOT16

			pop	dword ebp
			ret

			align	4
.ad__attack_cycles:	dd	2000
			dd	8000
			dd	16000
			dd	24000
			dd	38000
			dd	56000
			dd	68000
			dd	80000
			dd	100000
			dd	240000
			dd	500000
			dd	800000
			dd	1000000
			dd	3000000
			dd	5000000
			dd	8000000

.ad__decay_cycles:	dd	6000
			dd	24000
			dd	48000
			dd	72000
			dd	114000
			dd	168000
			dd	204000
			dd	240000
			dd	300000
			dd	750000
			dd	1500000
			dd	2400000
			dd	3000000
			dd	9000000
			dd	15000000
			dd	24000000



;****************************************************************************
;* sid6581_triangle *********************************************************
;****************************************************************************
;* eax=>  cycle difference
;* edi=>  pointer to tsidvoice
;* ebp=>  audio output
;* <=edi  pointer to tsidvoice
;* <=ebp  audio output
;* <=eax  sample
;****************************************************************************
section .text
sid6581_triangle:
			mov	dword ebx, [edi + SIDVOICE_FREQUENCY]
			mul	dword ebx
			add	dword eax, [edi + SIDVOICE_REMAINING_OFFSET]
			xor	dword edx, edx
			and	dword eax, 000ffffffh
			mov	dword [edi + SIDVOICE_REMAINING_OFFSET], eax
			div	dword ebx
			push	dword eax
			xor	dword edx, edx
			mov	dword eax, 001000000h
			div	dword ebx
			add	dword edx, [edi + SIDVOICE_REMAINING_CYCLES]
			cmp	dword edx, ebx
			jb	.no_increase
			sub	dword edx, ebx
			inc	dword eax
.no_increase:		mov	dword ecx, eax
			mov	dword [edi + SIDVOICE_REMAINING_CYCLES], edx
			pop	dword eax
			xor	dword edx, edx
			shl	dword eax, 15
			div	dword ecx
			cmp	dword eax, 04000h
			jb	.no_substract
			mov	dword ebx, 08000h
			sub	dword ebx, eax
			mov	dword eax, ebx
.no_substract:		sub	dword eax, 02000h
			ret



;****************************************************************************
;* sid6581_sawtooth *********************************************************
;****************************************************************************
;* eax=>  cycle difference
;* edi=>  pointer to tsidvoice
;* ebp=>  audio output
;* <=edi  pointer to tsidvoice
;* <=ebp  audio output
;* <=eax  sample
;****************************************************************************
section .text
sid6581_sawtooth:
			mov	dword ebx, [edi + SIDVOICE_FREQUENCY]
			mul	dword ebx
			add	dword eax, [edi + SIDVOICE_REMAINING_OFFSET]
			xor	dword edx, edx
			and	dword eax, 000ffffffh
			mov	dword [edi + SIDVOICE_REMAINING_OFFSET], eax
			div	dword ebx
			push	dword eax
			xor	dword edx, edx
			mov	dword eax, 001000000h
			div	dword ebx
			add	dword edx, [edi + SIDVOICE_REMAINING_CYCLES]
			cmp	dword edx, ebx
			jb	.no_increase
			sub	dword edx, ebx
			inc	dword eax
.no_increase:		mov	dword ecx, eax
			mov	dword [edi + SIDVOICE_REMAINING_CYCLES], edx
			pop	dword eax
			xor	dword edx, edx
			shl	dword eax, 14
			div	dword ecx
			sub	dword eax, 02000h
			ret



;****************************************************************************
;* sid6581_pulse ************************************************************
;****************************************************************************
;* eax=>  cycle difference
;* edi=>  pointer to tsidvoice
;* ebp=>  audio output
;* <=edi  pointer to tsidvoice
;* <=ebp  audio output
;* <=eax  sample
;****************************************************************************
section .text
sid6581_pulse:
			mov	dword ebx, [edi + SIDVOICE_FREQUENCY]
			mul	dword ebx
			add	dword eax, [edi + SIDVOICE_REMAINING_OFFSET]
			xor	dword edx, edx
			and	dword eax, 000ffffffh
			mov	dword [edi + SIDVOICE_REMAINING_OFFSET], eax
			div	dword ebx
			push	dword eax
			xor	dword edx, edx
			mov	dword eax, 001000000h
			div	dword ebx
			add	dword edx, [edi + SIDVOICE_REMAINING_CYCLES]
			cmp	dword edx, ebx
			jb	.no_increase
			sub	dword edx, ebx
			inc	dword eax
.no_increase:		mov	dword ecx, eax
			mov	dword [edi + SIDVOICE_REMAINING_CYCLES], edx
			pop	dword eax
			xor	dword edx, edx
			shl	dword eax, 12
			div	dword ecx
			mov	dword ebx, 01fffh
			cmp	dword eax, [edi + SIDVOICE_PULSE_WIDTH]
			jb	.pulse
			mov	dword ebx, -02000h
.pulse:			mov	dword eax, ebx
			ret



;****************************************************************************
;* sid6581_noise ************************************************************
;****************************************************************************
section .text
sid6581_noise:
			mov	dword ebx, [edi + SIDVOICE_FREQUENCY]
			shl	dword ebx, 4
			mul	dword ebx
			add	dword eax, [edi + SIDVOICE_REMAINING_OFFSET]
			mov	dword ecx, eax
			shl	dword edx, 8
			shr	dword ecx, 24
			and	dword eax, 000ffffffh
			or	dword ecx, edx
			mov	dword [edi + SIDVOICE_REMAINING_OFFSET], eax
			mov	dword eax, [edi + SIDVOICE_NOISE_REGISTER]
			test	dword ecx, ecx
			jz	.calculate_sample
.loop:			xor	dword ebx, ebx
			test	dword eax, 0400000h
			jz	.skip1
			inc	dword ebx
.skip1:			test	dword eax, 020000h
			jz	.skip2
			xor	byte  bl, 001h
.skip2:			shl	dword eax, 1
			or	dword eax, ebx
			dec	dword ecx
			jnz	.loop
			mov	dword [edi + SIDVOICE_NOISE_REGISTER], eax
.calculate_sample:	xor	dword ebx, ebx
			test	byte  al, 008h
			jz	.bit0
			or	byte  bl, 001h
.bit0:			test	byte  al, 010h
			jz	.bit1
			or	byte  bl, 002h
.bit1:			test	byte  al, 080h
			jz	.bit2
			or	byte  bl, 004h
.bit2:			test	dword eax, 00800h
			jz	.bit3
			or	byte  bl, 008h
.bit3:			test	dword eax, 02000h
			jz	.bit4
			or	byte  bl, 010h
.bit4:			test	dword eax, 000010000h
			jz	.bit5
			or	byte  bl, 020h
.bit5:			test	dword eax, 000100000h
			jz	.bit6
			or	byte  bl, 040h
.bit6:			test	dword eax, 000400000h
			jz	.bit7
			or	byte  bl, 080h
.bit7:			shl	dword ebx, 6
			sub	dword ebx, 02000h
			mov	dword eax, ebx
			ret



;****************************************************************************
;* sid6581_cycle ************************************************************
;****************************************************************************
section .text
sid6581_cycle:
			call	sid6581_get_sample
			mov	dword ebx, [BUFFER_OFFSET]
			mov	dword ecx, [ebx]
			mov	dword edx, [BUFFER]
			mov	word  [edx + 2 * ecx], ax
			inc	dword ecx
			cmp	dword ecx, [BUFFER_SIZE]
			jb	.end
			xor	dword ecx, ecx
.end:			mov	dword [ebx], ecx
			ret
;******************************************************** ec64@unusedino.de *

