/****************************************************************************
 ****************************************************************************
 *
 * socket.h
 *
 ****************************************************************************
 ****************************************************************************/





#ifndef EC64SYS_SOCKET_H
#define EC64SYS_SOCKET_H

#define AF_UNSPEC	0
#define AF_UNIX		1		/* Unix domain sockets */
#define AF_LOCAL	1		/* POSIX name for AF_UNIX */
#define AF_INET		2		/* Internet IP Protocol */
#define AF_AX25		3		/* Amateur Radio AX.25 */
#define AF_IPX		4		/* Novell IPX */
#define AF_APPLETALK	5		/* AppleTalk DDP */
#define AF_NETROM	6		/* Amateur Radio NET/ROM */
#define AF_BRIDGE	7		/* Multiprotocol bridge */
#define AF_ATMPVC	8		/* ATM PVCs */
#define AF_X25		9		/* Reserved for X.25 project */
#define AF_INET6	10		/* IP version 6 */
#define AF_ROSE		11		/* Amateur Radio X.25 PLP */
#define AF_DECnet	12		/* Reserved for DECnet project */
#define AF_NETBEUI	13		/* Reserved for 802.2LLC project */
#define AF_SECURITY	14		/* Security callback pseudo AF */
#define AF_KEY		15      	/* PF_KEY key management API */
#define AF_NETLINK	16
#define AF_ROUTE	AF_NETLINK	/* Alias to emulate 4.4BSD */
#define AF_PACKET	17		/* Packet family */
#define AF_ASH		18		/* Ash */
#define AF_ECONET	19		/* Acorn Econet */
#define AF_ATMSVC	20		/* ATM SVCs */
#define AF_SNA		22		/* Linux SNA Project (nutters!) */
#define AF_IRDA		23		/* IRDA sockets */
#define AF_MAX		32		/* For now.. */

/* Protocol families, same as address families. */

#define PF_UNSPEC	AF_UNSPEC
#define PF_UNIX		AF_UNIX
#define PF_LOCAL	AF_LOCAL
#define PF_INET		AF_INET
#define PF_AX25		AF_AX25
#define PF_IPX		AF_IPX
#define PF_APPLETALK	AF_APPLETALK
#define	PF_NETROM	AF_NETROM
#define PF_BRIDGE	AF_BRIDGE
#define PF_ATMPVC	AF_ATMPVC
#define PF_X25		AF_X25
#define PF_INET6	AF_INET6
#define PF_ROSE		AF_ROSE
#define PF_DECnet	AF_DECnet
#define PF_NETBEUI	AF_NETBEUI
#define PF_SECURITY	AF_SECURITY
#define PF_KEY		AF_KEY
#define PF_NETLINK	AF_NETLINK
#define PF_ROUTE	AF_ROUTE
#define PF_PACKET	AF_PACKET
#define PF_ASH		AF_ASH
#define PF_ECONET	AF_ECONET
#define PF_ATMSVC	AF_ATMSVC
#define PF_SNA		AF_SNA
#define PF_IRDA		AF_IRDA
#define PF_MAX		AF_MAX

#define SOCK_STREAM    1		/* stream (connection) socket */
#define SOCK_DGRAM     2		/* datagram (conn.less) socket */
#define SOCK_RAW       3		/* raw socket */
#define SOCK_RDM       4		/* reliably-delivered message */
#define SOCK_SEQPACKET 5		/* sequential packet socket */
#define SOCK_PACKET    10		/* linux specific way of */
					/* getting packets at the dev */
					/* level.  For writing rarp and */
					/* other similar things on the */
					/* user level. */

/* Maximum queue length specifiable by listen. */

#define SOMAXCONN	128

/* Flags we can use with send/ and recv.  */
/* Added those for 1003.1g not all are supported yet */
 
#define MSG_OOB		1
#define MSG_PEEK	2
#define MSG_DONTROUTE	4
#define MSG_TRYHARD     4       /* Synonym for MSG_DONTROUTE for DECnet */
#define MSG_CTRUNC	8
#define MSG_PROXY	0x10	/* Supply or ask second address. */
#define MSG_TRUNC	0x20
#define MSG_DONTWAIT	0x40	/* Nonblocking io */
#define MSG_EOR         0x80	/* End of record */
#define MSG_WAITALL	0x100	/* Wait for a full request */
#define MSG_FIN         0x200
#define MSG_SYN		0x400
#define MSG_URG		0x800
#define MSG_RST		0x1000
#define MSG_ERRQUEUE	0x2000
#define MSG_NOSIGNAL	0x4000

#define MSG_CTLIGNORE   0x80000000

#define MSG_EOF         MSG_FIN
#define MSG_CTLFLAGS	(MSG_OOB | MSG_URG | MSG_FIN | MSG_SYN | MSG_RST)

/* Setsockoptions(2) level. Thanks to BSD these must match IPPROTO_xxx */

#define SOL_IP		0
/* #define SOL_ICMP	1	No-no-no! Due to Linux :-) we cannot use SOL_ICMP=1 */
#define SOL_TCP		6
#define SOL_UDP		17
#define SOL_IPV6	41
#define SOL_ICMPV6	58
#define SOL_RAW		255
#define SOL_IPX		256
#define SOL_AX25	257
#define SOL_ATALK	258
#define SOL_NETROM	259
#define SOL_ROSE	260
#define SOL_DECNET	261
#define	SOL_X25		262
#define SOL_PACKET	263
#define SOL_ATM		264	/* ATM layer (cell level) */
#define SOL_AAL		265	/* ATM Adaption Layer (packet level) */
#define SOL_IRDA        266

/* IPX options */

#define IPX_TYPE	1

/* TCP options */

#define TCP_NODELAY	1
#define TCP_MAXSEG	2
#define TCP_CORK	3		/* Linux specific (for use with sendfile) */

#define IPPROTO_IP	0		/* Dummy protocol for TCP */
#define IPPROTO_ICMP	1		/* Internet Control Message Protocol */
#define IPPROTO_IGMP	2		/* Internet Group Management Protocol */
#define IPPROTO_IPIP	4		/* IPIP tunnels (older KA9Q tunnels use 94) */
#define IPPROTO_TCP	6		/* Transmission Control Protocol */
#define IPPROTO_EGP	8		/* Exterior Gateway Protocol */
#define IPPROTO_PUP	12		/* PUP protocol */
#define IPPROTO_UDP	17		/* User Datagram Protocol */
#define IPPROTO_IDP	22		/* XNS IDP protocol */
#define IPPROTO_RSVP	46		/* RSVP protocol */
#define IPPROTO_GRE	47		/* Cisco GRE tunnels (rfc 1701,1702) */
#define IPPROTO_IPV6	41		/* IPv6-in-IPv4 tunnelling */
#define IPPROTO_PIM	103		/* Protocol Independent Multicast */
#define IPPROTO_ESP	50		/* Encapsulation Security Payload protocol */
#define IPPROTO_AH	51		/* Authentication Header protocol */
#define IPPROTO_COMP	108		/* Compression Header protocol */
#define IPPROTO_RAW	255		/* Raw IP packets */

#define SYS_SOCKET	1		/* sys_socket(2) */
#define SYS_BIND	2		/* sys_bind(2) */
#define SYS_CONNECT	3		/* sys_connect(2) */
#define SYS_LISTEN	4		/* sys_listen(2) */
#define SYS_ACCEPT	5		/* sys_accept(2) */
#define SYS_GETSOCKNAME	6		/* sys_getsockname(2) */
#define SYS_GETPEERNAME	7		/* sys_getpeername(2) */
#define SYS_SOCKETPAIR	8		/* sys_socketpair(2) */
#define SYS_SEND	9		/* sys_send(2) */
#define SYS_RECV	10		/* sys_recv(2) */
#define SYS_SENDTO	11		/* sys_sendto(2) */
#define SYS_RECVFROM	12		/* sys_recvfrom(2) */
#define SYS_SHUTDOWN	13		/* sys_shutdown(2) */
#define SYS_SETSOCKOPT	14		/* sys_setsockopt(2) */
#define SYS_GETSOCKOPT	15		/* sys_getsockopt(2) */
#define SYS_SENDMSG	16		/* sys_sendmsg(2) */
#define SYS_RECVMSG	17		/* sys_recvmsg(2) */

typedef struct
	{
	w		familiy;
	b		data[14];
	}
tsockaddr;

typedef struct
	{
	w		familiy;
	b		path[108];
	}
tsockaddrun;



#endif
/******************************************************** ec64@unusedino.de */

