;****************************************************************************
;****************************************************************************
;*
;* socket.i
;*
;****************************************************************************
;****************************************************************************





%ifndef EC64SYS_SOCKET_I
%define EC64SYS_SOCKET_I

%assign AF_UNSPEC	0
%assign AF_UNIX		1		;Unix domain sockets
%assign AF_LOCAL	1		;POSIX name for AF_UNIX
%assign AF_INET		2		;Internet IP Protocol
%assign AF_AX25		3		;Amateur Radio AX.25
%assign AF_IPX		4		;Novell IPX
%assign AF_APPLETALK	5		;AppleTalk DDP
%assign AF_NETROM	6		;Amateur Radio NET/ROM
%assign AF_BRIDGE	7		;Multiprotocol bridge
%assign AF_ATMPVC	8		;ATM PVCs
%assign AF_X25		9		;Reserved for X.25 project
%assign AF_INET6	10		;IP version 6
%assign AF_ROSE		11		;Amateur Radio X.25 PLP
%assign AF_DECnet	12		;Reserved for DECnet project
%assign AF_NETBEUI	13		;Reserved for 802.2LLC project
%assign AF_SECURITY	14		;Security callback pseudo AF
%assign AF_KEY		15      	;PF_KEY key management API
%assign AF_NETLINK	16
%assign AF_ROUTE	AF_NETLINK	;Alias to emulate 4.4BSD
%assign AF_PACKET	17		;Packet family
%assign AF_ASH		18		;Ash
%assign AF_ECONET	19		;Acorn Econet
%assign AF_ATMSVC	20		;ATM SVCs
%assign AF_SNA		22		;Linux SNA Project (nutters!)
%assign AF_IRDA		23		;IRDA sockets
%assign AF_MAX		32		;For now..

;Protocol families, same as address families.

%assign PF_UNSPEC	AF_UNSPEC
%assign PF_UNIX		AF_UNIX
%assign PF_LOCAL	AF_LOCAL
%assign PF_INET		AF_INET
%assign PF_AX25		AF_AX25
%assign PF_IPX		AF_IPX
%assign PF_APPLETALK	AF_APPLETALK
%assign	PF_NETROM	AF_NETROM
%assign PF_BRIDGE	AF_BRIDGE
%assign PF_ATMPVC	AF_ATMPVC
%assign PF_X25		AF_X25
%assign PF_INET6	AF_INET6
%assign PF_ROSE		AF_ROSE
%assign PF_DECnet	AF_DECnet
%assign PF_NETBEUI	AF_NETBEUI
%assign PF_SECURITY	AF_SECURITY
%assign PF_KEY		AF_KEY
%assign PF_NETLINK	AF_NETLINK
%assign PF_ROUTE	AF_ROUTE
%assign PF_PACKET	AF_PACKET
%assign PF_ASH		AF_ASH
%assign PF_ECONET	AF_ECONET
%assign PF_ATMSVC	AF_ATMSVC
%assign PF_SNA		AF_SNA
%assign PF_IRDA		AF_IRDA
%assign PF_MAX		AF_MAX

%assign SOCK_STREAM    1		;stream (connection) socket
%assign SOCK_DGRAM     2		;datagram (conn.less) socket
%assign SOCK_RAW       3		;raw socket
%assign SOCK_RDM       4		;reliably-delivered message
%assign SOCK_SEQPACKET 5		;sequential packet socket
%assign SOCK_PACKET    10		;linux specific way of
					;getting packets at the dev
					;level.  For writing rarp and
					;other similar things on the
					;user level.

;Maximum queue length specifiable by listen.

%assign SOMAXCONN	128

;Flags we can use with send/ and recv. 
;Added those for 1003.1g not all are supported yet
 
%assign MSG_OOB		1
%assign MSG_PEEK	2
%assign MSG_DONTROUTE	4
%assign MSG_TRYHARD     4       ;Synonym for MSG_DONTROUTE for DECnet
%assign MSG_CTRUNC	8
%assign MSG_PROXY	010h	;Supply or ask second address.
%assign MSG_TRUNC	020h
%assign MSG_DONTWAIT	040h	;Nonblocking io
%assign MSG_EOR         080h	;End of record
%assign MSG_WAITALL	0100h	;Wait for a full request
%assign MSG_FIN         0200h
%assign MSG_SYN		0400h
%assign MSG_URG		0800h
%assign MSG_RST		01000h
%assign MSG_ERRQUEUE	02000h
%assign MSG_NOSIGNAL	04000h

%assign MSG_CTLIGNORE   080000000h

%assign MSG_EOF         MSG_FIN
%assign MSG_CTLFLAGS	(MSG_OOB | MSG_URG | MSG_FIN | MSG_SYN | MSG_RST)

;Setsockoptions(2) level. Thanks to BSD these must match IPPROTO_xxx

%assign SOL_IP		0
;%assign SOL_ICMP	1	No-no-no! Due to Linux :-) we cannot use SOL_ICMP=1
%assign SOL_TCP		6
%assign SOL_UDP		17
%assign SOL_IPV6	41
%assign SOL_ICMPV6	58
%assign SOL_RAW		255
%assign SOL_IPX		256
%assign SOL_AX25	257
%assign SOL_ATALK	258
%assign SOL_NETROM	259
%assign SOL_ROSE	260
%assign SOL_DECNET	261
%assign	SOL_X25		262
%assign SOL_PACKET	263
%assign SOL_ATM		264	;ATM layer (cell level)
%assign SOL_AAL		265	;ATM Adaption Layer (packet level)
%assign SOL_IRDA        266

;IPX options

%assign IPX_TYPE	1

;TCP options

%assign TCP_NODELAY	1
%assign TCP_MAXSEG	2
%assign TCP_CORK	3		;Linux specific (for use with sendfile)

%assign IPPROTO_IP	0		;Dummy protocol for TCP
%assign IPPROTO_ICMP	1		;Internet Control Message Protocol
%assign IPPROTO_IGMP	2		;Internet Group Management Protocol
%assign IPPROTO_IPIP	4		;IPIP tunnels (older KA9Q tunnels use 94)
%assign IPPROTO_TCP	6		;Transmission Control Protocol
%assign IPPROTO_EGP	8		;Exterior Gateway Protocol
%assign IPPROTO_PUP	12		;PUP protocol
%assign IPPROTO_UDP	17		;User Datagram Protocol
%assign IPPROTO_IDP	22		;XNS IDP protocol
%assign IPPROTO_RSVP	46		;RSVP protocol
%assign IPPROTO_GRE	47		;Cisco GRE tunnels (rfc 1701,1702)
%assign IPPROTO_IPV6	41		;IPv6-in-IPv4 tunnelling
%assign IPPROTO_PIM	103		;Protocol Independent Multicast
%assign IPPROTO_ESP	50		;Encapsulation Security Payload protocol
%assign IPPROTO_AH	51		;Authentication Header protocol
%assign IPPROTO_COMP	108		;Compression Header protocol
%assign IPPROTO_RAW	255		;Raw IP packets

%assign SYS_SOCKET	1		;sys_socket(2)
%assign SYS_BIND	2		;sys_bind(2)
%assign SYS_CONNECT	3		;sys_connect(2)
%assign SYS_LISTEN	4		;sys_listen(2)
%assign SYS_ACCEPT	5		;sys_accept(2)
%assign SYS_GETSOCKNAME	6		;sys_getsockname(2)
%assign SYS_GETPEERNAME	7		;sys_getpeername(2)
%assign SYS_SOCKETPAIR	8		;sys_socketpair(2)
%assign SYS_SEND	9		;sys_send(2)
%assign SYS_RECV	10		;sys_recv(2)
%assign SYS_SENDTO	11		;sys_sendto(2)
%assign SYS_RECVFROM	12		;sys_recvfrom(2)
%assign SYS_SHUTDOWN	13		;sys_shutdown(2)
%assign SYS_SETSOCKOPT	14		;sys_setsockopt(2)
%assign SYS_GETSOCKOPT	15		;sys_getsockopt(2)
%assign SYS_SENDMSG	16		;sys_sendmsg(2)
%assign SYS_RECVMSG	17		;sys_recvmsg(2)



			struc	tsockaddr
			alignb	4
w__sockaddr_family:	resw	1
ab__sockaddr_data:	resb	14
			endstruc

			struc	tsockaddrun
			alignb	4
w__sockaddrun_family:	resw	1
ab__sockaddrun_path:	resb	108
			endstruc



%endif
;******************************************************** ec64@unusedino.de *

