;****************************************************************************
;****************************************************************************
;*
;* draw.n
;*
;****************************************************************************
;****************************************************************************





%define DRAW_N
%include "draw.i"
%include "ec64rom.i"	;draw_text needs access to ec64rom_c64_character




;****************************************************************************
;* draw_initialize **********************************************************
;****************************************************************************
section .text
draw_initialize:
			ret



;****************************************************************************
;* draw_text ****************************************************************
;****************************************************************************
;* [1]=>  pointer to framebuffer
;* [2]=>  virtual x resolution
;* [3]=>  pointer to text string
;****************************************************************************
section .text
draw_text:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			push	dword esi
			push	dword edi
			push	dword ebp
			mov	dword edi, [esp + 32]
			mov	dword edx, [esp + 36]
			mov	dword esi, [esp + 40]
			mov	dword ebp, edx
			sub	dword edx, byte 8
			shr	dword ebp, 3
			jz	.end
			jmp	.next_character
.continue:		and	byte  bl, 07fh
			mov	byte  bl, [.ab__translate_table + ebx]
			shl	dword ebx, 3
			push	dword edi
			mov	byte  cl, 008h
.character_line:	mov	byte  al, [ec64rom_c64_character + 00800h + ebx]
			mov	byte  ch, 008h
.character_pixel:	xor	byte  ah, ah
			shl	byte  al, 1
			jnc	.pixel_clear
			mov	byte  ah, 00fh
.pixel_clear:		mov	byte  [edi], ah
			inc	dword edi
			dec	byte  ch
			jnz	.character_pixel
			inc	dword ebx
			add	dword edi, edx
			dec	byte  cl
			jnz	.character_line
			pop	dword edi
			dec	dword ebp
			jz	.end
			add	dword edi, byte 8
.next_character:	xor	dword ebx, ebx
			mov	byte  bl, [esi]
			inc	dword esi
			test	byte  bl, bl
			jnz	.continue
.end:			pop	dword ebp
			pop	dword edi
			pop	dword esi
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret

.ab__translate_table:	db	000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
			db	000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
			db	000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
			db	000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
			db	020h, 021h, 022h, 023h, 024h, 025h, 026h, 027h
			db	028h, 029h, 02ah, 02bh, 02ch, 02dh, 02eh, 02fh
			db	030h, 031h, 032h, 033h, 034h, 035h, 036h, 037h
			db	038h, 039h, 03ah, 03bh, 03ch, 03dh, 03eh, 03fh
			db	000h, 041h, 042h, 043h, 044h, 045h, 046h, 047h
			db	048h, 049h, 04ah, 04bh, 04ch, 04dh, 04eh, 04fh
			db	050h, 051h, 052h, 053h, 054h, 055h, 056h, 057h
			db	058h, 059h, 05ah, 01bh, 01fh, 01dh, 01ch, 064h
			db	07ah, 001h, 002h, 003h, 004h, 005h, 006h, 007h
			db	008h, 009h, 00ah, 00bh, 00ch, 00dh, 00eh, 00fh
			db	010h, 011h, 012h, 013h, 014h, 015h, 016h, 017h
			db	018h, 019h, 01ah, 01bh, 05dh, 01dh, 01eh, 01fh


;****************************************************************************
;* draw_led *****************************************************************
;****************************************************************************
;* [1]=>  pointer to framebuffer
;* [2]=>  x padding
;* [3]=>  on/off
;****************************************************************************
section .text
draw_led:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edi
			mov	dword edi, [esp + 20]
			mov	dword ebx, [esp + 24]
			mov	dword eax, [esp + 28]
			xor	dword ecx, ecx
			test	byte  al, al
			mov	byte  al, 00bh
			jz	.led_off
			mov	byte  al, 00dh
.led_off:		cld
			xor	byte  ah, ah
			mov	byte  cl, 007h
			rep stosb
			mov	byte  [edi], ah
			lea	dword edi, [edi + ebx - 7]
			mov	byte  cl, 007h
			rep stosb
			mov	byte  [edi], ah
			pop	dword edi
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* draw_plane ***************************************************************
;****************************************************************************
;* [1]=>  destination
;* [2]=>  source
;* [3]=>  length
;****************************************************************************
section .text
draw_plane:
			push	dword eax
			push	dword esi
			push	dword edi
			push	dword ebp
			mov	dword edi, [esp + 20]
			mov	dword esi, [esp + 24]
			mov	dword ebp, [esp + 28]
			test	dword ebp, ebp
			jz	.end
.loop:			mov	byte  al, [esi]
			add	dword esi, 4
			mov	byte  [edi], al
			inc	dword edi
			dec	dword ebp
			jnz	.loop
.end:			pop	dword ebp
			pop	dword edi
			pop	dword esi
			pop	dword eax
			ret



;****************************************************************************
;* draw_planes **************************************************************
;****************************************************************************
;* [1]=>  destination
;* [2]=>  source
;* [3]=>  x padding destination
;* [4]=>  x padding source
;* [5]=>  length
;****************************************************************************
section .text
draw_planes:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			push	dword esi
			push	dword edi
			push	dword ebp
			mov	dword edi, [esp + 32]
			mov	dword esi, [esp + 36]
			mov	dword edx, [esp + 40]
			mov	dword ecx, [esp + 44]
			mov	dword ebp, [esp + 48]
			test	dword ebp, ebp
			jz	.end
			cmp	dword ebp, 8
			jb	.bytes_only
			test	dword edi, 3
			jz	.dword_loop
			push	dword ebp
			mov	dword ebp, edi
			and	dword ebp, 3
			xor	dword ebp, 3
			inc	dword ebp
			sub	dword [esp], ebp
			call	.byte_loop
			pop	dword ebp
.dword_loop:		mov	byte  al, [esi + 8]
			mov	byte  bl, [esi + ecx + 8]
			mov	byte  ah, [esi + 12]
			mov	byte  bh, [esi + ecx + 12]
			shl	dword eax, 16
			shl	dword ebx, 16
			mov	byte  al, [esi]
			mov	byte  bl, [esi + ecx]
			mov	byte  ah, [esi + 4]
			mov	byte  bh, [esi + ecx + 4]
			add	dword esi, 16
			mov	dword  [edi], eax
			mov	dword  [edi + edx], ebx
			sub	dword ebp, 4
			add	dword edi, 4
			cmp	dword ebp, 4
			jae	.dword_loop
			test	dword ebp, ebp
			jz	.end
.bytes_only:		call	.byte_loop
.end:			pop	dword ebp
			pop	dword edi
			pop	dword esi
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret

.byte_loop:		mov	byte  al, [esi]
			mov	byte  bl, [esi + ecx]
			add	dword esi, 4
			mov	byte  [edi], al
			mov	byte  [edi + edx], bl
			inc	dword edi
			dec	dword ebp
			jnz	.byte_loop
			ret
;******************************************************** ec64@unusedino.de *

