/****************************************************************************
 ****************************************************************************
 *
 * ec64io.c
 *
 ****************************************************************************
 ****************************************************************************/





#include "ec64io.h"
#include "ec64sys.h"
#include "ec64ui.h"
#include "memory.h"
#include "string.h"
#include "error.h"
#include "video.h"
#include "fb.h"		/* for fb_set_path */
#include "audio.h"
#include "tty.h"
#include "gpm.h"
#include "sync.h"
#include "vga.h"	/* for vga_set_mode_string */



static d				input_mode = INPUT_MODE_C64_KEYBOARD;
static d				c64_keyboard_input = 0;
static d				c64_joystick1_input = 0;
static d				c64_joystick2_input = 0;
static d				monitor_input = 0;
static b				*d64_path = NULL;
static b				*script_path = NULL;
static b				log_path[1024];
static tvideo				video_options =
					{
					400,
					260,
					80,
					20,
/*					352,
					232,
					104,
					34,*/
					NULL,
					NULL,
					0,
					0
					};
static taudio				audio_options =
					{
					44100,
/*					22050,*/
					0,
					NULL,
					0
					};




/****************************************************************************
 * ec64io_default_handler ***************************************************
 ****************************************************************************/
void	ec64io_default_handler(
	d				signal)

	{
	ec64io_end();
	}



/****************************************************************************
 * ec64io_sigsegv_handler ***************************************************
 ****************************************************************************/
void	ec64io_sigsegv_handler(
	d				signal)

	{
#ifdef QUICK_CYCLE_HACK
#include "ec64core.h"
	const tc64stat			*c64_status = ec64core_c64_get_status();
	b				string[1024];
#endif

	ec64io_restore();

#ifdef QUICK_CYCLE_HACK
	ec64io_string_print(string, 1023, STRING_TYPE "cycles: " STRING_HEX64 "\n", c64_status->cycles.high, c64_status->cycles.low);
	tty_print_text(string);
#endif

	tty_print_text(STRING_TYPE "THE PROGRAM ITSELF CAUSED A SEGMENTATION VIOLATION, EXITING ...\n");
	ec64sys_exit(1);
	}



/****************************************************************************
 * ec64io_sigfpe_handler ****************************************************
 ****************************************************************************/
void	ec64io_sigfpe_handler(
	d				signal)

	{
	ec64io_restore();
	tty_print_text(STRING_TYPE "THE PROGRAM ITSELF CAUSED A FLOATING POINT EXCEPTION, EXITING ...\n");
	ec64sys_exit(1);
	}



/****************************************************************************
 * ec64io_get_version *******************************************************
 ****************************************************************************/
#ifdef EC64_ENABLE_FB_SUPPORT
#define EC64IO_FB			" +fb"
#else
#define EC64IO_FB			""
#endif
#ifdef EC64_ENABLE_VGA_SUPPORT
#define EC64IO_VGA			" +vga"
#else
#define EC64IO_VGA			""
#endif
#ifdef EC64_ENABLE_SCREENSHOT_SUPPORT
#define EC64IO_SCREENSHOT		" +screenshot"
#else
#define EC64IO_SCREENSHOT		""
#endif

const b	*ec64io_get_version(
	void)

	{
	static const b			text[] = "0.1";
	static b			string[256];

	string_print(string, 255, STRING_TYPE STRING_STR " +[ec64sys: " STRING_STR "]" EC64IO_FB EC64IO_VGA EC64IO_SCREENSHOT, text, ec64sys_get_version());
	return (string);
	}



/****************************************************************************
 * ec64io_process_cmdline ***************************************************
 ****************************************************************************/
void	ec64io_process_cmdline(
	d				argument_counter,
	b				*arguments[])

	{
	d				offset = 1;
	b				*argument;

	while (offset < argument_counter)
		{
		argument = arguments[offset ++];
		if (string_compare(argument, STRING_TYPE "-d") == 0)
			{
			if (offset < argument_counter) d64_path = arguments[offset ++];
			else error_text_exit(STRING_TYPE "d64 file name expected");
			}
/*		else if (string_compare(argument, STRING_TYPE "-g") == 0)
			{
			if (offset < argument_counter) g64_path = arguments[offset ++];
			else error_text_exit(STRING_TYPE "g64 file name expected");
			}*/
		else if (string_compare(argument, STRING_TYPE "-s") == 0)
			{
			if (offset < argument_counter) script_path = arguments[offset ++];
			else error_text_exit(STRING_TYPE "script file name expected");
			}
		else if (string_compare(argument, STRING_TYPE "-dsp") == 0)
			{
			if (offset < argument_counter) audio_set_path(arguments[offset ++]);
			else error_text_exit(STRING_TYPE "audio device expected");
			}
#ifdef EC64_ENABLE_FB_SUPPORT
		else if (string_compare(argument, STRING_TYPE "-fb") == 0)
			{
			if (offset < argument_counter) fb_set_path(arguments[offset ++]);
			else error_text_exit(STRING_TYPE "fb device expected");
			}
		else if (string_compare(argument, STRING_TYPE "-f") == 0) video_options.flags |= TVIDEO_FULLSCREEN;
#endif
#ifdef EC64_ENABLE_VGA_SUPPORT
		else if (string_compare(argument, STRING_TYPE "-vm") == 0)
			{
			if (offset < argument_counter) vga_set_mode_string(arguments[offset ++]);
			else error_text_exit(STRING_TYPE "vga mode expected");
			}
#endif
		else if (string_compare(argument, STRING_TYPE "-h") == 0)
			{
			tty_print_error(STRING_TYPE 
				"Usage: ec64 [-h] [-d <file>] [-s <file>] [-dsp <file>]"
#ifdef EC64_ENABLE_FB_SUPPORT
				" [-fb <file>] [-f]"
#endif
#ifdef EC64_ENABLE_VGA_SUPPORT
				"\n            [-vm <mode>]"
#endif
				"\n\n"
				"  -h          this help\n"
				"  -s <file>   load and execute script on startup\n"
				"  -d <file>   load d64 file on startup\n"
/*				"  -g <file>   load g64 file on startup\n"*/
				"  -dsp <file> audio device file\n"
#ifdef EC64_ENABLE_FB_SUPPORT
				"  -fb <file>  fb device file\n"
				"  -f          fullscreen 400x300@60Hz on fb device\n"
				"              (MAY BE NOT SUPPORTED ON ALL DEVICES!)\n"
#endif
#ifdef EC64_ENABLE_VGA_SUPPORT
				"  -vm <mode>  vga mode (supported values: 320x240, 360x240)\n"
#endif
				);
			error_exit();
			}
		else
			{
			b		string[1024];

			string_print(string, 1023, STRING_TYPE "unknown commandline option: " STRING_STR, argument);
			error_text_exit(string);
			}
		}
	}


	
/****************************************************************************
 * ec64io_start *************************************************************
 ****************************************************************************/
void	ec64io_start(
	d				argument_counter,
	b				*arguments[])

	{

	/* check operating system */

	if (ec64sys_os_type != EC64SYS_OS_LINUX)
		{
		tty_print_error(STRING_TYPE "BSD style kernels are not supported at the moment\n");
		ec64sys_exit(1);
		}

	/* set signal behavior */

	ec64sys_signal(SIGHUP, SIG_IGN);
	ec64sys_signal(SIGINT, ec64io_default_handler);
	ec64sys_signal(SIGQUIT, SIG_IGN);
	ec64sys_signal(SIGILL, SIG_IGN);
	ec64sys_signal(SIGTRAP, SIG_IGN);
	ec64sys_signal(SIGABRT, SIG_IGN);
	ec64sys_signal(SIGBUS, SIG_IGN);
	ec64sys_signal(SIGFPE, ec64io_sigfpe_handler);
	ec64sys_signal(SIGUSR1, SIG_IGN);
	ec64sys_signal(SIGSEGV, ec64io_sigsegv_handler);
	ec64sys_signal(SIGUSR2, SIG_IGN);
	ec64sys_signal(SIGPIPE, SIG_IGN);
	ec64sys_signal(SIGALRM, SIG_IGN);
	ec64sys_signal(SIGTERM, ec64io_default_handler);
	ec64sys_signal(SIGSTKFLT, SIG_IGN);
	ec64sys_signal(SIGCHLD, SIG_IGN);
	ec64sys_signal(SIGCONT, ec64io_default_handler);
	ec64sys_signal(SIGTSTP, SIG_IGN);
	ec64sys_signal(SIGTTIN, SIG_IGN);
	ec64sys_signal(SIGTTOU, SIG_IGN);
	ec64sys_signal(SIGURG, SIG_IGN);
	ec64sys_signal(SIGXCPU, SIG_IGN);
	ec64sys_signal(SIGXFSZ, SIG_IGN);
	ec64sys_signal(SIGVTALRM, SIG_IGN);
	ec64sys_signal(SIGPROF, SIG_IGN);
	ec64sys_signal(SIGWINCH, SIG_IGN);
	ec64sys_signal(SIGIO, SIG_IGN);

	/* set default name for logging */

		{
		b			path[] = "ec64.log";

		memory_copy(log_path, path, string_length(path) + 1);
		}

	/* process cmdline */

	ec64io_process_cmdline(argument_counter, arguments);

	/* pass program control to ec64ui_start() */

	ec64ui_start();

	/* exit */

	ec64io_end();
	}



/****************************************************************************
 * ec64io_initialize ********************************************************
 ****************************************************************************/
void	ec64io_initialize(
	void)

	{
#ifndef EC64_ENABLE_KDSETMODE

	/* the gpm hack should be initialized before the tty */

	gpm_initialize();
#endif
	tty_initialize();
	video_initialize(&video_options);
	audio_initialize(&audio_options);
	sync_initialize();

	/* set filesystem rights (if this binary runs setuid root, we have
	   to do this) */

	ec64sys_setfsuid(ec64sys_getuid());
	ec64sys_setfsgid(ec64sys_getgid());
	}



/****************************************************************************
 * ec64io_restore ***********************************************************
 ****************************************************************************/
void	ec64io_restore(
	void)

	{
	sync_restore();
	audio_restore();
	video_restore();
	tty_restore();
#ifndef EC64_ENABLE_KDSETMODE
	gpm_restore();
#endif
	}



/****************************************************************************
 * ec64io_end ***************************************************************
 ****************************************************************************/
void	ec64io_end(
	void)

	{
	ec64io_restore();
	tty_print_text(STRING_TYPE "ec64-" EC64_VERSION ", Author:   Karsten Scheibler\n"
			"           Homepage: http://unusedino.de/ec64/\n"
			"           eMail:    ec64@unusedino.de\n\n");
	ec64sys_exit(0);
	}



/****************************************************************************
 * ec64io_exit **************************************************************
 ****************************************************************************/
void	ec64io_exit(
	const d				number)

	{
	ec64sys_exit(number);
	}



/****************************************************************************
 * ec64io_error *************************************************************
 ****************************************************************************/
void	ec64io_error(
	const b				*text)

	{
	error_text_exit(text);
	}



/****************************************************************************
 * ec64io_dump **************************************************************
 ****************************************************************************/
void	ec64io_dump(
	const void			*data,
	const d				data_length)

	{
	static d			dumps = 0;
	b				dump_name[64];
	sd				dump_handle;

	string_print(dump_name, 63, STRING_TYPE "ec64dump-" STRING_HEX32, dumps);
	if ((dump_handle = ec64sys_open(dump_name, (O_WRONLY | O_TRUNC | O_CREAT), (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH))) > 0)
		{
		ec64sys_write(dump_handle, data, data_length);
		ec64sys_close(dump_handle);
		dumps ++;
		}
	}



/****************************************************************************
 * ec64io_log_path **********************************************************
 ****************************************************************************/
void	ec64io_log_path(
	const b				*path)

	{
	d				length = string_length(path);

	/*
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	TODO: if name is too long throw an error
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/

	if (length > sizeof(log_path) - 1) length = sizeof(log_path) - 1;
	memory_copy(log_path, path, length);
	log_path[length] = 0;
	}



/****************************************************************************
 * ec64io_log_write *********************************************************
 ****************************************************************************/
sd	ec64io_log_write(
	const b				*buffer,
	const d				buffer_size)

	{
	sd				log_handle;

	if ((log_handle = ec64sys_open(log_path, (O_WRONLY | O_APPEND | O_CREAT), (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH))) > 0)
		{
		if (ec64sys_write(log_handle, buffer, buffer_size) == -1) return (error_number());
		ec64sys_close(log_handle);
		return (0);
		}
	else return (error_number());
	}



/****************************************************************************
 * ec64io_get_error_message *************************************************
 ****************************************************************************/
const b	*ec64io_get_error_message(
	void)

	{
	return (error_message());
	}



/****************************************************************************
 * ec64io_get_status ********************************************************
 ****************************************************************************/
d	ec64io_get_status(
	void)

	{
	d				status = 0;
	d				character = tty_get_character();
	static d			last_character = 0;
	static d			line_delay = 0;
	static d			cursor_delay = 0;
	static b			monitor = 0;
	static b			line = 0;
	static b			cursor = 0;

	/* calculate the time needed for the last frame */

	sync_get_time();

	/* make the cursor blink */
	/*
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	TODO: let the cursor blink at a fixed rate, don't let it speed up
	      when sync is off
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/

	if (input_mode == INPUT_MODE_MONITOR)
		{
		if (cursor_delay == 0)
			{
			cursor_delay = 20;
			cursor = 1 - cursor;
			if (cursor) video_monitor_cursor_on();
			else video_monitor_cursor_off();
			}
		else cursor_delay --;
		}

	/* delay for status line */

	if (line_delay > 0)
		{
		line_delay --;
		if ((line_delay == 0) && (! (line))) status |= (STATUS_LINE_OFF | STATUS_RESIZE_VIDEO);
		}

	/* do something if the input has changed */

	if (character == last_character) return (status);
	last_character = character;

	/* react on the F9 key */

	if (character == 0x43)
		{
		input_mode ++;
		if (input_mode > INPUT_MODE_MONITOR) input_mode = 0;
		if (input_mode == INPUT_MODE_MONITOR)
			{
			if ((! (monitor)) && (! (video_options.flags & TVIDEO_MONITOR_ALWAYS_VISIBLE)))
				status |= (STATUS_MONITOR_ON | STATUS_RESIZE_VIDEO);
			}
		else
			{
			if ((! (monitor)) && (! (video_options.flags & TVIDEO_MONITOR_ALWAYS_VISIBLE)))
				status |= (STATUS_MONITOR_OFF | STATUS_RESIZE_VIDEO);
			video_monitor_cursor_off();
			}
		if (! (line))
			{
			line_delay = 50;
			status |= (STATUS_LINE_ON | STATUS_RESIZE_VIDEO);
			}
		return (status);
		}

	/* react on the F10 key */

	if (character == 0x44)
		{
		line = 1 - line;
		if (line) status |= (STATUS_LINE_ON | STATUS_RESIZE_VIDEO);
		else status |= (STATUS_LINE_OFF | STATUS_RESIZE_VIDEO);
		return (status);
		}

	/* react on the F11 key */

	if (character == 0x57)
		{
		monitor = 1 - monitor;
		if (! (input_mode == INPUT_MODE_MONITOR))
			{
			if (monitor) status |= STATUS_MONITOR_ON;
			else status |= STATUS_MONITOR_OFF;
			if (! (video_options.flags & TVIDEO_MONITOR_ALWAYS_VISIBLE)) status |= STATUS_RESIZE_VIDEO;
			}
		return (status);
		}

#ifdef EC64_ENABLE_SCREENSHOT_SUPPORT

	/* react on the F12 key */

	if (character == 0x58)
		{
		video_make_screenshot();
		return (status);
		}
#endif

	/* copy character to the input source according to the actual mode */

	c64_keyboard_input = 0;
	c64_joystick1_input = 0;
	c64_joystick2_input = 0;
	monitor_input = 0;
	if (input_mode == INPUT_MODE_C64_KEYBOARD) c64_keyboard_input = character;
	if (input_mode == INPUT_MODE_C64_JOYSTICK1) c64_joystick1_input = character;
	if (input_mode == INPUT_MODE_C64_JOYSTICK2) c64_joystick2_input = character;
	if (input_mode == INPUT_MODE_MONITOR) monitor_input = character;
	return (status);
	}



/****************************************************************************
 * ec64io_set_status ********************************************************
 ****************************************************************************/
void	ec64io_set_status(
	const d				status)

	{
	if (status & STATUS_RESIZE_VIDEO)
		{
		if (status & STATUS_MONITOR_ON) video_monitor_on(&video_options);
		if (status & STATUS_MONITOR_OFF) video_monitor_off(&video_options);
		if (status & STATUS_LINE_ON) video_status_on(&video_options);
		if (status & STATUS_LINE_OFF) video_status_off(&video_options);
		}
	}



/****************************************************************************
 * ec64io_c64_get_video_options *********************************************
 ****************************************************************************/
const tvideo	*ec64io_c64_get_video_options(
	void)

	{
	return (&video_options);
	}



/****************************************************************************
 * ec64io_c64_draw_screen ***************************************************
 ****************************************************************************/
void	ec64io_c64_draw_screen(
	void)

	{
	video_draw_c64_screen();
	}



/****************************************************************************
 * ec64io_c64_get_audio_options *********************************************
 ****************************************************************************/
const taudio	*ec64io_c64_get_audio_options(
	void)

	{
	return (&audio_options);
	}



/****************************************************************************
 * ec64io_c64_play_samples **************************************************
 ****************************************************************************/
void	ec64io_c64_play_samples(
	void)

	{
	audio_play_samples();
	}



/****************************************************************************
 * ec64io_c64_get_keyboard **************************************************
 ****************************************************************************/
const b	*ec64io_c64_get_keyboard(
	void)

	{
	static b			pressed_keys[0x80] = { 0 };
	static d			translate_table[0x80] =
					{

					/* 0x00 - 0x3f */

					0, 0, 0x31, 0x32,
					0x33, 0x34, 0x35, 0x36,
					0x37, 0x38, 0x39, 0x30,
					0x2b, 0x2d, 0x22, 0x24,

					0x11, 0x17, 0x05, 0x12,
					0x14, 0x19, 0x15, 0x09,
					0x0f, 0x10, 0x40, 0x2a,
					0x25, 0, 0x01, 0x13,

					0x04, 0x06, 0x07, 0x08,
					0x0a, 0x0b, 0x0c, 0x3a,
					0x3b, 0x1f, 0x1b, 0x1c,
					0x1a, 0x18, 0x03, 0x16,

					0x02, 0x0e, 0x0d, 0x3c,
					0x3e, 0x3f, 0, 0,
					0, 0x20, 0, 0x29,
					0x1d29, 0x2c, 0x1d2c, 0x2e,

					/* 0x40 - 0x7f */

					0x1d2e, 0x2f, 0x1d2f, 0,
					0, 0, 0, 0,
					0, 0, 0, 0,
					0, 0, 0, 0,

					0, 0, 0, 0,
					0, 0, 0x3d, 0,
					0, 0, 0, 0,
					0, 0, 0, 0,

					0x1d27, 0x27, 0x1d28, 0x28,
					0x21, 0x1d22, 0x22, 0x1e,
					0, 0, 0, 0,
					0, 0, 0, 0,

					0, 0, 0, 0,
					0, 0, 0, 0,
					0, 0, 0, 0,
					0, 0, 0, 0x41
					};
	static b			c64_keycodes[16];
	d				metakeys = (c64_keyboard_input >> 8) & 0x1ff;
	d				keycode = c64_keyboard_input & 0xff;
	d				offset = 0;
	d				counter;

	if (metakeys)
		{
		if (metakeys & (1 << KG_CTRL)) c64_keycodes[offset ++] = 0x23;
		if (metakeys & (1 << KG_ALT)) c64_keycodes[offset ++] = 0x26;
		if (metakeys & (1 << KG_SHIFTL)) c64_keycodes[offset ++] = 0x1b;
		if (metakeys & (1 << KG_SHIFTR)) c64_keycodes[offset ++] = 0x1d;
		}
	if (keycode & 0x80) pressed_keys[keycode & 0x7f] = 0;
	else pressed_keys[keycode] = 1;
	for (counter = 0; counter < 0x80; counter ++)
		if (pressed_keys[counter])
			{
			if ((keycode = translate_table[counter] & 0xff) != 0)
				c64_keycodes[offset ++] = keycode;
			if ((keycode = (translate_table[counter] >> 8) & 0xff) != 0)
				c64_keycodes[offset ++] = keycode;
			}
	c64_keycodes[offset ++] = 0;
	return (c64_keycodes);
	}



/****************************************************************************
 * ec64io_c64_get_joystick1 *************************************************
 ****************************************************************************/
d	ec64io_c64_get_joystick1(
	void)

	{
	static d			joystick_bits = 0;

	switch (c64_joystick1_input)
		{
		case 0x39:
			joystick_bits |= 0x10;
			break;
		case 0xb9:
			joystick_bits &= ~ 0x10;
			break;
		case 0x60:
			joystick_bits |= 0x01;
			break;
		case 0xe0:
			joystick_bits &= ~ 0x01;
			break;
		case 0x61:
			joystick_bits |= 0x02;
			break;
		case 0xe1:
			joystick_bits &= ~ 0x02;
			break;
		case 0x62:
			joystick_bits |= 0x04;
			break;
		case 0xe2:
			joystick_bits &= ~ 0x04;
			break;
		case 0x63:
			joystick_bits |= 0x08;
			break;
		case 0xe3:
			joystick_bits &= ~ 0x08;
			break;
		default:
			joystick_bits = 0;
			break;
		}
	return (joystick_bits);
	}



/****************************************************************************
 * ec64io_c64_get_joystick2 *************************************************
 ****************************************************************************/
d	ec64io_c64_get_joystick2(
	void)

	{
	static d			joystick_bits = 0;

	switch (c64_joystick2_input)
		{
		case 0x39:
			joystick_bits |= 0x10;
			break;
		case 0xb9:
			joystick_bits &= ~ 0x10;
			break;
		case 0x60:
			joystick_bits |= 0x01;
			break;
		case 0xe0:
			joystick_bits &= ~ 0x01;
			break;
		case 0x61:
			joystick_bits |= 0x02;
			break;
		case 0xe1:
			joystick_bits &= ~ 0x02;
			break;
		case 0x62:
			joystick_bits |= 0x04;
			break;
		case 0xe2:
			joystick_bits &= ~ 0x04;
			break;
		case 0x63:
			joystick_bits |= 0x08;
			break;
		case 0xe3:
			joystick_bits &= ~ 0x08;
			break;
		default:
			joystick_bits = 0;
			break;
		}
	return (joystick_bits);
	}



/****************************************************************************
 * ec64io_c64_draw_status ***************************************************
 ****************************************************************************/
void	ec64io_c64_draw_status(
	const tc64stat			*c64_status)

	{
	}



/****************************************************************************
 * ec64io_c1541_draw_status *************************************************
 ****************************************************************************/
void	ec64io_c1541_draw_status(
	const tc1541stat		*c1541_status)

	{
	static d			frame_time;
	static d			frame_counter = 0;

	if (frame_counter >= 10)
		{
		frame_time = sync_frame_time();
		frame_counter = 0;
		}
	else frame_counter ++;
	video_draw_status(NULL, c1541_status, input_mode, frame_time);
	}



/****************************************************************************
 * ec64io_cmdline_d64_path **************************************************
 ****************************************************************************/
const b	*ec64io_cmdline_d64_path(
	void)

	{
	return (d64_path);
	}



/****************************************************************************
 * ec64io_cmdline_script_path ***********************************************
 ****************************************************************************/
const b	*ec64io_cmdline_script_path(
	void)

	{
	return (script_path);
	}



/****************************************************************************
 * ec64io_file_load *********************************************************
 ****************************************************************************/
sd	ec64io_file_load(
	const b				*path,
	b				*data,
	const d				length)

	{
	sd				file_handle;

	if ((file_handle = ec64sys_open(path, O_RDONLY)) == -1) return (error_number());
	return (ec64sys_read(file_handle, data, length));
	}



/****************************************************************************
 * ec64io_file_save *********************************************************
 ****************************************************************************/
sd	ec64io_file_save(
	const b				*path,
	const b				*data,
	const d				length)

	{
	sd				file_handle;

	if ((file_handle = ec64sys_open(path, (O_WRONLY | O_CREAT | O_TRUNC), (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH))) == -1) return (error_number());
	return (ec64sys_write(file_handle, data, length));
	}



/****************************************************************************
 * ec64io_tty_print_text ****************************************************
 ****************************************************************************/
void	ec64io_tty_print_text(
	const b				*text)

	{
	tty_print_text(text);
	}



/****************************************************************************
 * ec64io_tty_print_error ***************************************************
 ****************************************************************************/
void	ec64io_tty_print_error(
	const b				*error)

	{
	tty_print_error(error);
	}



/****************************************************************************
 * ec64io_sync_sleep ********************************************************
 ****************************************************************************/
void	ec64io_sync_sleep(
	void)

	{
	audio_sync();
	}



/****************************************************************************
 * ec64io_monitor_get_character *********************************************
 ****************************************************************************/
b	ec64io_monitor_get_character(
	void)

	{
	static d			last_character = 0;
	d				character = tty_translate_character(monitor_input);

	if (character == last_character) return (0);
	last_character = character;
	if (character & 0x80) character = 0;
	return (character);
	}



/****************************************************************************
 * ec64io_monitor_print_text ************************************************
 ****************************************************************************/
void	ec64io_monitor_print_text(
	const b				*text)

	{
	video_draw_text(text);
	}



/****************************************************************************
 * ec64io_monitor_print_notice **********************************************
 ****************************************************************************/
void	ec64io_monitor_print_notice(
	const b				*notice)

	{

	/*
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	TODO: if monitor is not visible it should be made visible
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/

	video_draw_text(notice);
	}



/****************************************************************************
 * ec64io_monitor_print_error ***********************************************
 ****************************************************************************/
void	ec64io_monitor_print_error(
	const b				*error)

	{

	/*
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	TODO: if monitor is not visible it should be made visible
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/

	video_draw_text(error);
	}



/****************************************************************************
 * ec64io_monitor_new_line **************************************************
 ****************************************************************************/
void	ec64io_monitor_new_line(
	void)

	{
	video_monitor_new_line();
	}



/****************************************************************************
 * ec64io_monitor_delete_character ******************************************
 ****************************************************************************/
void	ec64io_monitor_delete_character(
	void)

	{
	video_monitor_delete_character();
	}
/******************************************************** ec64@unusedino.de */

