/****************************************************************************
 ****************************************************************************
 *
 * error.c
 *
 ****************************************************************************
 ****************************************************************************/





#include "error.h"
#include "ec64sys.h"
#include "ec64io.h"
#include "string.h"
#include "video.h"
#include "audio.h"
#include "tty.h"



static const b				program_name[] = "ec64: ";


/****************************************************************************
 * error_number *************************************************************
 ****************************************************************************/
const sd error_number(
	void)

	{
	return (ec64sys_error_number);
	}



/****************************************************************************
 * error_get_message ********************************************************
 ****************************************************************************/
const b	*error_get_message(
	sd				error_number)

	{
	static d			error_numbers = 0;
	static const b			*error_messages[] =
					{
					STRING_TYPE "Unknown Error",
					STRING_TYPE "Operation not permitted",
					STRING_TYPE "No such file or directory",
					STRING_TYPE "No such process",
					STRING_TYPE "Interrupted system call",
					STRING_TYPE "I/O error",
					STRING_TYPE "No such device or address",
					STRING_TYPE "Arg list too long",
					STRING_TYPE "Exec format error",
					STRING_TYPE "Bad file number",
					STRING_TYPE "No child processes",
					STRING_TYPE "Try again",
					STRING_TYPE "Out of memory" ,
					STRING_TYPE "Permission denied",
					STRING_TYPE "Bad address",
					STRING_TYPE "Block device required",
					STRING_TYPE "Device or resource busy",
					STRING_TYPE "File exists",
					STRING_TYPE "Cross-device link",
					STRING_TYPE "No such device",
					STRING_TYPE "Not a directory",
					STRING_TYPE "Is a directory",
					STRING_TYPE "Invalid argument",
					STRING_TYPE "File table overflow",
					STRING_TYPE "Too many open files",
					STRING_TYPE "Not a typewriter",
					STRING_TYPE "Text file busy",
					STRING_TYPE "File too large",
					STRING_TYPE "No space left on device",
					STRING_TYPE "Illegal seek",
					STRING_TYPE "Read-only file system",
					STRING_TYPE "Too many links",
					STRING_TYPE "Broken pipe",
					STRING_TYPE "Math argument out of domain of func",
					STRING_TYPE "Math result not representable",
					STRING_TYPE "Resource deadlock would occur",
					STRING_TYPE "File name too long",
					STRING_TYPE "No record locks available",
					STRING_TYPE "Function not implemented",
					STRING_TYPE "Directory not empty",
					STRING_TYPE "Too many symbolic links encountered",
					STRING_TYPE "Operation would block",
					STRING_TYPE "No message of desired type",
					STRING_TYPE "Identifier removed",
					STRING_TYPE "Channel number out of range",
					STRING_TYPE "Level 2 not synchronized",
					STRING_TYPE "Level 3 halted",
					STRING_TYPE "Level 3 reset",
					STRING_TYPE "Link number out of range",
					STRING_TYPE "Protocol driver not attached",
					STRING_TYPE "No CSI structure available",
					STRING_TYPE "Level 2 halted",
					STRING_TYPE "Invalid exchange",
					STRING_TYPE "Invalid request descriptor",
					STRING_TYPE "Exchange full",
					STRING_TYPE "No anode",
					STRING_TYPE "Invalid request code",
					STRING_TYPE "Invalid slot",
					STRING_TYPE "unused errormessage!",
					STRING_TYPE "Bad font file format",
					STRING_TYPE "Device not a stream",
					STRING_TYPE "No data available",
					STRING_TYPE "Timer expired",
					STRING_TYPE "Out of streams resources",
					STRING_TYPE "Machine is not on the network",
					STRING_TYPE "Package not installed",
					STRING_TYPE "Object is remote",
					STRING_TYPE "Link has been severed",
					STRING_TYPE "Advertise error",
					STRING_TYPE "Srmount error",
					STRING_TYPE "Communication error on send",
					STRING_TYPE "Protocol error",
					STRING_TYPE "Multihop attempted",
					STRING_TYPE "RFS specific error",
					STRING_TYPE "Not a data message",
					STRING_TYPE "Value too large for defined data type",
					STRING_TYPE "Name not unique on network",
					STRING_TYPE "File descriptor in bad state",
					STRING_TYPE "Remote address changed",
					STRING_TYPE "Can not access a needed shared library",
					STRING_TYPE "Accessing a corrupted shared library",
					STRING_TYPE ".lib section in a.out corrupted",
					STRING_TYPE "Attempting to link in too many shared libraries",
					STRING_TYPE "Cannot exec a shared library directly",
					STRING_TYPE "Illegal byte sequence",
					STRING_TYPE "Interrupted system call should be restarted",
					STRING_TYPE "Streams pipe error",
					STRING_TYPE "Too many users",
					STRING_TYPE "Socket operation on non-socket",
					STRING_TYPE "Destination address required",
					STRING_TYPE "Message too long",
					STRING_TYPE "Protocol wrong type for socket",
					STRING_TYPE "Protocol not available",
					STRING_TYPE "Protocol not supported",
					STRING_TYPE "Socket type not supported",
					STRING_TYPE "Operation not supported on transport endpoint",
					STRING_TYPE "Protocol family not supported",
					STRING_TYPE "Address family not supported by protocol",
					STRING_TYPE "Address already in use",
					STRING_TYPE "Cannot assign requested address",
					STRING_TYPE "Network is down",
					STRING_TYPE "Network is unreachable",
					STRING_TYPE "Network dropped connection because of reset",
					STRING_TYPE "Software caused connection abort",
					STRING_TYPE "Connection reset by peer",
					STRING_TYPE "No buffer space available",
					STRING_TYPE "Transport endpoint is already connected",
					STRING_TYPE "Transport endpoint is not connected",
					STRING_TYPE "Cannot send after transport endpoint shutdown",
					STRING_TYPE "Too many references: cannot splice",
					STRING_TYPE "Connection timed out",
					STRING_TYPE "Connection refused",
					STRING_TYPE "Host is down",
					STRING_TYPE "No route to host",
					STRING_TYPE "Operation already in progress",
					STRING_TYPE "Operation now in progress",
					STRING_TYPE "Stale NFS file handle",
					STRING_TYPE "Structure needs cleaning",
					STRING_TYPE "Not a XENIX named type file",
					STRING_TYPE "No XENIX semaphores available",
					STRING_TYPE "Is a named type file",
					STRING_TYPE "Remote I/O error",
					STRING_TYPE "Quota exceeded",
					STRING_TYPE "No medium found",
					STRING_TYPE "Wrong medium type",
		        		NULL
					};

	error_number = (NOT32 - error_number) + 1;
	while (error_messages[error_numbers] != 0) error_numbers ++;
	if (error_number > error_numbers) error_number = 0;
	return error_messages[error_number];
	}


/****************************************************************************
 * error_message ************************************************************
 ****************************************************************************/
const b	*error_message(
	void)

	{
	return (error_get_message(error_number()));
	}



/****************************************************************************
 * error_exit ***************************************************************
 ****************************************************************************/
void	error_exit(
	void)

	{
	ec64io_restore();
	ec64sys_exit(1);
	}



/****************************************************************************
 * error_message_exit *******************************************************
 ****************************************************************************/
void	error_message_exit(
	void)

	{
	b				string[1024];
	sd				number = error_number();
    
	ec64io_restore();
	string_print(string, 1023, STRING_TYPE STRING_STR STRING_STR "\n", program_name, error_get_message(number));
	tty_print_error(string);
	ec64sys_exit(1);
	}



/****************************************************************************
 * error_text_exit **********************************************************
 ****************************************************************************/
void	error_text_exit(
	const b				*error_text)

	{
	b				string[1024];

	ec64io_restore();
	string_print(string, 1023, STRING_TYPE STRING_STR STRING_STR "\n", program_name, error_text);
	tty_print_error(string);
	ec64sys_exit(1);
	}



/****************************************************************************
 * error_text_message_exit **************************************************
 ****************************************************************************/
void	error_text_message_exit(
	const b				*error_text)

	{
	b				string[1024];
	sd				number = error_number();

	ec64io_restore();
	string_print(string, 1023, STRING_TYPE STRING_STR STRING_STR "\n", program_name, error_text);
	tty_print_error(string);
	string_print(string, 1023, STRING_TYPE STRING_STR STRING_STR "\n", program_name, error_get_message(number));
	tty_print_error(string);
	ec64sys_exit(1);
	}
/******************************************************** ec64@unusedino.de */

