/****************************************************************************
 ****************************************************************************
 *
 * sync.c
 *
 ****************************************************************************
 ****************************************************************************/





#include "sync.h"
#include "ec64sys.h"



static d				time_sum = 0;
static d				time_sum_no_sleep = 0;
static d				time_table_entries = 0;
static d				time_table_offset = 0;
static d				time_table[SYNC_TIME_TABLE_SIZE];
static d				time_table_no_sleep[SYNC_TIME_TABLE_SIZE];
static d				time_until_sleep = 0;
static d				time_sleeped = 0;
static sd				time_difference = 0;



/****************************************************************************
 * sync_initialize **********************************************************
 ****************************************************************************/
void	sync_initialize(
	void)

	{
	d				counter;
    
	for (counter = 0; counter < SYNC_TIME_TABLE_SIZE; counter ++)
		time_table[counter] = 0;
	ec64sys_time();
	}



/****************************************************************************
 * sync_restore *************************************************************
 ****************************************************************************/
void	sync_restore(
	void)

	{
	}



/****************************************************************************
 * sync_frame_time **********************************************************
 ****************************************************************************/
const d	sync_frame_time(
	void)

	{
	if (time_table_entries == 0) return (0);
	return (time_sum_no_sleep / time_table_entries);
	}



/****************************************************************************
 * sync_get_time ************************************************************
 ****************************************************************************/
void	sync_get_time(
	void)

	{
	d				time = ec64sys_time();
    
	if (time > SYNC_MAX_FRAME_TIME) time = SYNC_MAX_FRAME_TIME;
	time_sum -= time_table[time_table_offset];
	time_sum += time + time_until_sleep + time_sleeped;
	time_table[time_table_offset] = time + time_until_sleep + time_sleeped;
	time_sum_no_sleep -= time_table_no_sleep[time_table_offset];
	time_sum_no_sleep += time + time_until_sleep;
	time_table_no_sleep[time_table_offset] = time + time_until_sleep;
	time_difference = (SYNC_FRAME_TIME * time_table_entries) - time_sum;
	time_table_offset ++;
	if (time_table_offset >= SYNC_TIME_TABLE_SIZE) time_table_offset = 0;
	if (time_table_entries < SYNC_TIME_TABLE_SIZE) time_table_entries ++;
	}



/****************************************************************************
 * sync_delay ***************************************************************
 ****************************************************************************/
d	sync_delay(
	d				time_to_sleep)

	{
	if (time_to_sleep > SYNC_SLEEP_TIME)
		{
		register d		sleep_time = time_to_sleep;

		time_until_sleep = ec64sys_time();
		if (time_to_sleep > SYNC_FRAME_TIME)
			{
			sleep_time = SYNC_FRAME_TIME;
			time_to_sleep -= SYNC_FRAME_TIME;
			}
		else time_to_sleep = 0;
		ec64sys_sleep(sleep_time);
		time_sleeped = ec64sys_time();
		}
	else
		{
		time_until_sleep = 0;
		time_sleeped = 0;
		}
	if (time_to_sleep > (SYNC_FRAME_TIME + SYNC_SLEEP_TIME)) time_to_sleep = SYNC_FRAME_TIME + SYNC_SLEEP_TIME;
	return (time_to_sleep);
	}



/****************************************************************************
 * sync_sleep ***************************************************************
 ****************************************************************************/
void	sync_sleep(
	void)

	{
	sync_delay(time_difference);
	}
/******************************************************** ec64@unusedino.de */

