/****************************************************************************
 ****************************************************************************
 *
 * vga.h
 *
 ****************************************************************************
 ****************************************************************************/





#ifndef VGA_H
#define VGA_H

#include "defines.h"
#include "ec64types.h"
#include "video.h"

#define VGA_CRT_REGISTERS		24
#define VGA_ATTRIBUTE_REGISTERS		21
#define VGA_GRAPHIC_REGISTERS		9
#define VGA_SEQUENCER_REGISTERS		5
#define VGA_MISC_REGISTERS		1

#define VGA_IO_BASE			0x3c0
#define VGA_IO_SIZE			0x20
#define VGA_IO_BASE_MONOCHROME1		0x3b4
#define VGA_IO_SIZE_MONOCHROME1		2
#define VGA_IO_BASE_MONOCHROME2		0x3ba
#define VGA_IO_SIZE_MONOCHROME2		1

#define VGA_ATTRIBUTE_INDEX		0x3c0
#define VGA_ATTRIBUTE_DATA_WRITE	0x3c0
#define VGA_ATTRIBUTE_DATA_READ		0x3c1
#define VGA_MISC_DATA_WRITE		0x3c2
#define VGA_SEQUENCER_INDEX		0x3c4
#define VGA_SEQUENCER_DATA		0x3c5
#define VGA_PEL_MASK			0x3c6
#define VGA_PEL_INDEX_READ		0x3c7
#define VGA_PEL_INDEX_WRITE		0x3c8
#define VGA_PEL_DATA			0x3c9
#define VGA_MISC_DATA_READ		0x3cc
#define VGA_GRAPHIC_INDEX		0x3ce
#define VGA_GRAPHIC_DATA		0x3cf
#define VGA_CRT_INDEX			0x3d4
#define VGA_CRT_INDEX_MONOCHROME	0x3b4
#define VGA_CRT_DATA			0x3d5
#define VGA_CRT_DATA_MONOCHROME		0x3b5
#define VGA_INPUT_STATUS		0x3da
#define VGA_INPUT_STATUS_MONOCHROME	0x3ba

#define VGA_X_RESOLUTION		360	/* max possible value */
#define VGA_Y_RESOLUTION		240	/* max possible value */

#define VGA_PLANES			4

#define VGA_CHARSET_SIZE		0x40000	/* 256 KB: the whole VGA
						   memory */

const sd				vga_initialize(tvidinit *);
void					vga_restore(void);
void					vga_set_mode_string(const b *);
void					vga_set_options(tvidinit *);
void					vga_draw_text(const d, const d, const b *);
void					vga_draw_led(const d, const d, const b);
void					vga_draw_c64_screen(void);
void					vga_area_clear(const d, const d, const d, const d);
void					vga_area_move(const d, const d, const d, const d, const d, const d);
#ifdef VGA_ENABLE_SCREENSHOT_SUPPORT
void					vga_get_line(b *, const d, const d);
#endif



#endif
/******************************************************** ec64@unusedino.de */

