/****************************************************************************
 ****************************************************************************
 *
 * video.c
 *
 ****************************************************************************
 ****************************************************************************/





#include "video.h"
#include "ec64sys.h"
#include "draw.h"
#include "error.h"
#include "fb.h"
#include "memory.h"
#include "string.h"
#include "vga.h"



#define VIDEO_MONITOR_LINES		16
#define VIDEO_MONITOR_ROWS		128



static d				video_state = 0;
static d				video_functions_registered = 0;
static sd				video_functions_selected = -1;
static tvidfunc				video_functions[2];
static d				video_x_resolution;
static d				video_y_resolution;
static d				video_max_c64_x_resolution;
static d				video_max_c64_y_resolution;
static d				video_c64_x_offset;
static d				video_c64_y_offset;
static d				video_status_x_offset;
static d				video_status_y_offset;
static b				video_monitor_buffer[(VIDEO_MONITOR_LINES * (VIDEO_MONITOR_ROWS + 1))];
static d				video_monitor_x_resolution;
static d				video_monitor_y_resolution;
static d				video_monitor_x_offset;
static d				video_monitor_y_offset;
static d				video_monitor_x_characters;
static d				video_monitor_y_characters;
static d				video_monitor_x_cursor;
static d				video_monitor_y_cursor;
struct
	{
	b				red;
	b				green;
	b				blue;
	}				video_c64_colors[16] =
					{
#ifdef VIDEO_USE_VICE_PALETTE
					  { 0x00, 0x00, 0x00 },
					  { 0xfd, 0xfe, 0xfc },
					  { 0xbe, 0x1a, 0x24 },
					  { 0x30, 0xe6, 0xc6 },
					  { 0xb4, 0x1a, 0xe2 },
					  { 0x1f, 0xd2, 0x1e },
					  { 0x21, 0x1b, 0xae },
					  { 0xdf, 0xf6, 0x0a },
					  { 0xb8, 0x41, 0x04 },
					  { 0x6a, 0x33, 0x04 },
					  { 0xfe, 0x4a, 0x57 },
					  { 0x42, 0x45, 0x40 },
					  { 0x70, 0x74, 0x6f },
					  { 0x59, 0xfe, 0x59 },
					  { 0x5f, 0x53, 0xfe },
					  { 0xa4, 0xa7, 0xa2 }
#elif defined(VIDEO_USE_PEPTO_PALETTE)
					  { 0x00, 0x00, 0x00 },
					  { 0xff, 0xff, 0xff },
					  { 0x68, 0x37, 0x2b },
					  { 0x70, 0xa4, 0xb2 },
					  { 0x6f, 0x3d, 0x86 },
					  { 0x58, 0x8d, 0x43 },
					  { 0x35, 0x28, 0x79 },
					  { 0xb8, 0xc7, 0x6f },
					  { 0x6f, 0x4f, 0x25 },
					  { 0x43, 0x39, 0x00 },
					  { 0x9a, 0x67, 0x59 },
					  { 0x44, 0x44, 0x44 },
					  { 0x6c, 0x6c, 0x6c },
					  { 0x9a, 0xd2, 0x84 },
					  { 0x6c, 0x5e, 0xb5 },
					  { 0x95, 0x95, 0x95 }
#else
					  { 0x00, 0x00, 0x00 },
					  { 0xe0, 0xe0, 0xe0 },
					  { 0x60, 0x20, 0x20 },
					  { 0x00, 0xe0, 0xe0 },
					  { 0x60, 0x20, 0x60 },
					  { 0x00, 0x60, 0x40 },
					  { 0x00, 0x00, 0x60 },
					  { 0xe0, 0xe0, 0x60 },
					  { 0x60, 0x40, 0x20 },
					  { 0x40, 0x20, 0x00 },
					  { 0xe0, 0x60, 0x60 },
					  { 0x20, 0x20, 0x20 },
					  { 0x40, 0x40, 0x40 },
					  { 0x60, 0xe0, 0x60 },
					  { 0x60, 0x60, 0xe0 },
					  { 0x60, 0x60, 0x60 }
#endif
					};



#define VIDEO_REGISTERED		0x01
#define VIDEO_INITIALIZED		0x02
#define VIDEO_STATUS			0x04
#define VIDEO_MONITOR_ALWAYS_VISIBLE	0x08
#define VIDEO_MONITOR			0x10

#define VIDEO_STATUS_Y_SIZE		DRAW_Y_CHARACTER_SIZE
#define VIDEO_MONITOR_Y_SIZE		(VIDEO_MONITOR_LINES * DRAW_Y_CHARACTER_SIZE)
#define VIDEO_MONITOR_X_SIZE		(VIDEO_MONITOR_ROWS * DRAW_X_CHARACTER_SIZE)



/****************************************************************************
 * video_register ***********************************************************
 ****************************************************************************/
void	video_register(
	void)

	{
#ifdef VIDEO_ENABLE_FB_SUPPORT
	video_functions[video_functions_registered].initialize = fb_initialize;
	video_functions[video_functions_registered].restore = fb_restore;
	video_functions[video_functions_registered].set_options = fb_set_options;
	video_functions[video_functions_registered].draw_text = fb_draw_text;
	video_functions[video_functions_registered].draw_led = fb_draw_led;
	video_functions[video_functions_registered].draw_c64_screen = fb_draw_c64_screen;
	video_functions[video_functions_registered].area_clear = fb_area_clear;
	video_functions[video_functions_registered].area_move = fb_area_move;
#ifdef VIDEO_ENABLE_SCREENSHOT_SUPPORT
	video_functions[video_functions_registered].get_line = fb_get_line;
#endif
	video_functions_registered ++;
#endif

#ifdef VIDEO_ENABLE_VGA_SUPPORT
	video_functions[video_functions_registered].initialize = vga_initialize;
	video_functions[video_functions_registered].restore = vga_restore;
	video_functions[video_functions_registered].set_options = vga_set_options;
	video_functions[video_functions_registered].draw_text = vga_draw_text;
	video_functions[video_functions_registered].draw_led = vga_draw_led;
	video_functions[video_functions_registered].draw_c64_screen = vga_draw_c64_screen;
	video_functions[video_functions_registered].area_clear = vga_area_clear;
	video_functions[video_functions_registered].area_move = vga_area_move;
#ifdef VIDEO_ENABLE_SCREENSHOT_SUPPORT
	video_functions[video_functions_registered].get_line = vga_get_line;
#endif
	video_functions_registered ++;
#endif
	}



/****************************************************************************
 * video_initialize *********************************************************
 ****************************************************************************/
void	video_initialize(
	tvideo				*video_options)

	{
	static w			red[16];
	static w			green[16];
	static w			blue[16];
	static w			transparent[16];
	static tcmap			video_colormap = { 0, 16, red, green, blue, transparent };
	tvidinit			options;

	/* register available functions */

	if (! (video_state & VIDEO_REGISTERED))
		{
		video_register();
		video_state |= VIDEO_REGISTERED;
		}

	/* initialize the colormap */

		{
		d			counter;

		for (counter = 0; counter < 16; counter ++)
			{
			red[counter] = ((w) video_c64_colors[counter].red) << 8;
			green[counter] = ((w) video_c64_colors[counter].green) << 8;
			blue[counter] = ((w) video_c64_colors[counter].blue) << 8;
			transparent[counter] = 0;
			}
		}

	/* select video_functions */

		{
		d			counter = 0;
		sd			success = -1;

		options.colormap = &video_colormap;
		options.flags = video_options->flags;
		while ((counter < video_functions_registered) && ((success = video_functions[counter].initialize(&options)) == -1))
			 counter ++;
		if (success == -1)
			error_text_exit(STRING_TYPE "couldn't initialize video");
		video_functions_selected = counter;
		video_state |= VIDEO_INITIALIZED;
		}

	/* get the values */

	video_x_resolution = options.x_resolution;
	video_y_resolution = options.y_resolution;
	video_max_c64_x_resolution = options.c64_x_resolution;
	video_max_c64_y_resolution = options.c64_y_resolution;

	/* check some error conditions */

	if ((video_x_resolution < 320) || (video_y_resolution < 200) || (video_max_c64_x_resolution < 320) || (video_max_c64_y_resolution < 200))
		error_text_exit(STRING_TYPE "available resolution smaller than 320x200");
	if ((video_max_c64_x_resolution > video_x_resolution) || (video_max_c64_y_resolution > video_y_resolution))
		error_text_exit(STRING_TYPE "!INTERNAL FAILURE! video.c: something went wrong with the resolution settings");

	/* check c64 resolution */

		{
		d			center_x = video_options->c64_x_offset + (video_options->c64_x_resolution / 2);
		d			center_y = video_options->c64_y_offset + (video_options->c64_y_resolution / 2);

		if (video_options->c64_x_resolution > video_max_c64_x_resolution)
			{
			video_options->c64_x_resolution = video_max_c64_x_resolution & ~ 0x0f;
			video_options->c64_x_offset = center_x - (video_options->c64_x_resolution / 2);
			}
		if (video_options->c64_y_resolution > video_max_c64_y_resolution)
			{
			video_options->c64_y_resolution = video_max_c64_y_resolution;
			video_options->c64_y_offset = center_y - (video_options->c64_y_resolution / 2);
			}
		}

	/*
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	TODO: this calculations may fail if the draw buffer for the c64
	      screen is larger in x resolution than the physical screen
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/

	/* clear video_monitor_buffer */

	memory_fill(video_monitor_buffer, sizeof(video_monitor_buffer), 0);

	/* set c64 and monitor resolution */

	if ((video_y_resolution - video_options->c64_y_resolution) < VIDEO_MONITOR_Y_SIZE)
		{

		/* center the c64 screen */

		video_c64_x_offset = (video_x_resolution - video_options->c64_x_resolution) / 2;
		video_c64_y_offset = (video_y_resolution - video_options->c64_y_resolution) / 2;
		options.c64_x_offset = video_c64_x_offset;
		options.c64_y_offset = video_c64_y_offset;
		video_functions[video_functions_selected].set_options(&options);

		/* set monitor position (not visible) */

		video_monitor_x_offset = 0;
		video_monitor_y_offset = video_c64_y_offset + video_options->c64_y_resolution - VIDEO_MONITOR_Y_SIZE;
		if (video_x_resolution > VIDEO_MONITOR_X_SIZE) video_monitor_x_resolution = VIDEO_MONITOR_X_SIZE;
		else video_monitor_x_resolution = video_x_resolution;
		video_monitor_y_resolution = VIDEO_MONITOR_Y_SIZE;

		video_monitor_x_characters = video_monitor_x_resolution / DRAW_X_CHARACTER_SIZE;
		video_monitor_y_characters = video_monitor_y_resolution / DRAW_Y_CHARACTER_SIZE;
		video_monitor_x_cursor = 0;
		video_monitor_y_cursor = 0;
		}
	else
		{

		/* center the c64 screen */

		video_c64_x_offset = (video_x_resolution - video_options->c64_x_resolution) / 2;
		video_c64_y_offset = (video_y_resolution - video_options->c64_y_resolution - VIDEO_MONITOR_Y_SIZE) / 2;
		options.c64_x_offset = video_c64_x_offset;
		options.c64_y_offset = video_c64_y_offset;
		video_functions[video_functions_selected].set_options(&options);

		/* set monitor position (always visible) */

		video_monitor_x_offset = 0;
		video_monitor_y_offset = video_c64_y_offset + video_options->c64_y_resolution;
		if (video_x_resolution > VIDEO_MONITOR_X_SIZE) video_monitor_x_resolution = VIDEO_MONITOR_X_SIZE;
		else video_monitor_x_resolution = video_x_resolution;
		video_monitor_y_resolution = VIDEO_MONITOR_Y_SIZE;

		video_monitor_x_characters = video_monitor_x_resolution / DRAW_X_CHARACTER_SIZE;
		video_monitor_y_characters = video_monitor_y_resolution / DRAW_Y_CHARACTER_SIZE;
		video_monitor_x_cursor = 0;
		video_monitor_y_cursor = 0;

		video_options->flags |= TVIDEO_MONITOR_ALWAYS_VISIBLE;
		video_state |= (VIDEO_MONITOR_ALWAYS_VISIBLE | VIDEO_MONITOR);
		}

	/* set the status line position */

	video_status_x_offset = video_c64_x_offset;
	video_status_y_offset = video_c64_y_offset;

	/* parameters needed by ec64core */

	video_options->buffer = options.c64_screen_buffer;
	video_options->decoded_lines_list = options.c64_screen_lines;
	video_options->virtual_x_resolution = options.virtual_x_resolution;

	/* clear the video memory */

	video_functions[video_functions_selected].area_clear(0, 0, video_x_resolution, video_y_resolution);
	}



/****************************************************************************
 * video_restore ************************************************************
 ****************************************************************************/
void	video_restore(
	void)

	{
	if (video_state & VIDEO_INITIALIZED)
		{
		video_functions[video_functions_selected].restore();
		video_state &= ~ VIDEO_INITIALIZED;
		}
	}



/****************************************************************************
 * video_monitor_scroll *****************************************************
 ****************************************************************************/
void	video_monitor_scroll(
	void)

	{
	if (video_state & VIDEO_INITIALIZED)
		{
		if (video_monitor_x_cursor >= video_monitor_x_characters)
			{
			video_monitor_x_cursor = 0;
			video_monitor_y_cursor ++;
			}
		if (video_monitor_y_cursor >= video_monitor_y_characters)
			{
			if (video_state & VIDEO_MONITOR)
				{
				video_functions[video_functions_selected].area_move(
					video_monitor_x_offset,
					(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE),
					(video_monitor_x_offset + DRAW_X_CHARACTER_SIZE * video_monitor_x_characters),
					(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE * video_monitor_y_characters),
					video_monitor_x_offset,
					video_monitor_y_offset);
				video_functions[video_functions_selected].area_clear(
					video_monitor_x_offset,
					(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE * (video_monitor_y_characters - 1)),
					(video_monitor_x_offset + DRAW_X_CHARACTER_SIZE * video_monitor_x_characters),
					(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE * video_monitor_y_characters));
				}
			if (! (video_state & VIDEO_MONITOR_ALWAYS_VISIBLE))
				{
				d	line;

				for (line = 0; line < (VIDEO_MONITOR_LINES - 1); line ++)
					{
					memory_copy(
						(video_monitor_buffer + (VIDEO_MONITOR_ROWS + 1) * line),
						(video_monitor_buffer + (VIDEO_MONITOR_ROWS + 1) * (line + 1)),
						(VIDEO_MONITOR_ROWS + 1));
					}
				memory_fill(
					(video_monitor_buffer + (VIDEO_MONITOR_ROWS + 1) * (VIDEO_MONITOR_LINES - 1)),
					(VIDEO_MONITOR_ROWS + 1),
					0);
				}
			video_monitor_y_cursor --;
			}
		}
	}



/****************************************************************************
 * video_draw_text **********************************************************
 ****************************************************************************/
void	video_draw_text(
	const b				*text)

	{
	if (video_state & VIDEO_INITIALIZED)
		{
		d			text_length = string_length(text);
		d			text_offset = 0;
		d			line_length;
		b			string[VIDEO_MONITOR_ROWS];

		while (text_length > 0)
			{
			line_length = video_monitor_x_characters - video_monitor_x_cursor;
			if (text_length > line_length)
				{
				memory_copy(string, (text + text_offset), line_length);
				string[line_length] = 0;
				if (! (video_state & VIDEO_MONITOR_ALWAYS_VISIBLE)) memory_copy(
					(video_monitor_buffer + video_monitor_y_cursor * (VIDEO_MONITOR_ROWS + 1) + video_monitor_x_cursor),
					string,
					line_length);
				if (video_state & VIDEO_MONITOR) video_functions[video_functions_selected].draw_text(
					(video_monitor_x_offset + DRAW_X_CHARACTER_SIZE * video_monitor_x_cursor),
					(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE * video_monitor_y_cursor),
					string);
				video_monitor_x_cursor = 0;
				video_monitor_y_cursor ++;
				text_length -= line_length;
				text_offset += line_length;
				}
			else
				{
				if (! (video_state & VIDEO_MONITOR_ALWAYS_VISIBLE)) memory_copy(
					(video_monitor_buffer + (VIDEO_MONITOR_ROWS + 1) * video_monitor_y_cursor + video_monitor_x_cursor),
					(text + text_offset),
					text_length);
				if (video_state & VIDEO_MONITOR) video_functions[video_functions_selected].draw_text(
					(video_monitor_x_offset + DRAW_X_CHARACTER_SIZE * video_monitor_x_cursor),
					(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE * video_monitor_y_cursor),
					(text + text_offset));
				video_monitor_x_cursor += text_length;
				text_length = 0;
				}
			video_monitor_scroll();
			}
		}
	}



/****************************************************************************
 * video_draw_c64_screen ****************************************************
 ****************************************************************************/
void	video_draw_c64_screen(
	void)

	{
	if (video_state & VIDEO_INITIALIZED)
		video_functions[video_functions_selected].draw_c64_screen();
	}



/****************************************************************************
 * video_draw_status ********************************************************
 ****************************************************************************/
void	video_draw_status(
	const tc64stat			*c64_status,
	const tc1541stat		*c1541_status,
	const d				input_mode,
	const d				frame_time)

	{
	static tc64stat			c64_status_last;
	static tc1541stat		c1541_status_last;
	static d			input_mode_last;
	static d			frame_time_last;
	static b			status_visible = 0;

	if ((video_state & VIDEO_INITIALIZED) && (video_state & VIDEO_STATUS))
		{
		if (! (status_visible))
			{
			memory_fill(&c64_status_last, sizeof(c64_status_last), 0xff);
			memory_fill(&c1541_status_last, sizeof(c1541_status_last), 0xff);
			input_mode_last = ~ 0;
			frame_time_last = ~ 0;
			status_visible = 1;
			}
		if (c1541_status_last.track != c1541_status->track)
			{
			b		string[40];
			b		track = (c1541_status->track >> 1) + 1;
			b		half_track = 5 * (c1541_status->track & 1);
			b		space[2] = { 0, 0 };

			if (track < 10) space[0] = ' ';
			string_print(string, 39, STRING_TYPE STRING_STR STRING_DEC32 "." STRING_DEC32, space, track, half_track);
			video_functions[video_functions_selected].draw_text(
				(video_status_x_offset + 16),
				video_status_y_offset,
				string);
			c1541_status_last.track = c1541_status->track;
			}
		if (c1541_status_last.led != c1541_status->led)
			{
			video_functions[video_functions_selected].draw_led(
				video_status_x_offset,
				video_status_y_offset,
				c1541_status->led);
			c1541_status_last.led = c1541_status->led;
			}
		if (c1541_status_last.motor != c1541_status->motor)
			{
			video_functions[video_functions_selected].draw_led(
				video_status_x_offset,
				(video_status_y_offset + 4),
				c1541_status->motor);
			c1541_status_last.motor = c1541_status->motor;
			}
		if (input_mode != input_mode_last)
			{
			static const b	keyboard[]  = "Keyboard  ";
			static const b	joystick1[] = "Joystick 1";
			static const b	joystick2[] = "Joystick 2";
			static const b	monitor[]   = "Monitor   ";
			const b		*string = monitor;

			if (input_mode == INPUT_MODE_C64_KEYBOARD) string = keyboard;
			if (input_mode == INPUT_MODE_C64_JOYSTICK1) string = joystick1;
			if (input_mode == INPUT_MODE_C64_JOYSTICK2) string = joystick2;
			video_functions[video_functions_selected].draw_text(
				(video_status_x_offset + 56),
				video_status_y_offset,
				string);
			input_mode_last = input_mode;
			}
		if (frame_time != frame_time_last)
			{
			b		string[40];
			b		space[5] = { 0, 0, 0, 0, 0 };

			if (frame_time < 100000)
				{
				if (frame_time < 10000) space[0] = ' ';
				if (frame_time < 1000) space[1] = ' ';
				if (frame_time < 100) space[2] = ' ';
				if (frame_time < 10) space[3] = ' ';
				string_print(string, 39, STRING_TYPE STRING_STR STRING_DEC32 "us", space, frame_time);
				}
			else string_print(string, 39, STRING_TYPE "99999us");
			video_functions[video_functions_selected].draw_text(
				(video_status_x_offset + 144),
				video_status_y_offset,
				string);
			}
		}
	if ((video_state & VIDEO_INITIALIZED) && (! (video_state & VIDEO_STATUS)))
		status_visible = 0;
	}



/****************************************************************************
 * video_status_on **********************************************************
 ****************************************************************************/
void	video_status_on(
	tvideo				*video_options)

	{
	if (! (video_state & VIDEO_STATUS))
		{
		tvidinit		options;

		video_options->c64_y_offset += VIDEO_STATUS_Y_SIZE;
		video_c64_y_offset += VIDEO_STATUS_Y_SIZE;
		video_options->c64_y_resolution -= VIDEO_STATUS_Y_SIZE;
		options.c64_x_offset = video_c64_x_offset;
		options.c64_y_offset = video_c64_y_offset;
		video_functions[video_functions_selected].set_options(&options);
		video_functions[video_functions_selected].area_clear(
			video_status_x_offset,
			video_status_y_offset,
			(video_status_x_offset + video_options->c64_x_resolution),
			(video_status_y_offset + VIDEO_STATUS_Y_SIZE));
		video_options->buffer = options.c64_screen_buffer;
		video_options->decoded_lines_list = options.c64_screen_lines;
		video_options->virtual_x_resolution = options.virtual_x_resolution;
		video_state |= VIDEO_STATUS;
		}
	}



/****************************************************************************
 * video_status_off *********************************************************
 ****************************************************************************/
void	video_status_off(
	tvideo				*video_options)

	{
	if (video_state & VIDEO_STATUS)
		{
		tvidinit		options;

		video_options->c64_y_offset -= VIDEO_STATUS_Y_SIZE;
		video_c64_y_offset -= VIDEO_STATUS_Y_SIZE;
		video_options->c64_y_resolution += VIDEO_STATUS_Y_SIZE;
		options.c64_x_offset = video_c64_x_offset;
		options.c64_y_offset = video_c64_y_offset;
		video_functions[video_functions_selected].set_options(&options);
		video_options->buffer = options.c64_screen_buffer;
		video_options->decoded_lines_list = options.c64_screen_lines;
		video_options->virtual_x_resolution = options.virtual_x_resolution;
		video_state &= ~ VIDEO_STATUS;
		}
	}



/****************************************************************************
 * video_monitor_on *********************************************************
 ****************************************************************************/
void	video_monitor_on(
	tvideo				*video_options)

	{
	if ((! (video_state & VIDEO_MONITOR_ALWAYS_VISIBLE)) && (! (video_state & VIDEO_MONITOR)))
		{
		d			line;
		tvidinit		options;

		video_options->c64_y_resolution -= VIDEO_MONITOR_Y_SIZE;
		options.c64_x_offset = video_c64_x_offset;
		options.c64_y_offset = video_c64_y_offset;
		video_functions[video_functions_selected].set_options(&options);
		video_functions[video_functions_selected].area_clear(
			video_monitor_x_offset,
			video_monitor_y_offset,
			(video_monitor_x_offset + video_monitor_x_resolution),
			(video_monitor_y_offset + video_monitor_y_resolution));
		for (line = 0; line < VIDEO_MONITOR_LINES; line ++)
			{
			video_functions[video_functions_selected].draw_text(
				video_monitor_x_offset,
				(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE * line),
				(video_monitor_buffer + line * (VIDEO_MONITOR_ROWS + 1)));
			}
		video_options->buffer = options.c64_screen_buffer;
		video_options->decoded_lines_list = options.c64_screen_lines;
		video_options->virtual_x_resolution = options.virtual_x_resolution;
		video_state |= VIDEO_MONITOR;
		}
	}



/****************************************************************************
 * video_monitor_off ********************************************************
 ****************************************************************************/
void	video_monitor_off(
	tvideo				*video_options)

	{
	if ((! (video_state & VIDEO_MONITOR_ALWAYS_VISIBLE)) && (video_state & VIDEO_MONITOR))
		{
		tvidinit		options;

		video_options->c64_y_resolution += VIDEO_MONITOR_Y_SIZE;
		options.c64_x_offset = video_c64_x_offset;
		options.c64_y_offset = video_c64_y_offset;
		video_functions[video_functions_selected].set_options(&options);
		video_functions[video_functions_selected].area_clear(
			video_monitor_x_offset,
			video_monitor_y_offset,
			(video_monitor_x_offset + video_monitor_x_resolution),
			(video_monitor_y_offset + video_monitor_y_resolution));
		video_options->buffer = options.c64_screen_buffer;
		video_options->decoded_lines_list = options.c64_screen_lines;
		video_options->virtual_x_resolution = options.virtual_x_resolution;
		video_state &= ~ VIDEO_MONITOR;
		}
	}



/****************************************************************************
 * video_monitor_new_line ***************************************************
 ****************************************************************************/
void	video_monitor_new_line(
	void)

	{
	if (video_state & VIDEO_INITIALIZED)
		{
		video_monitor_cursor_off();
		if (! (video_state & VIDEO_MONITOR_ALWAYS_VISIBLE))
			video_monitor_buffer[video_monitor_y_cursor * (VIDEO_MONITOR_ROWS + 1) + video_monitor_x_cursor] = 0;
		video_monitor_x_cursor = 0;
		video_monitor_y_cursor ++;
		video_monitor_scroll();
		}
	}



/****************************************************************************
 * video_monitor_delete_character *******************************************
 ****************************************************************************/
void	video_monitor_delete_character(
	void)

	{
	if (video_state & VIDEO_INITIALIZED)
		{
		video_monitor_cursor_off();
		if (video_monitor_x_cursor > 0) video_monitor_x_cursor --;
		else
			{
			if (video_monitor_y_cursor > 0)
				{
				video_monitor_x_cursor = video_monitor_x_characters - 1;
				video_monitor_y_cursor --;
				}
			else return;
			}
		if (! (video_state & VIDEO_MONITOR_ALWAYS_VISIBLE))
			video_monitor_buffer[video_monitor_y_cursor * (VIDEO_MONITOR_ROWS + 1) + video_monitor_x_cursor] = 0;
		if (video_state & VIDEO_MONITOR)
			video_functions[video_functions_selected].area_clear(
				(video_monitor_x_offset + DRAW_X_CHARACTER_SIZE * video_monitor_x_cursor),
				(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE * video_monitor_y_cursor),
				(video_monitor_x_offset + DRAW_X_CHARACTER_SIZE * (video_monitor_x_cursor + 1)),
				(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE * (video_monitor_y_cursor + 1)));
		}
	}



/****************************************************************************
 * video_monitor_cursor_on **************************************************
 ****************************************************************************/
void	video_monitor_cursor_on(
	void)

	{
	if ((video_state & VIDEO_INITIALIZED) && (video_state & VIDEO_MONITOR))
		{
		video_functions[video_functions_selected].draw_text(
			(video_monitor_x_offset + DRAW_X_CHARACTER_SIZE * video_monitor_x_cursor),
			(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE * video_monitor_y_cursor),
			STRING_TYPE "_");
		}
	}



/****************************************************************************
 * video_monitor_cursor_off *************************************************
 ****************************************************************************/
void	video_monitor_cursor_off(
	void)

	{
	if ((video_state & VIDEO_INITIALIZED) && (video_state & VIDEO_MONITOR))
		{
		video_functions[video_functions_selected].area_clear(
			(video_monitor_x_offset + DRAW_X_CHARACTER_SIZE * video_monitor_x_cursor),
			(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE * video_monitor_y_cursor),
			(video_monitor_x_offset + DRAW_X_CHARACTER_SIZE * (video_monitor_x_cursor + 1)),
			(video_monitor_y_offset + DRAW_Y_CHARACTER_SIZE * (video_monitor_y_cursor + 1)));
		}
	}



#ifdef VIDEO_ENABLE_SCREENSHOT_SUPPORT
/****************************************************************************
 * video_make_screenshot ****************************************************
 ****************************************************************************/
#define SET_WORD(x, y, z)		\
	{				\
	(x)[y] = (z) & 0xff;		\
	(x)[y + 1] = ((z) >> 8) & 0xff;	\
	}
#define SET_DWORD(x, y, z)			\
	{					\
	(x)[y] = (z) & 0xff;			\
	(x)[y + 1] = ((z) >> 8) & 0xff;		\
	(x)[y + 2] = ((z) >> 16) & 0xff;	\
	(x)[y + 3] = ((z) >> 24) & 0xff;	\
	}


void	video_make_screenshot(
	void)

	{
	b				screenshot_name[64];
	static d			screenshots = 0;
	sd				screenshot_handle;

	string_print(screenshot_name, 1023, STRING_TYPE "ec64screen-" STRING_HEX32 ".bmp", screenshots);
	if ((screenshot_handle = ec64sys_open(screenshot_name, (O_WRONLY | O_TRUNC | O_CREAT), (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH))) > 0)
		{
		b			buffer[VIDEO_SCREENSHOT_BUFFER_SIZE];
		d			header_size = 0x0436;
		d			line_size = 4 * ((video_x_resolution + 3) / 4);
		d			bitmap_size = line_size * video_y_resolution;
		d			counter;

		/* write the bitmap header */

		memory_fill(buffer, header_size, 0);
		buffer[0] = 'B';
		buffer[1] = 'M';
		SET_DWORD(buffer, 0x02, bitmap_size + header_size);
		SET_DWORD(buffer, 0x0a, header_size);
		SET_DWORD(buffer, 0x0e, 0x28);
		SET_DWORD(buffer, 0x12, video_x_resolution);
		SET_DWORD(buffer, 0x16, video_y_resolution);
		SET_WORD(buffer, 0x1a, 1);
		SET_WORD(buffer, 0x1c, 8);
		SET_DWORD(buffer, 0x22, bitmap_size);
		for (counter = 0; counter < 16; counter ++)
			{
			buffer[0x36 + 4 * counter] = video_c64_colors[counter].blue;
			buffer[0x37 + 4 * counter] = video_c64_colors[counter].green;
			buffer[0x38 + 4 * counter] = video_c64_colors[counter].red;
			}
		ec64sys_write(screenshot_handle, buffer, header_size);

		/* write the bitmap data */

		memory_fill(buffer, line_size, 0);
		for (counter = 0; counter < video_y_resolution; counter ++)
			{
			video_functions[video_functions_selected].get_line(buffer, (video_y_resolution - 1 - counter), video_x_resolution);
			ec64sys_write(screenshot_handle, buffer, line_size);
			}

		/* close the file */

		ec64sys_close(screenshot_handle);
		screenshots ++;
		}
	}
#endif
/******************************************************** ec64@unusedino.de */

