/****************************************************************************
 ****************************************************************************
 *
 * video.h
 *
 ****************************************************************************
 ****************************************************************************/





#ifndef VIDEO_H
#define VIDEO_H

#include "defines.h"
#include "ec64types.h"
#include "sys/fb.h" /* for tcmap */
#include "io/video.h"
#include "io/status.h"

#define VIDEO_SCREENSHOT_BUFFER_SIZE	0x4000

typedef struct
	{
	tcmap				*colormap;
	d				x_resolution;
	d				y_resolution;
	d				virtual_x_resolution;
	d				flags;
	d				c64_x_resolution;
	d				c64_y_resolution;
	d				c64_x_offset;
	d				c64_y_offset;
	b				*c64_screen_buffer;
	d				*c64_screen_lines;
	}
tvidinit;

typedef struct
	{
	const sd			(*initialize)(tvidinit *);
	void				(*restore)(void);
	void				(*set_options)(tvidinit *);
	void				(*draw_text)(const d, const d, const b *);
	void				(*draw_led)(const d, const d, const b);
	void				(*draw_c64_screen)(void);
	void				(*area_clear)(const d, const d, const d, const d);
	void				(*area_move)(const d, const d, const d, const d, const d, const d);
#ifdef VIDEO_ENABLE_SCREENSHOT_SUPPORT
	void				(*get_line)(b *, const d, const d);
#endif
	}
tvidfunc;

extern void				video_initialize(tvideo *);
extern void				video_restore(void);
extern void				video_draw_text(const b *);
extern void				video_draw_c64_screen(void);
extern void				video_draw_status(const tc64stat *, const tc1541stat *, const d, const d);
extern void				video_status_on(tvideo *);
extern void				video_status_off(tvideo *);
extern void				video_monitor_on(tvideo *);
extern void				video_monitor_off(tvideo *);
extern void				video_monitor_new_line(void);
extern void				video_monitor_delete_character(void);
extern void				video_monitor_cursor_on(void);
extern void				video_monitor_cursor_off(void);
#ifdef VIDEO_ENABLE_SCREENSHOT_SUPPORT
extern void				video_make_screenshot(void);
#endif

/*
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
TODO: find a better .h file, these defines are a little bit misplaced here
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/

#define INPUT_MODE_C64_KEYBOARD		0x00
#define INPUT_MODE_C64_JOYSTICK1	0x01
#define INPUT_MODE_C64_JOYSTICK2	0x02
#define INPUT_MODE_MONITOR		0x03



#endif
/******************************************************** ec64@unusedino.de */

