/****************************************************************************
 ****************************************************************************
 *
 * convert.c
 *
 ****************************************************************************
 ****************************************************************************/





#include "convert.h"
#include "ec64io.h"
#include "core/core.h"





struct
	{
	sd				error_code;
	b				*message;
	} 				error_messages[] =
					{
					  { -CORE_ERROR_EXCEPTION,	STRING_TYPE "exception" },
					  { -CORE_ERROR_ASSERTION,	STRING_TYPE "assertion" },
					  { -CORE_ERROR_TRAP,		STRING_TYPE "trap" },
					  { -CORE_ERROR_BPLOG_FULL,	STRING_TYPE "breakpoint log full" },
					  { -CORE_ERROR_NOT_IMPLEMENTED,	STRING_TYPE "function not implemented" },
					  { -CORE_ERROR_BAD_VALUE,	STRING_TYPE "bad value" },
					  { -CORE_ERROR_BAD_BPTYPE,	STRING_TYPE "bad breakpoint type" },
					  { -CORE_ERROR_BPTABLE_FULL,	STRING_TYPE "breakpoint table full" },
					  { -CORE_ERROR_BAD_MODULID,	STRING_TYPE "bad modulid" },
					  { 0, 				STRING_TYPE "unknown error" }
					};
struct
	{
	d				modulid;
	b				*message;
	} 				modulid_messages[] =
					{
					  { CORE_MODULID_CORE,		STRING_TYPE "core" },
					  { CORE_MODULID_LINK,		STRING_TYPE "link" },
					  { CORE_MODULID_C64LINK,	STRING_TYPE "c64link" },
					  { CORE_MODULID_C64MEMORY,	STRING_TYPE "c64memory" },
					  { CORE_MODULID_C64CPU,	STRING_TYPE "c64cpu" },
					  { CORE_MODULID_C64VIC,	STRING_TYPE "c64vic" },
					  { CORE_MODULID_C64SID,	STRING_TYPE "c64sid" },
					  { CORE_MODULID_C64CIA1,	STRING_TYPE "c64cia1" },
					  { CORE_MODULID_C64CIA2,	STRING_TYPE "c64cia2" },
					  { CORE_MODULID_C1541LINK,	STRING_TYPE "c1541link" },
					  { CORE_MODULID_C1541MEMORY,	STRING_TYPE "c1541memory" },
					  { CORE_MODULID_C1541CPU,	STRING_TYPE "c1541cpu" },
					  { CORE_MODULID_C1541VIA1,	STRING_TYPE "c1541via1" },
					  { CORE_MODULID_C1541VIA2,	STRING_TYPE "c1541via2" },
					  { CORE_MODULID_C1541DISC,	STRING_TYPE "c1541disc" },
					  { 0,				STRING_TYPE "unknown modulid" }
					};
struct
	{
	d				bptype;
	b				*short_message;
	b				*long_message;
	}				bptype_messages[] =
					{
					  { CORE_BPTYPE_CPU_READ_OPCODE,	STRING_TYPE "ro", STRING_TYPE "read_opcode" },
					  { CORE_BPTYPE_CPU_READ_STACK,		STRING_TYPE "rs", STRING_TYPE "read_stack" },
					  { CORE_BPTYPE_CPU_READ_DATA,		STRING_TYPE "rd", STRING_TYPE "read_data" },
					  { CORE_BPTYPE_CPU_WRITE_STACK,	STRING_TYPE "ws", STRING_TYPE "write_stack" },
					  { CORE_BPTYPE_CPU_WRITE_DATA,		STRING_TYPE "wd", STRING_TYPE "write_data" },
					  { 0, 					STRING_TYPE "unknown bptype", STRING_TYPE "unknown bptype" }
					};





/****************************************************************************
 * convert_core_error_to_string *********************************************
 ****************************************************************************/
const b	*convert_core_error_to_string(
	const sd			error_code)

	{
	d				offset = 0;

	while ((error_messages[offset].error_code != 0) &&
		(error_messages[offset].error_code != error_code))
		offset ++;
	return (error_messages[offset].message);
	}



/****************************************************************************
 * convert_core_modulid_to_string *******************************************
 ****************************************************************************/
const b	*convert_core_modulid_to_string(
	const d				modulid)

	{
	d				offset = 0;

	while ((modulid_messages[offset].modulid != 0) &&
		(modulid_messages[offset].modulid != modulid))
		offset ++;
	return (modulid_messages[offset].message);
	}



/****************************************************************************
 * convert_core_bptype_to_string ********************************************
 ****************************************************************************/
const b	*convert_core_bptype_to_string(
	const d				bptype)

	{
	d				offset = 0;

	while ((bptype_messages[offset].bptype != 0) &&
		(bptype_messages[offset].bptype != bptype))
		offset ++;
	return (bptype_messages[offset].long_message);
	}



/****************************************************************************
 * convert_string_to_core_modulid *******************************************
 ****************************************************************************/
d	convert_string_to_core_modulid(
	const b				*message)

	{
	d				offset = 0;

	while ((modulid_messages[offset].modulid != 0) &&
		(ec64io_string_compare(modulid_messages[offset].message, message) != 0))
		offset ++;
	return (modulid_messages[offset].modulid);
	}



/****************************************************************************
 * convert_string_to_core_bptype ********************************************
 ****************************************************************************/
d	convert_string_to_core_bptype(
	const b				*message)

	{
	d				offset = 0;

	while ((bptype_messages[offset].bptype != 0) && 
		(ec64io_string_compare(bptype_messages[offset].short_message, message) != 0) &&
		(ec64io_string_compare(bptype_messages[offset].long_message, message) != 0))
		offset ++;
	return (bptype_messages[offset].bptype);
	}
/******************************************************** ec64@unusedino.de */

