/****************************************************************************
 ****************************************************************************
 *
 * gcr.c
 *
 ****************************************************************************
 ****************************************************************************/





/*
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
TODO: gcr decoding works at the moment only with byte aligned input data
      it should also work with SYNCs that end non byte aligned
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/

#include "gcr.h"
#include "ec64core.h"



static b				sectors_per_track[] =
					{
					21, 21, 21, 21, 21, 21, 21, 21,
					21, 21, 21, 21, 21, 21, 21, 21,
					21, 19, 19, 19, 19, 19, 19, 19,
					18, 18, 18, 18, 18, 18, 17, 17,
					17, 17, 17, 0
					};



/****************************************************************************
 * gcr_encode_cbm_bytes *****************************************************
 ****************************************************************************/
sd	gcr_encode_cbm_bytes(
	b				*gcr_bytes,
	const b				*data_bytes)

	{
	static b			gcr_codes[16] =
					{
					0x0a, 0x0b, 0x12, 0x13,
					0x0e, 0x0f, 0x16, 0x17,
					0x09, 0x19, 0x1a, 0x1b,
					0x0d, 0x1d, 0x1e, 0x15
					};
	d				gcr_bits;

	gcr_bits = gcr_codes[data_bytes[0] >> 4];
	gcr_bits <<= 5;
	gcr_bits |= gcr_codes[data_bytes[0] & 0x0f];
	gcr_bytes[0] = (gcr_bits >> 2) & 0xff;

	gcr_bits <<= 5;
	gcr_bits |= gcr_codes[data_bytes[1] >> 4];
	gcr_bits <<= 5;
	gcr_bits |= gcr_codes[data_bytes[1] & 0x0f];
	gcr_bytes[1] = (gcr_bits >> 4) & 0xff;

	gcr_bits <<= 5;
	gcr_bits |= gcr_codes[data_bytes[2] >> 4];
	gcr_bits <<= 5;
	gcr_bits |= gcr_codes[data_bytes[2] & 0x0f];
	gcr_bytes[2] = (gcr_bits >> 6) & 0xff;

	gcr_bits <<= 5;
	gcr_bits |= gcr_codes[data_bytes[3] >> 4];
	gcr_bits <<= 5;
	gcr_bits |= gcr_codes[data_bytes[3] & 0x0f];
	gcr_bytes[3] = (gcr_bits >> 8) & 0xff;
	gcr_bytes[4] = gcr_bits & 0xff;

	return (0);
	}



/****************************************************************************
 * gcr_decode_cbm_bytes *****************************************************
 ****************************************************************************/
sd	gcr_decode_cbm_bytes(
	const b				*gcr_bytes,
	b				*data_bytes)

	{
	static b			cbm_codes[32] =
					{
					0xff, 0xff, 0xff, 0xff,
					0xff, 0xff, 0xff, 0xff,
					0xff, 0x08, 0x00, 0x01,
					0xff, 0x0c, 0x04, 0x05,
					0xff, 0xff, 0x02, 0x03,
					0xff, 0x0f, 0x06, 0x07,
					0xff, 0x09, 0x0a, 0x0b,
					0xff, 0x0d, 0x0e, 0xff
					};
	d				counter;
	b				nibbles[8];

	nibbles[0] = cbm_codes[gcr_bytes[0] >> 3];
	nibbles[1] = cbm_codes[((gcr_bytes[0] << 2) | (gcr_bytes[1] >> 6)) & 0x1f];

	nibbles[2] = cbm_codes[(gcr_bytes[1] >> 1) & 0x1f];
	nibbles[3] = cbm_codes[((gcr_bytes[1] << 4) | (gcr_bytes[2] >> 4)) & 0x1f];

	nibbles[4] = cbm_codes[((gcr_bytes[2] << 1) | (gcr_bytes[3] >> 7)) & 0x1f];
	nibbles[5] = cbm_codes[(gcr_bytes[3] >> 2) & 0x1f];

	nibbles[6] = cbm_codes[((gcr_bytes[3] << 3) | (gcr_bytes[4] >> 5)) & 0x1f];
	nibbles[7] = cbm_codes[gcr_bytes[4] & 0x1f];

	for (counter = 0; counter < 8; counter ++)
		if (nibbles[counter] == 0xff) return (-1);

	data_bytes[0] = (nibbles[0] << 4) | nibbles[1];
	data_bytes[1] = (nibbles[2] << 4) | nibbles[3];
	data_bytes[2] = (nibbles[4] << 4) | nibbles[5];
	data_bytes[3] = (nibbles[6] << 4) | nibbles[7];

	return (0);
	}



/****************************************************************************
 * gcr_encode_cbm_sector ****************************************************
 ****************************************************************************/
sd	gcr_encode_cbm_sector(
	b				*gcr_image,
	const b				*d64_image,
	const d				track,
	const d				sector)

	{
	b				bytes[4];
	d				counter;
	b				block_checksum = 0;

	/* Header: 5 * 0xff */

	gcr_image[0] = 0xff;
	gcr_image[1] = 0xff;
	gcr_image[2] = 0xff;
	gcr_image[3] = 0xff;
	gcr_image[4] = 0xff;
	gcr_image += 5;

	/* Header: 0x08, Checksum, Sector, Track */

	bytes[0] = 0x08;
	bytes[1] = track ^ sector;
	bytes[2] = sector;
	bytes[3] = track;
	gcr_encode_cbm_bytes(gcr_image, bytes);
	gcr_image += 5;

	/* Header: 0x30, 0x30, 0x0f, 0x0f */

	bytes[0] = 0x30;
	bytes[1] = 0x30;
	bytes[2] = 0x0f;
	bytes[3] = 0x0f;
	gcr_encode_cbm_bytes(gcr_image, bytes);
	gcr_image += 5;

	/* Header: 8 * 0x55 */

	for (counter = 0; counter < 8; counter ++) (*(gcr_image ++)) = 0x55;

	/* Block: 5 * 0xff*/

	for (counter = 0; counter < 5; counter ++) (*(gcr_image ++)) = 0xff;

	/* Block: calculate Checksum */

	for (counter = 0; counter < 256; counter ++) block_checksum ^= d64_image[counter];

	/* Block: 0x07, 3 * Databyte */

	bytes[0] = 0x07;
	bytes[1] = d64_image[0];
	bytes[2] = d64_image[1];
	bytes[3] = d64_image[2];
	gcr_encode_cbm_bytes(gcr_image, bytes);
	gcr_image += 5;

	/* Block: 252 * Databyte */

	for (counter = 0; counter < 252; counter += 4)
		{
		bytes[0] = d64_image[counter + 3];
		bytes[1] = d64_image[counter + 4];
		bytes[2] = d64_image[counter + 5];
		bytes[3] = d64_image[counter + 6];
		gcr_encode_cbm_bytes(gcr_image, bytes);
		gcr_image += 5;
		}

	/* Block: 1 * Databyte, Checksum, $00, $00 */

	bytes[0] = d64_image[255];
	bytes[1] = block_checksum;
	bytes[2] = 0;
	bytes[3] = 0;
	gcr_encode_cbm_bytes(gcr_image, bytes);
	gcr_image += 5;

	/* Block: 8 * 0x55 */

	for (counter = 0; counter < 8; counter ++) (*(gcr_image ++)) = 0x55;

	return (0);
	}



/****************************************************************************
 * gcr_decode_cbm_sector ****************************************************
 ****************************************************************************/
#define GET_GCR_BYTES							\
	for (gcr_counter = 0; gcr_counter < 5; gcr_counter ++)		\
		{							\
		gcr_bytes[gcr_counter] = gcr_image[track_offset];	\
		if (track_offset >= bytes_per_track) track_offset = 0;	\
		else track_offset ++;					\
		}							\
	if (gcr_decode_cbm_bytes(gcr_bytes, bytes) == -1) return (-1);

sd	gcr_decode_cbm_sector(
	const b				*gcr_image,
	b				*d64_image,
	const d				bytes_per_track,
	d				track_offset)

	{
	d				gcr_counter;
	d				counter;
	b				gcr_bytes[5];
	b				bytes[4];
	b				block_checksum = 0;

	/* extract the first 3 data bytes */

	GET_GCR_BYTES
	if (bytes[0] != 0x07) return (-1);
	d64_image[0] = bytes[1];
	d64_image[1] = bytes[2];
	d64_image[2] = bytes[3];

	/* extract 252 data bytes */

	for (counter = 0; counter < 252; counter += 4)
		{
		GET_GCR_BYTES
		d64_image[3 + counter] = bytes[0];
		d64_image[4 + counter] = bytes[1];
		d64_image[5 + counter] = bytes[2];
		d64_image[6 + counter] = bytes[3];
		}

	/* extract the last data byte and compare the checksums */

	GET_GCR_BYTES
	d64_image[255] = bytes[0];
	for (counter = 0; counter < 256; counter ++) block_checksum ^= d64_image[counter];
	if (block_checksum != bytes[1]) return (-1);

	return (track_offset);
	}



/****************************************************************************
 * gcr_find_sync ************************************************************
 ****************************************************************************/
sd	gcr_find_sync(
	const b				*gcr_image,
	const d				bytes_per_track,
	d				track_offset)

	{
	d				sync_counter = 0;
	d				counter = 0;

	while (counter < bytes_per_track)
		{
		if (gcr_image[track_offset] != 0xff)
			{
			if (sync_counter > 1) return (track_offset);
			sync_counter = 0;
			}
		else sync_counter ++;
		if (track_offset >= bytes_per_track) track_offset = 0;
		else track_offset ++;
		counter ++;
		}
	return (-1);
	}



/****************************************************************************
 * gcr_encode_d64 ***********************************************************
 ****************************************************************************/
sd	gcr_encode_d64(
	b				*gcr_image,
	const b				*d64_image)

	{
	d				track = 0;
	d				sector;
	d				sectors;
	d				gcr_offset = DISC_GCR_TRACK_SIZE;
	d				gcr_offset_last;
	d				d64_offset = 0;

	/*
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	TODO: develop a more advanced gcr format also used in the core and
	      here. At the moment for every halftrack are 0x2000 bytes
	      reserved
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/

	while ((sectors = sectors_per_track[track]) != 0)
		{
		gcr_offset_last = gcr_offset;
		for (sector = 0; sector < sectors; sector ++)
			{
			gcr_encode_cbm_sector(
				(gcr_image + gcr_offset),
				(d64_image + d64_offset),
				(track + 1),
				sector);
			gcr_offset += 0x168;
			d64_offset += 0x100;
			}
		gcr_offset = gcr_offset_last + (2 * DISC_GCR_TRACK_SIZE);
		track ++;
		}
	return (0);
	}



/****************************************************************************
 * gcr_decode_d64 ***********************************************************
 ****************************************************************************/
sd	gcr_decode_d64(
	const b				*gcr_image,
	b				*d64_image)

	{
	d				track = 0;
	d				sectors;
	d				sector;
	d				bytes_per_track;
	sd				track_offset = 0;
	d				gcr_offset = DISC_GCR_TRACK_SIZE;
	d				d64_offset = 0;
	d				try;
	d				header_offset;
	b				bytes[4];
	b				header[10];

	while ((sectors = sectors_per_track[track]) != 0)
		{

		/*
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		TODO: find another more "clean" way to pass the length
		      of the gcr tracks
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		*/

		switch (sectors)
			{
			case 17:
				bytes_per_track = DISC_GCR_SPEED0_BYTES;
				break;
			case 18:
				bytes_per_track = DISC_GCR_SPEED1_BYTES;
				break;
			case 19:
				bytes_per_track = DISC_GCR_SPEED2_BYTES;
				break;
			default:
				bytes_per_track = DISC_GCR_SPEED3_BYTES;
				break;
			}

		/* extract the sectors */

		for (sector = 0; sector < sectors; sector ++)
			{

			/* generate the header */

			bytes[0] = 0x08;
			bytes[1] = (track + 1) ^ sector;
			bytes[2] = sector;
			bytes[3] = (track + 1);
			gcr_encode_cbm_bytes(header, bytes);
			bytes[0] = 0x30;
			bytes[1] = 0x30;
			bytes[2] = 0x0f;
			bytes[3] = 0x0f;
			gcr_encode_cbm_bytes((header + 5), bytes);

			/* find SYNC and compare with generated header */

			try = 2 * sectors;
			header_offset = 0;
			while (header_offset < sizeof(header))
				{
				if ((track_offset = gcr_find_sync((gcr_image + gcr_offset), bytes_per_track, track_offset)) == -1) return (-1);
				for (header_offset = 0; header_offset < sizeof(header); header_offset ++)
					{
					if (header[header_offset] != (gcr_image + gcr_offset)[track_offset]) break;
					if (track_offset >= bytes_per_track) track_offset = 0;
					else track_offset ++;
					}
				if ((try --) == 0) return (-1);
				}

			/* decode sector */

			if ((track_offset = gcr_find_sync((gcr_image + gcr_offset), bytes_per_track, track_offset)) == -1) return (-1);
			if ((track_offset = gcr_decode_cbm_sector((gcr_image + gcr_offset), (d64_image + d64_offset), bytes_per_track, track_offset)) == -1) return (-1);
			d64_offset += 0x100;
			}
		gcr_offset += (2 * DISC_GCR_TRACK_SIZE);
		track ++;
		}
	return (0);
	}
/******************************************************** ec64@unusedino.de */

