/****************************************************************************
 ****************************************************************************
 *
 * monitor.c
 *
 ****************************************************************************
 ****************************************************************************/





/*
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
TODO:
* 40 track d64 images
* usage of monitor_type_running is more or less a quick hack
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/





#include "monitor.h"
#include "ec64ui.h"
#include "ec64io.h"
#include "ec64core.h"
#include "convert.h"
#include "gcr.h"
#include "ui.h"



#define COMMAND_ENDLIST			0x00
#define COMMAND_NOTHING			0x01
#define COMMAND_EXIT			0x02
#define COMMAND_VERSION			0x03
#define COMMAND_LOAD			0x04
#define COMMAND_CYCLES			0x05
#define COMMAND_HELP			0x06
#define COMMAND_STOP			0x07
#define COMMAND_CONTINUE		0x08
#define COMMAND_RESET			0x09
#define COMMAND_SYNC			0x0a
#define COMMAND_SAVE			0x0b
#define COMMAND_BPLIST			0x0c
#define COMMAND_BPADD			0x0d
#define COMMAND_BPDEL			0x0e
#define COMMAND_SCRIPT			0x0f
#define COMMAND_TYPE			0x10
#define COMMAND_WAIT			0x11

#define PARAMETERS_MAX			32
#define SCRIPT_MAX_SIZE			0x4000
#define TYPE_MAX_SIZE			0x0100

#define MONITOR_INTERACTIVE_PROMPT	"> "
#define MONITOR_SCRIPT_PROMPT		"# "

typedef struct
	{
	d				command;
	d				parameters_minimal;
	d				parameters_maximal;
	b				*command_string;
	b				*parameter_string;
	}
tcommand;


static d				monitor_state = 0;
static const b				unknown_command[] = "unknown command";
static const b				unknown_parameter[] = "unknown parameter";
static const b				unknown_format[] = "unknown format";
static const b				unknown_modulid[] = "unknown modul_id";
static const b				unknown_bptype[] = "unkown bptype";
static const b				unknown_bpflags[] = "unknown bpflags";
static const b				unknown_number[] = "hexadecimal number expected";
static const b				bad_script[] = "bad script, execution halted";
static const tcommand			command_list[] =
					{ { COMMAND_VERSION,	1, 1,	STRING_TYPE "version",	NULL },
					  { COMMAND_LOAD,	3, 3,	STRING_TYPE "load",	STRING_TYPE "<d64|g64> <filename>" },
					  { COMMAND_SAVE,	3, 3,	STRING_TYPE "save",	STRING_TYPE "<d64|g64> <filename>" },
					  { COMMAND_CYCLES,	1, 1,	STRING_TYPE "cycles",	NULL },
					  { COMMAND_STOP,	1, 1,	STRING_TYPE "stop",	NULL },
					  { COMMAND_CONTINUE,	1, 1,	STRING_TYPE "continue",	NULL },
					  { COMMAND_RESET,	2, 2,	STRING_TYPE "reset", 	STRING_TYPE "<soft|hard>" },
					  { COMMAND_SYNC,	2, 2,	STRING_TYPE "sync",	STRING_TYPE "<on|off>" },
					  { COMMAND_SCRIPT,	2, 2,	STRING_TYPE "script",	STRING_TYPE "<filename>" },
					  { COMMAND_TYPE,	2, 2,	STRING_TYPE "type",	STRING_TYPE "<text>" },
					  { COMMAND_WAIT,	2, 2,	STRING_TYPE "wait",	STRING_TYPE "<number of frames>" },
#ifdef MONITOR_ENABLE_BP
					  { COMMAND_BPLIST,	3, 3,	STRING_TYPE "bplist",	STRING_TYPE "<modulid> <bptype>" },
					  { COMMAND_BPADD,	4, 8,	STRING_TYPE "bpadd",	STRING_TYPE "<modulid> <bptype> <bpflags> [ ... ]" },
					  { COMMAND_BPDEL,	4, 4,	STRING_TYPE "bpdel",	STRING_TYPE "<modulid> <bptype> <bpnumber>" },
#endif
					  { COMMAND_EXIT,	1, 1,	STRING_TYPE "exit",	NULL },
					  { COMMAND_EXIT,	1, 1,	STRING_TYPE "quit",	NULL },
					  { COMMAND_HELP,	1, 1,	STRING_TYPE "help",	NULL },
					  { COMMAND_ENDLIST,	0, 0,	NULL,			NULL } };
static b				command_line[MONITOR_COMMAND_LINE_SIZE];
static b				*parameters[PARAMETERS_MAX];
static d				parameter_length = 0;
static d				command_line_length = 0;
static b				script_buffer[SCRIPT_MAX_SIZE];
static sd				script_size;
static d				script_offset = 0;
static b				type_buffer[TYPE_MAX_SIZE];
static d				type_offset = 0;
static d				wait_frames = 0;


#define MONITOR_SCRIPT			0x01
#define MONITOR_TYPE			0x02
#define MONITOR_WAIT			0x04



/****************************************************************************
 * monitor_get_number *******************************************************
 ****************************************************************************/
sd	monitor_get_number(
	const b				*string,
	d				*number)

	{
	d				value = 0;
	d				length = ec64io_string_length(string) - 1;
	d				offset;
	b				character;

	if (string[length] == 'h') length --;
	if (length > 7)
		{
		ec64io_monitor_print_error(unknown_number);
		ec64io_monitor_new_line();
		return (-1);
		}
	for (offset = 0; offset <= length; offset ++)
		{
		character = string[offset];
		value <<= 4;
		if ((character >= '0') && (character <= '9')) value += character - '0';
		else if ((character >= 'a') && (character <= 'f')) value += character + 10 - 'a';
		else if ((character >= 'A') && (character <= 'F')) value += character + 10 - 'A';
		else
			{
			ec64io_monitor_print_error(unknown_number);
			ec64io_monitor_new_line();
			return (-1);
			}
		}
	(*number) = value;
	return (0);
	}


/****************************************************************************
 * monitor_load_d64 *********************************************************
 ****************************************************************************/
void	monitor_load_d64(
	const b				*d64_path)

	{
	b				d64_image[DISC_D64_IMAGE_SIZE + 1];
	b				gcr_image[DISC_GCR_IMAGE_SIZE];
	b				string[1024];
	sd				d64_size;

	ec64io_string_print(string, 1023, STRING_TYPE "loading d64 image: '" STRING_STR "' ... ", d64_path);
	ec64io_monitor_print_notice(string);
	if ((d64_size = ec64io_file_load(d64_path, d64_image, (DISC_D64_IMAGE_SIZE + 1))) == DISC_D64_IMAGE_SIZE)
		{
		ec64io_memory_fill(gcr_image, sizeof(gcr_image), 0x55);
		gcr_encode_d64(gcr_image, d64_image);
		ec64core_c1541_set_disc_image(gcr_image);
		ec64io_monitor_print_notice(STRING_TYPE "ok");
		}
	else
		{
		if (d64_size < 0) ec64io_monitor_print_error(ec64io_get_error_message());
		else ec64io_monitor_print_error(STRING_TYPE "failed");
		}
	ec64io_monitor_new_line();
	}



/****************************************************************************
 * monitor_load_g64 *********************************************************
 ****************************************************************************/
void	monitor_load_g64(
	const b				*g64_path)

	{
	ec64io_monitor_print_notice(STRING_TYPE "load g64: not implemented yet");
	ec64io_monitor_new_line();
	}



/****************************************************************************
 * monitor_save_d64 *********************************************************
 ****************************************************************************/
void	monitor_save_d64(
	const b				*d64_path)

	{
	b				d64_image[DISC_D64_IMAGE_SIZE];
	b				gcr_image[DISC_GCR_IMAGE_SIZE];
	b				string[1024];
	sd				d64_size = 0;

	ec64io_string_print(string, 1023, STRING_TYPE "saving d64 image: '" STRING_STR "' ... ", d64_path);
	ec64io_monitor_print_notice(string);
	ec64io_memory_fill(d64_image, DISC_D64_IMAGE_SIZE, 0);
	ec64io_memory_copy(gcr_image, ec64core_c1541_get_disc_image(), sizeof(gcr_image));
	if ((gcr_decode_d64(gcr_image, d64_image) == 0) &&
		((d64_size = ec64io_file_save(d64_path, d64_image, DISC_D64_IMAGE_SIZE)) == DISC_D64_IMAGE_SIZE))
		{
		ec64io_monitor_print_notice(STRING_TYPE "ok");
		}
	else
		{
		if (d64_size < 0) ec64io_monitor_print_error(ec64io_get_error_message());
		else ec64io_monitor_print_error(STRING_TYPE "failed");
		}
	ec64io_monitor_new_line();
	}



/****************************************************************************
 * monitor_save_g64 *********************************************************
 ****************************************************************************/
void	monitor_save_g64(
	const b				*g64_path)

	{
	ec64io_monitor_print_notice(STRING_TYPE "save g64: not implemented yet");
	ec64io_monitor_new_line();
	}



/****************************************************************************
 * monitor_version **********************************************************
 ****************************************************************************/
#ifndef EC64_BUILD
#define EC64_BUILD			"???"
#endif

void	monitor_version(
	void)

	{
	b				string[1024];

	ec64io_monitor_print_text(STRING_TYPE "You are running ec64-" EC64_VERSION " build " EC64_BUILD);
	ec64io_monitor_new_line();
	ec64io_string_print(string, 1023, STRING_TYPE "ec64io:   " STRING_STR, ec64io_get_version());
	ec64io_monitor_print_text(string);
	ec64io_monitor_new_line();
	ec64io_string_print(string, 1023, STRING_TYPE "ec64ui:   " STRING_STR, ec64ui_get_version());
	ec64io_monitor_print_text(string);
	ec64io_monitor_new_line();
	ec64io_string_print(string, 1023, STRING_TYPE "ec64core: " STRING_STR, ec64core_get_version());
	ec64io_monitor_print_text(string);
	ec64io_monitor_new_line();
	}



/****************************************************************************
 * monitor_cycles ***********************************************************
 ****************************************************************************/
void	monitor_cycles(
	void)

	{
	const tc64stat			*c64_status = ec64core_c64_get_status();
	const tc1541stat		*c1541_status = ec64core_c1541_get_status();
	b				string[1024];

	ec64io_string_print(string, 1023, STRING_TYPE "C64 Cycles:   " STRING_HEX64, c64_status->cycles.high, c64_status->cycles.low);
	ec64io_monitor_print_text(string);
	ec64io_monitor_new_line();
	ec64io_string_print(string, 1023, STRING_TYPE "C1541 Cycles: " STRING_HEX64, c1541_status->cycles.high, c1541_status->cycles.low);
	ec64io_monitor_print_text(string);
	ec64io_monitor_new_line();
	}


/****************************************************************************
 * monitor_script ***********************************************************
 ****************************************************************************/
void	monitor_script(
	const b				*script_path)

	{
	b				string[1024];

	ec64io_string_print(string, 1023, STRING_TYPE "loading script: '" STRING_STR "' ... ", script_path);
	ec64io_monitor_print_notice(string);
	if ((script_size = ec64io_file_load(script_path, script_buffer, SCRIPT_MAX_SIZE)) >= 0)
		{
		monitor_state |= MONITOR_SCRIPT;
		script_offset = 0;
		ec64io_monitor_print_notice(STRING_TYPE "ok");
		}
	else ec64io_monitor_print_error(ec64io_get_error_message());
	ec64io_monitor_new_line();
	}



#ifdef MONITOR_ENABLE_BP
/****************************************************************************
 * monitor_get_modulid ******************************************************
 ****************************************************************************/
d	monitor_get_modulid(
	const b				*string)

	{
	d				modulid = convert_string_to_core_modulid(string);

	if (modulid == 0)
		{
		ec64io_monitor_print_error(unknown_modulid);
		ec64io_monitor_new_line();
		}
	return (modulid);
	}



/****************************************************************************
 * monitor_get_bptype *******************************************************
 ****************************************************************************/
d	monitor_get_bptype(
	const b				*string)

	{
	d				bptype = convert_string_to_core_bptype(string);

	if (bptype == 0)
		{
		ec64io_monitor_print_error(unknown_bptype);
		ec64io_monitor_new_line();
		}
	return (bptype);
	}



/****************************************************************************
 * monitor_bplist ***********************************************************
 ****************************************************************************/
void	monitor_bplist(
	d				modulid,
	d				bptype)

	{
	sd				core_error;
	d				entries = 0;
	d				offset;
	tbpcpu				bpcpu;
	static const b			string_empty[] = "";
	static const b			string_trap[] = "+trap";
	static const b			string_log[] = "+log";
	const b				*flag_trap;
	const b				*flag_log;
	b				string[1024];

	if ((core_error = ec64core_bp_entries(modulid, bptype, &entries)) != 0)
		{
		ec64io_string_print(string, 1023, STRING_TYPE "ec64core: " STRING_STR, convert_core_error_to_string(core_error));
		ec64io_monitor_print_text(string);
		ec64io_monitor_new_line();
		}
	for (offset = 0; offset < entries; offset ++)
		{
		bpcpu.number = offset;
		if ((core_error = ec64core_bp_get(modulid, bptype, &bpcpu)) != 0)
			{
			ec64io_string_print(string, 1023, STRING_TYPE "ec64core: " STRING_STR, convert_core_error_to_string(core_error));
			ec64io_monitor_print_text(string);
			ec64io_monitor_new_line();
			}
		flag_trap = string_empty;
		flag_log = string_empty;
		if (bpcpu.flags & CORE_BPFLAG_TRAP) flag_trap = string_trap;
		if (bpcpu.flags & CORE_BPFLAG_LOG) flag_log = string_log;
		ec64io_string_print(string, 1023, STRING_TYPE STRING_HEX8 ": " STRING_HEX16 " - " STRING_HEX16 "   xx & " STRING_HEX8 " = " STRING_HEX8 "   " STRING_STR STRING_STR, bpcpu.number, bpcpu.low_limit, bpcpu.high_limit, bpcpu.bit_mask, bpcpu.value, flag_trap, flag_log);
		ec64io_monitor_print_text(string);
		ec64io_monitor_new_line();
		}
	}
#endif



/****************************************************************************
 * monitor_help *************************************************************
 ****************************************************************************/
void	monitor_help(
	void)

	{
	register d			offset = 0;
	b				string[1024];

	while (command_list[offset].command != COMMAND_ENDLIST)
		{
		if (command_list[offset].parameter_string)
			{
			ec64io_string_print(string, 1023, STRING_TYPE STRING_STR " " STRING_STR, command_list[offset].command_string, command_list[offset].parameter_string);
			ec64io_monitor_print_text(string);
			}
		else
			{
			ec64io_monitor_print_text(command_list[offset].command_string);
			}
		ec64io_monitor_new_line();
		offset ++;
		}
	}



/****************************************************************************
 * monitor_initialize *******************************************************
 ****************************************************************************/
void	monitor_initialize(
	void)

	{
	monitor_version();
	ec64io_monitor_new_line();
	}



/****************************************************************************
 * monitor_first_prompt *****************************************************
 ****************************************************************************/
void	monitor_first_prompt(
	void)

	{
	monitor_set_character(0x0a);
	}



/****************************************************************************
 * monitor_interactive_prompt ***********************************************
 ****************************************************************************/
void	monitor_interactive_prompt(
	void)

	{
	static const b			prompt[] = MONITOR_INTERACTIVE_PROMPT;

	ec64io_monitor_print_text(prompt);
	}



/****************************************************************************
 * monitor_script_prompt ****************************************************
 ****************************************************************************/
void	monitor_script_prompt(
	void)

	{
	static const b			prompt[] = MONITOR_SCRIPT_PROMPT;

	ec64io_monitor_print_notice(prompt);
	}



/****************************************************************************
 * monitor_get_command ******************************************************
 ****************************************************************************/
d	monitor_get_command(
	b				*command_line)

	{
	parameter_length = 0;

	/* split commandline */

		{
		d			length = ec64io_string_length(command_line);
		d			offset;
		b			space = 1;
		b			seperate = 1;
		b			character;

		for (offset = 0; offset < length; offset ++)
			{
			character = command_line[offset];
			if (space)
				{
				if (character == ' ') continue;
				if (character == '\\')
					{
					parameters[parameter_length ++] = &command_line[offset + 1];
					seperate = 0;
					}
				else parameters[parameter_length ++] = &command_line[offset];
				if (parameter_length >= PARAMETERS_MAX)
					{
					ec64io_monitor_print_error(STRING_TYPE "too many parameters");
					ec64io_monitor_new_line();
					break;
					}
				space = 0;
				continue;
				}
			if ((character == ' ') && (seperate))
				{
				command_line[offset] = 0;
				space = 1;
				}
			}
		}

#ifdef MONITOR_DEBUG_CMDLINE
		{
		d			offset;

		for (offset = 0; offset < parameter_length; offset ++)
			{
			ec64io_monitor_print_text(parameters[offset]);
			ec64io_monitor_new_line();
			}
		return (COMMAND_NOTHING);
		}
#endif

	if (parameter_length == 0) return (COMMAND_NOTHING);

	/* get the command */

		{
		d			command;
		d			offset = 0;

		while ((command = command_list[offset].command) != COMMAND_ENDLIST)
			{
			if (ec64io_string_compare(command_list[offset].command_string, parameters[0]) == 0)
				{
				if (command_list[offset].parameters_minimal > parameter_length)
					{
					b	string[1024];

					ec64io_string_print(string, 1023, STRING_TYPE "command expects at least " STRING_DEC32 " parameters", (command_list[offset].parameters_minimal - 1));
					ec64io_monitor_print_error(string);
					ec64io_monitor_new_line();
					return (COMMAND_NOTHING);
					}
				if (command_list[offset].parameters_maximal < parameter_length)
					{
					b	string[1024];

					ec64io_string_print(string, 1023, STRING_TYPE "command expects not more than " STRING_DEC32 " parameters", (command_list[offset].parameters_maximal - 1));
					ec64io_monitor_print_error(string);
					ec64io_monitor_new_line();
					return (COMMAND_NOTHING);
					}
				return (command);
				}
			offset ++;
			}
		return (COMMAND_ENDLIST);
		}
	}



/****************************************************************************
 * monitor_process_command_line *********************************************
 ****************************************************************************/
void	monitor_process_command_line(
	b				*command_line)

	{
	switch (monitor_get_command(command_line))
		{

		case COMMAND_NOTHING:
			break;

		/* get version */

		case COMMAND_VERSION:
			monitor_version();
			break;

		/* exit */

		case COMMAND_EXIT:
			ec64io_end();
			break;

		/* load */

		case COMMAND_LOAD:
			if (ec64io_string_compare(parameters[1], STRING_TYPE "d64") == 0) monitor_load_d64(parameters[2]);
			else if (ec64io_string_compare(parameters[1], STRING_TYPE "g64") == 0) monitor_load_g64(parameters[2]);
			else
				{
				ec64io_monitor_print_error(unknown_format);
				ec64io_monitor_new_line();
				}
			break;

		/* save */

		case COMMAND_SAVE:
			if (ec64io_string_compare(parameters[1], STRING_TYPE "d64") == 0) monitor_save_d64(parameters[2]);
			else if (ec64io_string_compare(parameters[1], STRING_TYPE "g64") == 0) monitor_save_g64(parameters[2]);
			else
				{
				ec64io_monitor_print_error(unknown_format);
				ec64io_monitor_new_line();
				}
			break;

		/* cycles */

		case COMMAND_CYCLES:
			monitor_cycles();
			break;

		/* stop */

		case COMMAND_STOP:
			ui_core_stop();
			break;

		/* continue */

		case COMMAND_CONTINUE:
			ui_core_start();
			break;

		/* reset */

		case COMMAND_RESET:
			if (ec64io_string_compare(parameters[1], STRING_TYPE "hard") == 0)
				ec64core_reset_hard();
			else if (ec64io_string_compare(parameters[1], STRING_TYPE "soft") == 0)
				ec64core_reset_soft();
			else
				{
				ec64io_monitor_print_error(unknown_parameter);
				ec64io_monitor_new_line();
				}
			break;

		/* sync */

		case COMMAND_SYNC:
			if (ec64io_string_compare(parameters[1], STRING_TYPE "on") == 0)
				ui_sync_on();
			else if (ec64io_string_compare(parameters[1], STRING_TYPE "off") == 0)
				ui_sync_off();
			else
				{
				ec64io_monitor_print_error(unknown_parameter);
				ec64io_monitor_new_line();
				}
			break;

		/* script */

		case COMMAND_SCRIPT:
			monitor_script(parameters[1]);
			break;

		/* type */

		case COMMAND_TYPE:
				{
				d	length = ec64io_string_length(parameters[1]);

				if (length > TYPE_MAX_SIZE - 1) length = TYPE_MAX_SIZE;
				ec64io_memory_copy(type_buffer, parameters[1], length);
				type_buffer[length] = 0;
				type_offset = 0;
				monitor_state |= MONITOR_TYPE;
				}
			break;

		/* wait */

		case COMMAND_WAIT:
			if (monitor_get_number(parameters[1], &wait_frames) == 0)
				monitor_state |= MONITOR_WAIT;
			break;

#ifdef MONITOR_ENABLE_BP

		/* bplist */

		case COMMAND_BPLIST:
				{
				d	modulid;
				d	bptype;

				if ((modulid = monitor_get_modulid(parameters[1])) == 0) break;
				if ((bptype = monitor_get_bptype(parameters[2])) == 0) break;
				monitor_bplist(modulid, bptype);
				}
			break;

		/* bpadd */

		case COMMAND_BPADD:
				{
				d	modulid;
				d	bptype;
				d	value;
				tbpcpu	bpcpu;
				
				if ((modulid = monitor_get_modulid(parameters[1])) == 0) break;
				if ((bptype = monitor_get_bptype(parameters[2])) == 0) break;
				ec64io_memory_fill(&bpcpu, sizeof(bpcpu), 0);
				bpcpu.high_limit = 0xffff;
				if (ec64io_string_compare(parameters[3], STRING_TYPE "+trap") == 0) bpcpu.flags = CORE_BPFLAG_TRAP;
				else if (ec64io_string_compare(parameters[3], STRING_TYPE "+log") == 0) bpcpu.flags = CORE_BPFLAG_LOG;
				else if (ec64io_string_compare(parameters[3], STRING_TYPE "+trap+log") == 0) bpcpu.flags = (CORE_BPFLAG_TRAP | CORE_BPFLAG_LOG);
				else
					{
					ec64io_monitor_print_error(unknown_bpflags);
					ec64io_monitor_new_line();
					break;
					}

				/* get optional parameters */

				if (parameter_length > 4)
					{
					if (monitor_get_number(parameters[4], &value) != 0) break;
					bpcpu.low_limit = value;
					}
				if (parameter_length > 5)
					{
					if (monitor_get_number(parameters[5], &value) != 0) break;
					bpcpu.high_limit = value;
					}
				if (parameter_length > 6)
					{
					if (monitor_get_number(parameters[6], &value) != 0) break;
					bpcpu.bit_mask = value;
					}
				if (parameter_length > 7)
					{
					if (monitor_get_number(parameters[7], &value) != 0) break;
					bpcpu.value = value;
					}

				/* finally give the parameters to the core */

					{
					b	string[1024];
					sd	core_error;

					if ((core_error = ec64core_bp_add(modulid, bptype, &bpcpu)) != 0)
						{
						ec64io_string_print(string, 1023, STRING_TYPE "ec64core: " STRING_STR, convert_core_error_to_string(core_error));
						ec64io_monitor_print_text(string);
						ec64io_monitor_new_line();
						}
					else
						{
						ec64io_string_print(string, 1023, STRING_TYPE "bpnumber: " STRING_HEX8, bpcpu.number);
						ec64io_monitor_print_text(string);
						ec64io_monitor_new_line();
						}
					}
				}
			break;

		/* bpdel */

		case COMMAND_BPDEL:
				{
				d	modulid;
				d	bptype;
				d	bpnumber = 0x100;
				sd	core_error;
				
				if ((modulid = monitor_get_modulid(parameters[1])) == 0) break;
				if ((bptype = monitor_get_bptype(parameters[2])) == 0) break;
				if (monitor_get_number(parameters[3], &bpnumber) != 0) break;

				/* finally give the parameters to the core */

				if ((core_error = ec64core_bp_delete(modulid, bptype, bpnumber)) != 0)
					{
					b	string[1024];

					ec64io_string_print(string, 1023, STRING_TYPE "ec64core: " STRING_STR, convert_core_error_to_string(core_error));
					ec64io_monitor_print_text(string);
					ec64io_monitor_new_line();
					}
				}
			break;
#endif

		/* help */

		case COMMAND_HELP:
			monitor_help();
			break;

		/* unknown command */

		default:
			ec64io_monitor_print_error(unknown_command);
			ec64io_monitor_new_line();
			break;
		}
	}



/****************************************************************************
 * monitor_set_character ****************************************************
 ****************************************************************************/
void	monitor_set_character(
	const b				character)

	{

	/* do we accept keyboard input from the monitor at the moment ? */

	if (monitor_state & (MONITOR_SCRIPT | MONITOR_TYPE | MONITOR_WAIT))
		{

		/* Ctrl-C pressed ?, if yes abort all blocking activities */

		/* NOTE: all necessary clean up work for the script, type and
		   wait command must also be done here (at the moment this
		   means setting the command_line_length to 0 and
		   displaying the prompt */

		if (character == 0x03)
			{
			monitor_state &= ~ (MONITOR_SCRIPT | MONITOR_TYPE | MONITOR_WAIT);
			command_line_length = 0;
			monitor_interactive_prompt();
			}
		return;
		}

	/* we accept keyboard input, now process it */

	if (character == 0x0a)
		{
		command_line[command_line_length] = 0;
		ec64io_monitor_new_line();
		if (command_line_length > 0) monitor_process_command_line(command_line);
		command_line_length = 0;
		if (! (monitor_state & (MONITOR_SCRIPT | MONITOR_TYPE | MONITOR_WAIT)))
			monitor_interactive_prompt();
		return;
		}
	if (character == 0x7f)
		{
		if (command_line_length > 0)
			{
			ec64io_monitor_delete_character();
			command_line_length --;
			}
		return;
		}
	if ((character >= 0x20) && (character <= 0x7f) && (command_line_length < MONITOR_COMMAND_LINE_SIZE - 1))
		{
		b			input[2] = { 0, 0 };

		input[0] = character;
		command_line[command_line_length] = character;
		command_line_length ++;
		ec64io_monitor_print_text(input);
		}
#ifdef MONITOR_DEBUG_SHOW_UNKNOWN_CHARACTERS
	else
		{
		b			string[64];

		ec64io_string_print(string, 63, STRING_TYPE "<" STRING_HEX8 ">", character);
		ec64io_monitor_print_text(string);
		}
#endif
	}



/****************************************************************************
 * monitor_process_type *****************************************************
 ****************************************************************************/
void	monitor_process_type(
	void)

	{
	static d			delay = 0;
	static b			pressed_keys[0x42] = { 0 };
	static b			keystroke = 0;
	b				c64_keycodes[16];
	b				character;

	/* hold every keycode 2 frames, clear it in the 3rd frame only if it
	   not belongs to a keystroke (multiples keys pressed at the same
	   time) */

	if (delay < 3)
		{
		if ((! (keystroke)) && (delay > 1))
			{
			c64_keycodes[0] = 0;
			ec64core_c64_set_keyboard(c64_keycodes);
			}
		delay ++;
		return;
		}
	delay = 0;

	/* clear pressed_keys if we don't have a keystroke */

	if (! (keystroke)) ec64io_memory_fill(pressed_keys, sizeof(pressed_keys), 0);

	character = type_buffer[type_offset ++];

	/* if the end of type_buffer is reached, leave MONITOR_TYPE mode */

	if (character == 0)
		{
		monitor_state &= ~ MONITOR_TYPE;
		if (! (monitor_state & MONITOR_SCRIPT)) monitor_interactive_prompt();
		keystroke = 0;
		goto end;
		}

	/* start or end a keystroke */

	if (character == '`')
		{
		keystroke ^= 1;
		goto end;
		}

	/* process normal characters */

	if ((character >= '0') && (character <= '9')) pressed_keys[character - '0' + 0x30] = 1;
	if ((character >= 'a') && (character <= 'z')) pressed_keys[character - 'a' + 1] = 1;
	if ((character >= 'A') && (character <= 'Z'))
		{
		pressed_keys[0x1b] = 1;
		pressed_keys[character - 'A' + 1] = 1;
		}
	switch (character)
		{
		case '!':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x31] = 1;
			break;
		case '"':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x32] = 1;
			break;
		case '#':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x33] = 1;
			break;
		case '$':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x34] = 1;
			break;
		case '%':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x35] = 1;
			break;
		case '&':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x36] = 1;
			break;
		case '\'':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x37] = 1;
			break;
		case '(':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x38] = 1;
			break;
		case ')':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x39] = 1;
			break;
		case '+':
			pressed_keys[0x2b] = 1;
			break;
		case '-':
			pressed_keys[0x2d] = 1;
			break;
		case '@':
			pressed_keys[0x40] = 1;
			break;
		case '*':
			pressed_keys[0x2a] = 1;
			break;
		case ':':
			pressed_keys[0x3a] = 1;
			break;
		case '[':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x3a] = 1;
			break;
		case ';':
			pressed_keys[0x3b] = 1;
			break;
		case ']':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x3b] = 1;
			break;
		case '=':
			pressed_keys[0x3d] = 1;
			break;
		case ',':
			pressed_keys[0x3c] = 1;
			break;
		case '<':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x3c] = 1;
			break;
		case '.':
			pressed_keys[0x3e] = 1;
			break;
		case '>':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x3e] = 1;
			break;
		case '/':
			pressed_keys[0x3f] = 1;
			break;
		case '?':
			pressed_keys[0x1b] = 1;
			pressed_keys[0x3f] = 1;
			break;
		case ' ':
			pressed_keys[0x20] = 1;
			break;
		}

	/* process ^-escaped characters*/

	if (character == '^')
		{
		character = type_buffer[type_offset ++];
		switch (character)
			{
			case 'l': /* SHIFT_LEFT */
				pressed_keys[0x1b] = 1;
				break;
			case 'r': /* SHIFT_RIGHT */
				pressed_keys[0x1d] = 1;
				break;
			case 'c': /* CTRL */
				pressed_keys[0x23] = 1;
				break;
			case 'o': /* C= */
				pressed_keys[0x26] = 1;
				break;
			case '1': /* F1 */
				pressed_keys[0x29] = 1;
				break;
			case '2': /* F2 */
				pressed_keys[0x1b] = 1;
				pressed_keys[0x29] = 1;
				break;
			case '3': /* F3 */
				pressed_keys[0x2c] = 1;
				break;
			case '4': /* F4 */
				pressed_keys[0x1b] = 1;
				pressed_keys[0x2c] = 1;
				break;
			case '5': /* F5 */
				pressed_keys[0x2e] = 1;
				break;
			case '6': /* F6 */
				pressed_keys[0x1b] = 1;
				pressed_keys[0x2e] = 1;
				break;
			case '7': /* F7 */
				pressed_keys[0x2f] = 1;
				break;
			case '8': /* F8 */
				pressed_keys[0x1b] = 1;
				pressed_keys[0x2f] = 1;
				break;
			case 'L': /* <= */
				pressed_keys[0x1f] = 1;
				break;
			case 'E': /* \3 */
				pressed_keys[0x1c] = 1;
				break;
			case 'H': /* CLR/HOME */
				pressed_keys[0x21] = 1;
				break;
			case 'I': /* INST/DEL */
				pressed_keys[0x22] = 1;
				break;
			case 'U': /* ^ */
				pressed_keys[0x1e] = 1;
				break;
			case 'T': /* RESTORE */
				pressed_keys[0x41] = 1;
				break;
			case 'S': /* RUN_STOP */
				pressed_keys[0x24] = 1;
				break;
			case 'N': /* RETURN */
				pressed_keys[0x25] = 1;
				break;
			case 'D': /* CRSR_DOWN */
				pressed_keys[0x27] = 1;
				break;
			case 'R': /* CRSR_RIGHT */
				pressed_keys[0x28] = 1;
				break;
			}
		}

	/* get the currently active keycodes */

end:
		{
		d			keycode;
		d			offset = 0;

		for (keycode = 0; keycode < sizeof(pressed_keys); keycode ++)
			if ((offset < sizeof(c64_keycodes) - 1) && (pressed_keys[keycode])) c64_keycodes[offset ++] = keycode;
		c64_keycodes[offset] = 0;
		}

	/* give the keycodes to the core */

	ec64core_c64_set_keyboard(c64_keycodes);
	}



/****************************************************************************
 * monitor_process_script ***************************************************
 ****************************************************************************/
void	monitor_process_script(
	void)

	{
	b				character;
	b				line_incomplete = 1;

	/* extract one line from the script */

	do
		{
		character = script_buffer[script_offset];
		script_offset ++;
		if (character == 0x0a) goto end;
		if ((character >= 0x20) && (character < 0x7f) && (command_line_length < MONITOR_COMMAND_LINE_SIZE - 1))
			{
			command_line[command_line_length] = character;
			command_line_length ++;
			}
		else break;
		if (script_offset >= script_size)
			{
		end:
			command_line[command_line_length] = 0;
			line_incomplete = 0;
			}
		}
	while (line_incomplete);

	/* check if there is a valid line in command_line[], if not there
	   were bad characters in the script */

	if (line_incomplete)
		{
		ec64io_monitor_print_error(bad_script);
		ec64io_monitor_new_line();
		command_line_length = 0;
		goto leave;
		}

	/* output the extracted comdline to the monitor */

	monitor_script_prompt();
	ec64io_monitor_print_notice(command_line);
	ec64io_monitor_new_line();

	/* process the extracted cmdline */

	monitor_process_command_line(command_line);
	command_line_length = 0;

	/* exit MONITOR_SCRIPT mode if necessary */

	if (script_offset >= script_size)
		{
	leave:
		monitor_state &= ~ MONITOR_SCRIPT;

		/* display only a prompt if no type or wait command
		   is running, because they will display itself a prompt,
		   because MONITOR_SCRIPT is no longer on */

		if (! (monitor_state & (MONITOR_TYPE | MONITOR_WAIT)))
			monitor_interactive_prompt();
		}
	}



/****************************************************************************
 * monitor_new_frame ********************************************************
 ****************************************************************************/
void	monitor_new_frame(
	void)

	{

	/* process the type command */

	if (monitor_state & MONITOR_TYPE)
		{
		monitor_process_type();
		return;
		}

	/* process the wait command */

	if (monitor_state & MONITOR_WAIT)
		{
		if (wait_frames == 0)
			{
			monitor_state &= ~ MONITOR_WAIT;
			if (! (monitor_state & MONITOR_SCRIPT)) monitor_interactive_prompt();
			}
		else wait_frames --;
		return;
		}

	/* process the script command */

	if (monitor_state & MONITOR_SCRIPT) monitor_process_script();
	}



/****************************************************************************
 * monitor_type_running *****************************************************
 ****************************************************************************/
d	monitor_type_running(
	void)

	{
	return ((monitor_state & MONITOR_TYPE) != 0);
	}
/******************************************************** ec64@unusedino.de */

