/****************************************************************************
 ****************************************************************************
 *
 * ui.c
 *
 ****************************************************************************
 ****************************************************************************/





#include "ui.h"
#include "ec64io.h"
#include "ec64core.h"
#include "ec64rom.h"
#include "convert.h"
#include "gcr.h"
#include "monitor.h"



static d				ui_state = 0;
/* VIC6569_LOG_HACK */
static b				log_buffer[0x20];
static d				log_offset = 0;

#define UI_CORE_STOPPED			0x01
#define UI_CORE_REINITIALIZE		0x02
#define UI_SYNC_OFF			0x04



/****************************************************************************
 * ui_core_initialize *******************************************************
 ****************************************************************************/
void	ui_core_initialize(
	void)

	{
	ec64core_initialize();
	ec64core_c64_set_basic_rom(ec64rom_c64_basic);
	ec64core_c64_set_character_rom(ec64rom_c64_character);
	ec64core_c64_set_kernel_rom(ec64rom_c64_kernel);
	ec64core_c1541_set_kernel_rom(ec64rom_c1541_kernel);
	ec64core_c64_set_video_options(ec64io_c64_get_video_options());
	ec64core_c64_set_audio_options(ec64io_c64_get_audio_options());
	ec64core_set_log_options(log_buffer, &log_offset, sizeof(log_buffer));
	ec64core_reset_hard();
	}



/****************************************************************************
 * ui_core_stop *************************************************************
 ****************************************************************************/
void	ui_core_stop(
	void)

	{
	ui_state |= UI_CORE_STOPPED;
	}



/****************************************************************************
 * ui_core_start ************************************************************
 ****************************************************************************/
void	ui_core_start(
	void)

	{
	if (ui_state & UI_CORE_REINITIALIZE)
		{
		ec64io_monitor_print_notice(STRING_TYPE "reinitializing core");
		ec64io_monitor_new_line();
		ui_core_initialize();
		ui_state &= ~ UI_CORE_REINITIALIZE;
		}
	ui_state &= ~ UI_CORE_STOPPED;
	}



/****************************************************************************
 * ui_sync_on ***************************************************************
 ****************************************************************************/
void	ui_sync_on(
	void)

	{
	ui_state &= ~ UI_SYNC_OFF;
	}



/****************************************************************************
 * ui_sync_off **************************************************************
 ****************************************************************************/
void	ui_sync_off(
	void)

	{
	ui_state |= UI_SYNC_OFF;
	}



/****************************************************************************
 * ui_print_reason1 *********************************************************
 ****************************************************************************/
void	ui_print_reason1(
	void)

	{
	tcoreinfo			coreinfo;
	b				string[1024];

	ec64core_get_information(&coreinfo);
	ec64io_string_print(string, 1023, STRING_TYPE "modulid: " STRING_STR, convert_core_modulid_to_string(coreinfo.modulid));
	ec64io_monitor_print_error(string);
	ec64io_monitor_new_line();
	ec64io_string_print(string, 1023, STRING_TYPE "type:    " STRING_HEX32, coreinfo.type);
	ec64io_monitor_print_error(string);
	ec64io_monitor_new_line();
	ec64io_string_print(string, 1023, STRING_TYPE "number:  " STRING_HEX32, coreinfo.number);
	ec64io_monitor_print_error(string);
	ec64io_monitor_new_line();
	}



/****************************************************************************
 * ui_print_reason2 *********************************************************
 ****************************************************************************/
void	ui_print_reason2(
	void)

	{
	tcoreinfo			coreinfo;
	b				string[1024];

	ec64core_get_information(&coreinfo);
	ec64io_string_print(string, 1023, STRING_TYPE "modulid:  " STRING_STR, convert_core_modulid_to_string(coreinfo.modulid));
	ec64io_monitor_print_error(string);
	ec64io_monitor_new_line();
	ec64io_string_print(string, 1023, STRING_TYPE "bptype:   " STRING_STR, convert_core_bptype_to_string(coreinfo.type));
	ec64io_monitor_print_error(string);
	ec64io_monitor_new_line();
	ec64io_string_print(string, 1023, STRING_TYPE "bpnumber: " STRING_HEX8, coreinfo.number);
	ec64io_monitor_print_error(string);
	ec64io_monitor_new_line();
	}



/****************************************************************************
 * ui_start *****************************************************************
 ****************************************************************************/
void	ui_start(
	void)

	{
	sd				core_status;
	d				io_status;
	b				character;

	/* initialization */

	ec64io_initialize();
	monitor_initialize();
	ui_core_initialize();
	if (ec64io_cmdline_d64_path()) monitor_load_d64(ec64io_cmdline_d64_path());
	if (ec64io_cmdline_script_path()) monitor_script(ec64io_cmdline_script_path());
	monitor_first_prompt();

	/* start the emulation */

	while (1)
		{
		if (! (ui_state & UI_CORE_STOPPED))
			{
			do
				{
				core_status = ec64core_emulate_pal_frame();
				switch (core_status)
					{
					case 0:
						ec64io_c64_draw_screen();
						ec64io_c64_play_samples();
						break;
					case -CORE_ERROR_BPLOG_FULL:
						ec64io_log_write(log_buffer, log_offset);
						break;
					case -CORE_ERROR_TRAP:
						ui_core_stop();
						ec64io_monitor_new_line();
						ec64io_monitor_print_error(STRING_TYPE "breakpoint trapped, emulation stopped");
						ec64io_monitor_new_line();
						ui_print_reason2();
						break;
					case -CORE_ERROR_ASSERTION:
						ui_core_stop();
						ui_state |= UI_CORE_REINITIALIZE;
						ec64io_monitor_new_line();
						ec64io_monitor_print_error(STRING_TYPE "core threw an assertion, emulation halted");
						ec64io_monitor_new_line();
						ui_print_reason1();
						break;
					case -CORE_ERROR_EXCEPTION:
						ui_core_stop();
						ui_state |= UI_CORE_REINITIALIZE;
						ec64io_monitor_new_line();
						ec64io_monitor_print_error(STRING_TYPE "core threw an exception, emulation halted");
						ec64io_monitor_new_line();
						ui_print_reason1();
						break;
					default:
						return;
					}
				}
			while (core_status == -CORE_ERROR_BPLOG_FULL);
			}
		ec64io_c1541_draw_status(ec64core_c1541_get_status());
		if ((io_status = ec64io_get_status()) != 0)
			{
			ec64io_set_status(io_status);
			if (io_status & STATUS_RESIZE_VIDEO)
				{
				ec64core_c64_set_video_options(ec64io_c64_get_video_options());
				ec64io_c64_draw_screen();
				}
			}
		if ((character = ec64io_monitor_get_character()) != 0) monitor_set_character(character);
		monitor_new_frame();
		if (! (monitor_type_running()))
			ec64core_c64_set_keyboard(ec64io_c64_get_keyboard());
		ec64core_c64_set_joystick1(ec64io_c64_get_joystick1());
		ec64core_c64_set_joystick2(ec64io_c64_get_joystick2());
		if ((ui_state & UI_CORE_STOPPED) || (! (ui_state & UI_SYNC_OFF)))
			ec64io_sync_sleep();
		}
	}
/******************************************************** ec64@unusedino.de */

